// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_cov.e
//              :
// Project      : csco_addrmap
//              :
// Created On   : 09/26/2005
//              :
//              :
// Description  : This file contains functional coverage definitions for addrmap
//              : entries.
//              :
//              : IMPORTANT: For functionl coverage to be collected, the entries
//              :            must be populated in the scope tree during generation
//              :            and prior to post_generate().
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------


<'
package csco_addrmap;


extend csco_addrmap_entry_s {
  update_shadow(trans : csco_addrmap_trans_s) is also {
    cur_access = trans;
    emit collect_cover;
  };
};

unit csco_addrmap_reg_entry_cov_u {
  !reg : csco_addrmap_reg_entry_s;

  event reg_access is @reg.collect_cover;

#ifdef SPECMAN_VERSION_6_1 { -- can use in 6.0 if you get a patch that must be compiled from Cadence
  cover reg_access using 
                         per_unit_instance,
                         unit_name = reg.long_name(""),
                         when = (reg.cur_access.access == FRONT_DOOR) is {

    item opcode : csco_addrmap_op_t = reg.cur_access.opcode using
      ignore = (opcode == FILL);

    item post_kind : csco_addrmap_post_kind_t = reg.cur_access.post_kind using
      no_collect;

    cross opcode, post_kind using
      name = opcode_x_post_kind,
      ignore = ((opcode == READ) and (post_kind == POSTED));
  };
};

};

unit csco_addrmap_mem_entry_cov_u {
  !mem : csco_addrmap_mem_entry_s;

  event mem_access is @mem.collect_cover;

#ifdef SPECMAN_VERSION_6_1 { -- can use in 6.0 if you get a patch that must be compiled from Cadence
  cover mem_access using 
                         per_unit_instance,
                         unit_name = reg.long_name(""),
                         when = (mem.cur_access.access == FRONT_DOOR) is {

    item opcode : csco_addrmap_op_t = mem.cur_access.opcode;

    item fill_kind : csco_addrmap_fill_kind_t = mem.cur_access.fill_kind using
      no_collect;

    item post_kind : csco_addrmap_post_kind_t = mem.cur_access.post_kind using
      no_collect;

    item num_bytes : uint = mem.cur_access.num_bytes using
      ranges = { range([1]);
                 range([2]);
                 range([3]);
                 range([4]);
                 range([5..8]);
                 range([9..16]);
                 range([17..32]);
                 range([33..64]);
                 range([65..128]);
                 range([129..256]);
                 range([257..512]);
                 range([513..1K]);
                 range([1K..MAX_UINT])
               };

    cross opcode, fill_kind, post_kind using
      name = opcode_x_post_kind,
      ignore = ((opcode == READ) and (post_kind == POSTED));
  };
};
};


extend csco_addrmap_env_u {
  package get_num_reg_entries(scp : csco_addrmap_name_scope_s) : uint is {
    result = scp.get_all_child_entries().all(it is a csco_addrmap_reg_entry_s).size();
  };

  package reg_cov_units : list of csco_addrmap_reg_entry_cov_u is instance;
    keep reg_cov_units.size() == get_num_reg_entries(scope);

  package get_num_mem_entries(scp : csco_addrmap_name_scope_s) : uint is {
    result = scp.get_all_child_entries().all(it is a csco_addrmap_mem_entry_s).size();
  };

  package mem_cov_units : list of csco_addrmap_mem_entry_cov_u is instance;
    keep mem_cov_units.size() == get_num_mem_entries(scope);

  package !all_cov_entries : list of csco_addrmap_entry_s;


  post_generate() is also {
    var reg_ind : uint;
    var mem_ind : uint;
    all_cov_entries = get_all_entries();
    for each (e) in all_cov_entries {
      if e is a csco_addrmap_reg_entry_s (reg_entry) {
        reg_cov_units[reg_ind].reg = reg_entry;
        reg_ind += 1;
      }
      else if e is a csco_addrmap_mem_entry_s (mem_entry) {
        mem_cov_units[mem_ind].mem = mem_entry;
        mem_ind += 1;
      };
    };
  };

  csco_init_test() @sys.any is also {
    var all_entries := get_all_entries();
    assert that all_entries.size() == all_cov_entries.size() else 
      error("CSCO_ADDRMAP_ERR_ENTRY_CREATION_TOO_LATE_FOR_COV\n:",
              "some addrmap entries were created after generation, ", 
              " this will cause the entries to miss coverage collection.\n",
              " Type me.show_uncovered_entries() to see a list of uncovered entries.");
  };

  show_uncovered_entries() is {
    var tmp_list : list of csco_addrmap_entry_s = get_all_entries().all(it not in all_cov_entries);
    out("Below are uncovered entries: \n");
    for each (e) in tmp_list {
      out(e.long_name(""));
    };
  };
};


'>

