// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_types.e
//              :
// Title        : csco_addrmap package header file
// Project      : na
//              :
// Created On   : 09/26/2005
//              :
//              :
// Purpose      : base types for csco_addrmap package
//              :
// Description  : 
//              :
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'
package csco_addrmap;

#ifndef ADDR_FIELD_WIDTHS_DEFINED {
#define ADDR_FIELD_WIDTHS_DEFINED;

  define CSCO_ADDRMAP_MAX_ADDR_WIDTH          32;
  define CSCO_ADDRMAP_MAX_FIELD_WIDTH         32;
};


// All addressing related fields should use this type.
type csco_addrmap_addr_t : uint(bits:CSCO_ADDRMAP_MAX_ADDR_WIDTH);

// Used to hold field shadow and reset values.
type csco_addrmap_field_val_t : uint(bits:CSCO_ADDRMAP_MAX_FIELD_WIDTH);


extend message_tag : [CSCO_ADDRMAP];

// This type should be updated whenever necessary to alway mirror
// the definitions in BluePrint.
// <br>
// R    = READ_ONLY
// <br>
// W    = WRITE_ONLY
// <br>
// RW   = READ + WRITE
// <br>
// NA   = Not accessible
type csco_addrmap_sw_access_t : [
  R,
  W,
  RW,
  NA 
];

// This type should be updated whenever necessary to alway mirror
// the side effect definitions in BluePrint.
// <br>
// RCLR       = Clear on Read
// <br>
// WOSET      = Write One to Set
// <br>
// WOCLR      = Write One to Clear
// <br>

type csco_addrmap_side_effect_t : [
  RCLR,  
  WOSET,
  WOCLR
];

// Attributes of the field. 
// <br>
// CRITICAL :           Field should not be touched by any auto-test.
// <br>
// AUTOCOMP_OVERRIDE :  Even if field is hardware writable, do auto-compare in auto-testing.
//                      Even if field is not hardware writable, do not auto-compare.
// <br>
// IN_AUTOTEST :        Field is currently involved in an auto-test.
// <br>       
// HW_W        :        Field is hardware modifiable 
// <br>
// EXTERNAL    :        Field is external to specified physical slave     
// <br>
// IS_WIRE    :         Field is a wire     

type csco_addrmap_field_attribute_t : [
  CRITICAL,
  AUTOCOMP_OVERRIDE,
  IN_AUTOTEST,
  HW_W,
  EXTERNAL,
  IS_WIRE
];

// Selects whether to pack/unpack reset or shadow values of an
// addrmap entry.
type csco_addrmap_field_packing_sel_t : [
  RESET,
  SHADOW
];

// Used to describe endianness with which multi-byte entry rows
// are packed into addrmap tranasaction byte stream.
type csco_addrmap_endian_t : [
  LITTLE,
  BIG
];

// Selects address allocation algorithm for logical regions when
// it assigns offsets to its child regions.
// <br>
// RANDOM_SHIFT  = Randomly place each child region in the parent address space.
// <br>
// SEQUENTIAL    = Place each child regions in adjacent offsets, starting from 0.
// <br>
// MANUAL        = User specifies offset.
type csco_addrmap_addr_gen_alg_t : [
  MANUAL,                               // User specifies offset.
  RANDOM_SHIFT,                         // Randomly place each child region in the parent address space.
  SEQUENTIAL                            // Place each child regions in adjacent offsets, starting from 0.
];

// Specifies region type. Be default, only memory mapped regions
// are defined. But this can be extended to add support for other
// region types such as PCI-Express CFG and I/O regions.
type csco_addrmap_route_kind_t : [ADDRESS];

type csco_addrmap_space_t : [UNDEFINED, DEFAULT];

// Instance when determinant for logical regions.
type csco_addrmap_log_region_name_t : [
  NO_NAME
];

// Specifies interface of an addrmap region or the target
// interface of an addrmap transaction.
type csco_addrmap_interface_t : [ NONE
];

// Type of addrmap transaction.
type csco_addrmap_op_t : [
  READ,
  WRITE,
  FILL
];

type csco_addrmap_fill_kind_t : [
  CONSTANT,
  SHADOW,
  RANDOM,
  SKIP,
  REPLICATE
];

// Type of addrmap addressing used by transaction.
type csco_addrmap_trans_addr_scheme_t : [
  OFFSET,
  ENTRY
];

// Type of access via which the transaction will be executed.
type csco_addrmap_access_t : [
  FRONT_DOOR,
  BACK_DOOR
];

// Whether a write is posted or non-psoted.
type csco_addrmap_post_kind_t : [
  POSTED,
  NON_POSTED
];

// Whether the 'do' action will block until the transaction is
// sent (BLOCKING) or return in 0 time (NON_BLOCKING).
type csco_addrmap_trans_return_t : [
  BLOCKING,
  NON_BLOCKING,
  UNKNOWN
];

// Whether a non-posted transaction response contains errors.
// The values for this enumerated type should be defined in 
// order of increasing severity.
type csco_addrmap_resp_kind_t : [
  NORMAL,
  ERROR,
  ABORTED
];

unit csco_addrmap_utils_u {};

extend sys {
  // Instantiate global workspace for csco_addrmap eVC.
  csco_addrmap_utils : csco_addrmap_utils_u is instance;
};


'>
