// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_xml_parser.e
//              :
// Project      : csco_addrmap
//              :
//              :
// Description  : XML meta data namespace parser implementation
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2006 (c) Cisco Systems Inc.
<'
package csco_addrmap;

import csco_xml/e/stax/csco_xml_stax_top;
    
package type csco_addrmap_populate_xml_context_kind_t : [ ADDRMAP, REGFILE ];
package type csco_addrmap_populate_xml_state_t : [ FILTER, PARSE ];

//------------------------------------------------------------
extend csco_addrmap_name_scope_s {
  populate_from_xml_file(filename: string, top_scope: string) : ADDRESS'route_kind csco_addrmap_logical_region_s is {
    result = populate_from_xml_file_with_addr_space(filename, top_scope, DEFAULT);
  };
  
  populate_from_xml_file_with_addr_space(filename: string, top_scope: string, addr_space: csco_addrmap_space_t) : ADDRESS'route_kind csco_addrmap_logical_region_s is {

    var r := new csco_xml_stax_XMLStreamReader;
    r.openURI(filename);

    compute r.nextTag();
    r.require(START_ELEMENT, NULL, "rdl");

    // root context
    var ctx := new csco_addrmap_populate_xml_context_s;

    compute r.nextTag();
    r.require(START_ELEMENT, NULL, "copyright");
    var copyright := ctx.parse_text_element(r);

    // Blueprint 3.6 no longer includes the following tags
    compute r.nextTag();
    r.require(START_ELEMENT, NULL, "blueprint_version");
    var version := ctx.parse_text_element(r);

     compute r.nextTag();
     r.require(START_ELEMENT, NULL, "creator");
    var creator := ctx.parse_text_element(r);

    compute r.nextTag();
    r.require(START_ELEMENT, NULL, "pubdate");
    var pubdate := ctx.parse_text_element(r);

    messagef(LOW, "reading XML register definitions from %s\n",
             r.getLocation().getUri())
    {
      if(at_message_verbosity(HIGH)) {
        out("    created: ", pubdate);
        out("    by ", creator, " using blueprint version ", version);
      };
    };

    // recursively populate root context
    ctx.parse(r);

    r.close();

    // extract top_scope from last read addrmap
    var target := ctx.ctxs.top();
    if(top_scope != NULL) {
      var path := str_split(top_scope, "/[./]/");
      for each in path {
        target = target.ctx_insts.key(it);
        assert(target != NULL) else
        error("FIXME");
      };
    };

    // populate name scope tree
    target.populate_name_scope(me);

    // new containing logical region for corresponding addressing tree
    var rgn := target.populate_region(addr_space);
    if(rgn is a ADDRESS'route_kind csco_addrmap_logical_region_s (log)) {
      result = log;
    } else {
      gen result keeping {
        .name == rgn.name;
        !.naturally_aligned;
      };
      result.add_child_region_with_offset(rgn, 0);
    };
  };
};


//------------------------------------------------------------
// application parser state data container

package struct csco_addrmap_populate_xml_context_s {

  // layer type
  package !kind : csco_addrmap_populate_xml_context_kind_t;
  // hierarchical name component
  !name : string;
  // instance offset
  !offset : csco_addrmap_addr_t;
  // context width (maximum)
  !width : int;

  // context stack
  package !parent : csco_addrmap_populate_xml_context_s;

  // locally defined types
  !fields : list (key: name) of csco_addrmap_field_s;
  !regs : list (key: name) of csco_addrmap_entry_s;
  !reg_widths : list of int;
  package !ctxs : list (key: name) of csco_addrmap_populate_xml_context_s;

  // local instances
  !reg_insts : list (key: name) of csco_addrmap_entry_s;
  package !ctx_insts : list (key: name) of csco_addrmap_populate_xml_context_s;

  // skip current element
  skip(r: csco_xml_stax_XMLStreamReader, tag: string) is inline {
    while((r.nextEvent() != END_ELEMENT) ||
          (r.getLocalName() != tag)) { };
  };

  parse_text_element(r: csco_xml_stax_XMLStreamReader) : string is {
    r.require(START_ELEMENT, NULL, NULL);
    var tag := r.getLocalName();
    compute r.nextEvent();
    r.require(CHARACTERS, NULL, NULL);
    result = r.getText();
    skip(r, tag);
  };

  parse(r: csco_xml_stax_XMLStreamReader) is {
    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        "field"        { parse_field(r) };
        "reg"          { parse_reg(r) };
        "regfile"      { parse_ctx(r, REGFILE) };
        "addrmap"      { parse_ctx(r, ADDRMAP) };
        "reg_inst"     { parse_reg_inst(r) };
        "regfile_inst" { parse_ctx_inst(r, REGFILE) };
        "addrmap_inst" { parse_ctx_inst(r, ADDRMAP) };
        "dynamic_assign" { parse_dynamic_assign(r) };
        default {
          if(tag not in {"name"; "description"; "sources"}) {
            warning("unexpected tag in <",
                    str_lower(kind.as_a(string)), ">: ", tag);
          };
          skip(r, tag);
        };
      };
    };
  };

  package parse_ctx(r: csco_xml_stax_XMLStreamReader,
                    kind: csco_addrmap_populate_xml_context_kind_t) is
  {
    var ctx := new csco_addrmap_populate_xml_context_s with {
      .name = r.getAttributeValueNS(NULL, "name");
      .kind = kind;
      .parent = me;
    };
    ctxs.add(ctx);

    // recurse
    ctx.parse(r);

    // cleanup
    ctx.parent = NULL;
    ctx.fields.clear();
    ctx.regs.clear();
    ctx.reg_widths.clear();
    ctx.ctxs.clear();
  };

  parse_reg(r: csco_xml_stax_XMLStreamReader) is {
    var reg := new csco_addrmap_entry_s with {
      .name = r.getAttributeValueNS(NULL, "name");
      .width = r.getAttributeValueNS(NULL, "width").as_a(int) >> 3;
    };
    regs.add(reg);
    reg_widths.add(r.getAttributeValueNS(NULL, "width").as_a(int));

    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        "field"      { parse_field(r) };
        "field_inst" { reg.add_field(parse_field_inst(r)) };
        default {
          if(tag not in {"name"; "description"; "dynamic_assign"}) {
            warning("unexpected tag in <reg> content: ", tag);
          };
          skip(r, tag);
        };
      };
    };
  };

  parse_field(r: csco_xml_stax_XMLStreamReader) is {

    var hw_r: bool = FALSE;
    
    var fld := new csco_addrmap_field_s with {
      .name = r.getAttributeValueNS(NULL, "name");
      .side_effects = 0;
    };
    fields.add(fld);

    if(r.getAttributeValueNS(NULL, "donttest") == "-1") {
      fld.add_attribute(CRITICAL);
    };
    
    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        "software" {
          var rd := (r.getAttributeValueNS(NULL, "read") == "true");
          var wr := (r.getAttributeValueNS(NULL, "write") == "true");
          if(rd && wr) { fld.sw_access = RW }
          else if(rd)  { fld.sw_access = R }
          else if(wr)  { fld.sw_access = W }
          else         { fld.sw_access = NA };

          for each (effect) in all_values(csco_addrmap_side_effect_t) {
            if(r.getAttributeValueNS(NULL, str_lower(effect.as_a(string))) == "true") {
              fld.add_side_effect(effect);
            };
          };
        };
        "hardware" {
          if(r.getAttributeValueNS(NULL, "write") == "true") {
            fld.add_attribute(HW_W);
          };
          if(r.getAttributeValueNS(NULL, "read") == "true") {
            hw_r = TRUE;       
          };
          if(r.getAttributeValueNS(NULL, "hwclr") == "true") {
            fld.add_attribute(HW_W);
            hw_r = TRUE;       
          };
          if(r.getAttributeValueNS(NULL, "hwset") == "true") {
            fld.add_attribute(HW_W);
            hw_r = TRUE;       
          };
          if(r.getAttributeValueNS(NULL, "we") == "low|high") {
            fld.add_attribute(HW_W);
            hw_r = TRUE;       
          };
        };
        "counter" { 
          fld.add_attribute(HW_W);
          hw_r = TRUE;       
        };
        default {
          if(tag not in {"name"; "description"; "interrupt"}) {
            warning("unexpected tag in <field> content: ", tag);
          };
        };
      };
      skip(r, tag);
    };
    
    if(fld.sw_access == R && !(hw_r && fld.has_attribute(HW_W))) {  fld.add_attribute(IS_WIRE) };
  };

  package parse_ctx_inst(r: csco_xml_stax_XMLStreamReader,
                         kind: csco_addrmap_populate_xml_context_kind_t) is
  {
    // lookup context type
    var typname := r.getAttributeValueNS(NULL, "type");
    var ctx := me;
    var t : csco_addrmap_populate_xml_context_s;
    while((t == NULL) && (ctx != NULL)) {
      t = ctx.ctxs.key(typname);
      ctx = ctx.parent;
    };
    assert(t != NULL) else
      error("FIXME");
    assert(t.kind == kind) else
      error("FIXME");

    var name := r.getAttributeValueNS(NULL, "name");
    var att := r.getAttributeValueNS(NULL, "num");
    var num := (att == NULL) ? 1 : att.as_a(int);
    att = r.getAttributeValueNS(NULL, "addr");
    var base := ((att != NULL) ? att.as_a(csco_addrmap_addr_t) : 0);
    att = r.getAttributeValueNS(NULL, "incr");
    var incr := ((att != NULL) ? att.as_a(csco_addrmap_addr_t) : 0);

    for i from 0 to num - 1 {
      ctx = t.instance();
      ctx.name = ((num == 1) ? name : dec(name, "[", i, "]"));
      ctx.offset = base + (i * incr);
      ctx_insts.add(ctx);
    };
    width = max(width, t.width);

    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        default {
          if(tag not in {"name"; "description"}) {
            warning("unexpected tag in <",
                    str_lower(kind.as_a(string)), "_inst> content: ", tag);
          };
        };
      };
      skip(r, tag);
    };
  };

  parse_reg_inst(r: csco_xml_stax_XMLStreamReader) is {
    // lookup reg type
    var typname := r.getAttributeValueNS(NULL, "type");
    var ctx := me;
    var t : csco_addrmap_entry_s;
    var w : int;
    while((t == NULL) && (ctx != NULL)) {
      var i := ctx.regs.key_index(typname);
      if(i >= 0) {
        t = ctx.regs[i];
        w = ctx.reg_widths[i];
      };
      ctx = ctx.parent;
    };
    assert(t != NULL) else
      error("FIXME");

    width = max(width, w);

    // create specific subtype based on entry depth
    var reg : csco_addrmap_entry_s;
    var att := r.getAttributeValueNS(NULL, "num");
    var depth := (att == NULL) ? 1 : att.as_a(int);
    if(depth > 1) {
      reg = new csco_addrmap_mem_entry_s with {
        .depth = depth;
        .initialized = FALSE; // FIXME?
        .width = t.width;
      };
    } else {
      reg = new csco_addrmap_reg_entry_s;
      reg.width = t.width;
    };

    // local reg attributes
    var alias := r.getAttributeValueNS(NULL, "alias");
    reg.name = r.getAttributeValueNS(NULL, "name");
    att = r.getAttributeValueNS(NULL, "addr");
    reg.offset = ((att == NULL) ? 0 : att.as_a(int));


    var external := r.getAttributeValueNS(NULL, "external");
    // copy saved type data
    for each (fld) in t.fields {

      //if EXTERNAL becomes available at field granulatiry this
      //should be moved to parse_field
      if(external == "1") {
        fld.add_attribute(EXTERNAL);
      };
      
      if(alias == NULL) {
        reg.fields.add(fld.copy());
      } else {
        // hmmm... better way todo this?
        reg.fields.add(new csco_addrmap_alias_field_s with {
          .name = fld.name;
          .width = fld.width;
          .lsb_pos = fld.lsb_pos;
          .sw_access = fld.sw_access;
          .attributes = fld.attributes;
          .side_effects = fld.side_effects;
        });
        // don't need reset/shadow for alias
      };
    };

    if(alias != NULL) {
      reg.aliased_from = new csco_addrmap_reg_entry_s with {
        .name = alias;
      };
    };

    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        default {
          if(tag not in {"name"; "description"}) {
            warning("unexpected tag in <reg_inst> content: ", tag);
          };
        };
      };
      skip(r, tag);
    };

    reg_insts.add(reg);
  };

  parse_dynamic_assign(r: csco_xml_stax_XMLStreamReader) is {
    // lookup assignment type
    var typname := r.getAttributeValueNS(NULL, "type");
    var ctx := me;

    case(typname) {
      "reset" {
        var lhs := r.getAttributeValueNS(NULL, "lhs");

        var field_path := str_split(lhs, ".");
        var entry := ctx.reg_insts.key(field_path[0]);
        assert entry != NULL else
          error("Unable to find reg_inst ", field_path[0]);
        var field := entry.get_field(field_path[1]);
          
        while(r.nextTag() != END_ELEMENT) {
          var tag := r.getLocalName();
          case(tag) {
            "dynamic_assign_rhs" {
              var att := r.getAttributeValueNS(NULL, "num");
              var reset := (att != NULL);
              if(reset) {
                if(field.width <= 32) {
                  field.set_reset_and_shadow_val(att.as_a(uint));
                } else {
                  field.set_reset_and_shadow_val(att.as_a(int(bits: *)));
                };
              };
            };
            default {
              warning("unexpected tag in <dynamic_assign> content: ", tag);
            };
          };
          skip(r, tag);
        };
      };
      default {
        skip(r, r.getLocalName());
      };
    };
  };

  parse_field_inst(r: csco_xml_stax_XMLStreamReader) : csco_addrmap_field_s is {
    // lookup field type
    var typname := r.getAttributeValueNS(NULL, "type");
    var ctx := me;
    var fld : csco_addrmap_field_s;
    while((fld == NULL) && (ctx != NULL)) {
      fld = ctx.fields.key(typname);
      ctx = ctx.parent;
    };
    assert(fld != NULL) else
      error("FIXME");

    // create specific subtype based on field width
    var att := r.getAttributeValueNS(NULL, "num");
    var width := (att == NULL) ? 1 : att.as_a(int);

    if(width <= 32) {
      result = new csco_addrmap_narrow_field_s;
    } else {
      result = new csco_addrmap_wide_field_s;
    };

    // local field attributes
    result.name = r.getAttributeValueNS(NULL, "name");
    result.width = width;
    att = r.getAttributeValueNS(NULL, "lsb");
    result.lsb_pos = (att == NULL) ? 0 : att.as_a(int);

    // copy saved type data
    result.sw_access = fld.sw_access;
    result.attributes = fld.attributes;
    result.side_effects = fld.side_effects;

    var reset : bool;

    while(r.nextTag() != END_ELEMENT) {
      var tag := r.getLocalName();
      case(tag) {
        "reset" {
          compute r.nextEvent();
          r.require(CHARACTERS, NULL, NULL);
          att = r.getText();
          reset = (att != NULL);
          if(reset) {
            if(width <= 32) {
              result.set_reset_and_shadow_val(att.as_a(uint));
            } else {
              // FIXME make sure specman does this right
              result.set_reset_and_shadow_val(att.as_a(int(bits: *)));
            };
          };
        };
        default {
          if(tag not in {"name"; "description"}) {
            warning("unexpected tag in <field_inst> content: ", tag);
          };
        };
      };
      skip(r, tag);
    };

    if(!reset) {
      // undefined reset value
      if(width <= 32) {
        var val : csco_addrmap_field_val_t;
        gen (val);
        result.set_reset_and_shadow_val(val);
      } else {
        var val : uint(bits: 256);  // number of bits is irrelevant
        gen (val);
        result.set_reset_and_shadow_val(val);
      };
    };
  };

  populate_name_scope(s: csco_addrmap_name_scope_s) is {
    s.name = name;
    for each (reg) in reg_insts {
      s.add_child_entry(reg);
    };
    for each (ctx) in ctx_insts {
      var ss := new csco_addrmap_name_scope_s;
      ctx.populate_name_scope(ss);
      s.add_child_scope(ss);
    };
  };

  populate_region(addr_space: csco_addrmap_space_t) : csco_addrmap_region_s is {
    if(ctx_insts.is_empty()) {
      // no need to create logical region for self-contained context
      var phy := new csco_addrmap_physical_region_s with {
        .name = name;
        //.width = width >> 3;
        .hdl_path = calc_hdl_path();
      };
      for each (reg) in reg_insts {
        phy.add_child_entry(reg);
      };
      result = phy;
    }
    else {
      var log : ADDRESS'route_kind csco_addrmap_logical_region_s;
      gen log keeping {
        .name == name;
        .addr_space == addr_space;
        !.naturally_aligned;
        //.addr_space_size == 0;
      };

      var addr : csco_addrmap_addr_t = 0;
      var phy : csco_addrmap_physical_region_s;

      var insts := { ctx_insts.as_a(list of any_struct); reg_insts }
        .sort((it is a csco_addrmap_entry_s (reg))
              ? reg.offset
              : it.as_a(csco_addrmap_populate_xml_context_s).offset);

      for each in insts {
        if(it is a csco_addrmap_entry_s (reg)) {
          if(phy == NULL) {
            // autovivify physical region
            addr = reg.offset;
            phy = new csco_addrmap_physical_region_s with {
              .name = hex(name, "@", addr);
              //.width = width >> 3;
              .hdl_path = calc_hdl_path();
            };
            log.add_child_region_with_offset(phy, addr);
          };
          reg.offset -= addr;
          phy.add_child_entry(reg);
        }
        else {
          var ctx := it.as_a(csco_addrmap_populate_xml_context_s);
          phy = NULL;
          var rgn := ctx.populate_region(addr_space);
          log.add_child_region_with_offset(rgn, ctx.offset);
          log.addr_space_size = ctx.offset + rgn.get_region_size();
        };
      };
      if(phy != NULL) {
        log.addr_space_size = addr + phy.get_region_size();
      };
      result = log;
    };
  };

  // additional regfile hdl path suffix
  package calc_hdl_path() : string is {
    if(kind == REGFILE) {
      var rf_path : list of string;
      var ctx := me;
      while((ctx != NULL) && (ctx.kind == REGFILE)) {
        rf_path.add(ctx.name);
        ctx = ctx.parent;
      };
      result = append(str_join(rf_path.reverse(), "_"), "_");
    };
  };

  package instance() : csco_addrmap_populate_xml_context_s is {
    result = new;
    result.kind = kind;
    result.name = name;
    result.width = width;
    result.offset = offset;
    for each (src) in reg_insts {
      var dst : csco_addrmap_entry_s;
      if(src.aliased_from == NULL) {
        dst = deep_copy(src);
      } else {
        dst = src.copy();
        dst.fields = { };
        dst.fields.add(src.fields.apply(.copy()));
      };
      result.reg_insts.add(dst);
    };
    // second pass link in alias entries
    for each (src) in reg_insts {
      if(src.aliased_from != NULL) {
        var dst := result.reg_insts.key(src.name);
        result.reg_insts.key(src.aliased_from.name).link_alias_entry(dst);
      };
    };
    for each (ctx) in ctx_insts {
      result.ctx_insts.add(ctx.instance());
    };
  };
};

'>
