// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_seq_err_inj.e
//              :
// Project      : csco_addrmap
//              :
// Created On   : 08/15/2005
//              :
//              :
// Description  : This is a an automatic addrmap sequence to inject errored
//                addrmap transactions into the system.
//
//
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'
package csco_addrmap;

import csco_addrmap/e/csco_addrmap_top;


//
// This sequence will issue transactions to 'holes' in the list
// of physical regions under the default scope. The user may also
// explicitly constrain the list of phy regions.
//
extend csco_addrmap_seq_kind_t : [ADDR_HOLE_ERR];

extend ADDR_HOLE_ERR csco_addrmap_seq {
  // The sequence will only work on phy regions in this list.
  all_phy_regions : list of csco_addrmap_physical_region_s;
    keep soft all_phy_regions == default_scope.get_all_child_phy_regions();
    
  // Number of errored transactions to issue.
  num_trans : uint;
    keep num_trans > 0;
    keep soft num_trans in [10..50];
    keep read_only(csco_infinite_len) => soft num_trans == UNDEF;

  // Whether register entries should be inlcuded in the intermix.
  include_regs : bool;
    keep soft include_regs == TRUE;

  // Whether memory entries should be inlcuded in the intermix.
  include_mems : bool;
    keep soft include_mems == TRUE;

  // User can use this field to define a delay distribution between each transaction.
  rand_delay : uint;
    keep soft rand_delay <= 500;

  package !cur_phy_region : csco_addrmap_physical_region_s;
    //keep cur_phy_region in read_only(all_phy_regions);    // Specman bug!

  package !cur_hole_offset : csco_addrmap_addr_s;
    //keep cur_hole_offset in read_only(cur_phy_region.get_hole_offsets());   // Specman bug!

  body() @driver.clock is {
    var hole_offsets : list of csco_addrmap_addr_s;

    message(MEDIUM, "ADDRMAP_ADDR_HOLE_ERR: launching ", kind, " sequence with scope as: ", 
                    default_scope.long_name(""), ", and num_trans=", num_trans);
    for i from 0 to num_trans - 1 {
      while hole_offsets.is_empty() {
        gen cur_phy_region keeping {
          it in all_phy_regions;
        };

        hole_offsets = cur_phy_region.get_hole_offsets();
        if hole_offsets.is_empty() {
          all_phy_regions = all_phy_regions.all(it != cur_phy_region);
        };

        if all_phy_regions.is_empty() {
          message(NONE, "ADDRMAP_ADDR_HOLE_ERR: cannot find a phy_region under ", default_scope, " that has hole offsets, aborting sequence...");
          return;
        };
      };

      gen cur_hole_offset keeping {
        it in value(hole_offsets);
      };

      message(CSCO_ADDRMAP, LOW, "ADDRMAP_ADDR_HOLE_ERR: issuing a transaction to ", 
                  cur_hole_offset.csco_short_name(), " of ", cur_phy_region.csco_short_name(), " physical region");

      var target := new csco_addrmap_region_addr_s with {
        .region = cur_phy_region;
        .addr = cur_hole_offset;
      };

      addrmap_do offset_trans keeping {
        .target == target;
        .access == FRONT_DOOR;
        .num_bytes in cur_phy_region.child_entries.width;
        soft .intf_iterator.seg_alg == GREEDY;
      };
      message(CSCO_ADDRMAP, MEDIUM, "ADDRMAP_ADDR_HOLE_ERR: finished a transaction to offset ", 
                  hex(cur_hole_offset), " of ", cur_phy_region.csco_short_name(), " physical region");
    };
  };
}; // extend ADDR_HOLE_ERR csco_addrmap_seq


'>
