// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_seq_reg_por.e
//              :
// Project      : csco_addrmap
//              :
// Created On   : 08/15/2005
//              :
//              :
// Description  : This is a directed and automatic register power on read test. 
//                The sequence will read all the registers in the sequence name
//                scope. If no scoreboards are instantiated in the system, then
//                the user should set the local_scoreboard == TRUE, which will
//                allow checking of the read response values.
//
//
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------


<'
package csco_addrmap;

import csco_addrmap/e/csco_addrmap_top;


extend csco_addrmap_seq_kind_t : [REG_POR];

// This sequence reads all the registers right after reset to ensure all fields
// power up to their documented RDL values.
extend REG_POR'kind csco_addrmap_seq {
  // Whether or not the sequence should do local scoreboarding. Should
  // be TRUE if the rest of the TB doesn't have scoreboard.
  local_scoreboard : bool;
    keep soft local_scoreboard == TRUE;
  
  // By default, includes all the registers under the sequence scope. But
  // user may also supply an explcity list of registers.
  all_regs : list of csco_addrmap_reg_entry_s;
    keep soft all_regs.size() == 0;

  // User can use this field to define a delay distribution between each transaction.
  rand_delay : uint;
    keep soft rand_delay <= 500;

  include_wo_regs : bool;
    // soft constrained to TRUE to force TB to disallow write-only
    // regs, this ensures we don't accidentally omit testable regs.
    keep soft include_wo_regs == TRUE;


  keep soft entry_trans.access == FRONT_DOOR;
  keep entry_trans.opcode == READ;
  keep soft entry_trans.blocking_kind == BLOCKING;

  package !filtered_entries : list of csco_addrmap_reg_entry_s;

  post_generate() is also {
    if all_regs.is_empty() {
      gen all_regs keeping {
        it.is_a_permutation(default_scope.get_all_child_entries().all(it is a csco_addrmap_reg_entry_s).as_a(list of csco_addrmap_reg_entry_s));
      };
    };
    filtered_entries = filter_entries(all_regs);
  };

  filter_entries(orig : list of csco_addrmap_reg_entry_s) : list of csco_addrmap_reg_entry_s is {
    for each (e) in orig {
      if e.fields.has(.sw_access == W) and !include_wo_regs { continue; };
      result.add(e);
    };
  };

  body() @driver.clock is also {
    messagef(MEDIUM, "ADDRMAP_POR: launching REG_POR test with scope as : %s\n", default_scope.long_name(""));
    messagef(CSCO_ADDRMAP, MEDIUM, "ADDRMAP_POR: Following addrmap entries will be read during REG_POR test:\n") {
      for each (e) in filtered_entries {
        out(e.long_name(default_scope.name));
      };
    };

    for each (r) in filtered_entries {
      for each (f) in r.fields {
        f.add_attribute(IN_AUTOTEST);
      };

      messagef(CSCO_ADDRMAP, HIGH, "ADDRMAP_POR: now reading %s\n", r.long_name(default_scope.name));
      if not r.fields.has(.is_auto_comp()) {
        warning("ADDRMAP_POR: ", r.long_name(default_scope.name), " fields don't have any auto_comparable fields");
      };

      addrmap_do entry_trans keeping {
        .entry_ptr == read_only(r);
      };

      if local_scoreboard {
        messagef(CSCO_ADDRMAP, HIGH, "ADDRMAP_POR: calling update_shadow() from %s for %s because local_scoreboard = TRUE\n", kind, 
                                        entry_trans.csco_short_name());
        r.update_shadow(entry_trans);
      };

      // remove autotest attribute from affected register fields.
      for each (f) in r.fields {
        f.remove_attribute(IN_AUTOTEST);
      };

      gen rand_delay;
      messagef(CSCO_ADDRMAP, FULL, "ADDRMAP_POR: wait %d cycles before issuing next random access\n", rand_delay);
      wait [rand_delay];
    }; // for each (r) in filtered_entries
  }; // body() 
}; // extend REG_POR'kind csco_addrmap_seq


'>
