// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_addrmap_seq_simple.e
//              :
// Project      : csco_addrmap
//              :
// Created On   : 08/15/2005
//              :
//              :
// Description  : Overrides default SIMPLE sequence implementation with backwards
//                compatible behavior where the user may constain the sequence 
//                item (gen_item) when launching the sequence. The user must 
//                constrain the 'addr_scheme' field to indicate what kind of addrmap
//                transaction he wishes to do and then constrain one of 'offset_item'
//                or 'entry_item'.
//
//
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------


<'
package csco_addrmap;

import csco_addrmap/e/csco_addrmap_top;


extend csco_addrmap_seq_kind_t : [SIMPLE_ENTRY, SIMPLE_OFFSET];


// Allows user to issue a single ENTRY based addrmap transaction
// from a virtual sequence.
extend SIMPLE_ENTRY csco_addrmap_seq {
  // User should constrain this field as the transaction that will
  // be sent.
  entry_item : ENTRY'addr_scheme csco_addrmap_trans_s;
    keep soft entry_item.addrmap_parent_seq == read_only(me);
    keep soft entry_item.scope == read_only(default_scope);
    keep entry_item.macro_call == TRUE;

  body() @driver.clock is only {
    // Tell is_relevant() what the source driver for this transaction is.
    prev_source = entry_item.source;

    do entry_trans keeping {
      it == entry_item;
    };
  };
};


// Allows user to issue a single OFFSET based addrmap transaction
// from a virtual sequence.
extend SIMPLE_OFFSET csco_addrmap_seq {
  // User should constrain this field as the transaction that will
  // be sent.
  offset_item : OFFSET'addr_scheme csco_addrmap_trans_s;
    keep soft offset_item.addrmap_parent_seq == read_only(me);
    keep offset_item.macro_call == TRUE;

  body() @driver.clock is only {
    // Tell is_relevant() what the source driver for this transaction is.
    prev_source = offset_item.source;

    do offset_trans keeping {
      it == offset_item;
    };
  };
};


'>
