# ----------------------------------------------------------------------------
#
# File name    : csco_addrmap_xml_parser.e
#              :
# Project      : csco_addrmap
#              :
# Developers   : spadix
#              :
# Description  : new XML output generator to simplify some processors
#              : focuses on the instance tree of the target addrmap component
#              : by including component (type) definition information
#              : with each instance directly
#              : (vs the indirect reference in the default xml output)
#              :
# ----------------------------------------------------------------------------
# >>>>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# ----------------------------------------------------------------------------
# Copyright 2005-2006 (c) Cisco Systems Inc.
# Cisco Confidential
# ----------------------------------------------------------------------------
package xmldirect;
use warnings;
use strict;
require xml;
our @ISA = qw(xml);

my $indent = 2;

# FIXME why isn't this a method of RDLInstancePath?!
sub path_local_name {
  my ($path) = @_;
  my $result = $path->{last}[0]->name();
  $result .= '[' . $path->{last}[1] . ']'
    if(($path->{last}[1] >= 0) && ($path->{last}[0]->num() > 1));
  return($result);
}

##----------------------------------------------------------------------------
## gen
##----------------------------------------------------------------------------
sub gen {
  my ($this) = @_;

  # this *must* be the first line in the file
  # unfortunately seems to be no way to get it there?!
  #$this->prepend("<?xml version='1.0'?>\n");

  my $comp = $this->{target};
  my $assign = [];
  my $path = RDLInstancePath->new();

  # include namespace and modifier attribute in document element
  my $ns = $::CL_xmldirect_namespace->[0] ||
    "http://www.cisco.com/2005/blueprint";  # default namespace

  $this->write("<rdl:body mode='direct' xmlns:rdl='$ns'>\n");

  $this->write("$this->{pad}<rdl:" . $comp->type());
  $this->write(" name='" . $comp->name() . "'");

  my $func = "gen_".$comp->type()."_component";
  {
    no strict 'refs';
    $this->$func($comp, $assign, $path);
  }
  $this->write(">\n"); #close addrmap tag
  $this->{pad} .= " " x $indent;

  $this->write($this->docprop_str($comp));

  $this->gen_component($comp, $assign, $path);

  $this->{pad} = substr($this->{pad}, 0, -$indent);
  $this->write("$this->{pad}</rdl:".$comp->type().">\n");

  $this->write("</rdl:body>\n");
  $this->write_output();
}


#------------------------------------------------------------------------
# skip component type elements and recurse instances instead
# this is what makes this version "direct"

# skip local components - only instances
sub gen_local_components {
  my( $this, $comp, $assign, $path ) = @_;
  # skip component (type) generation
}

sub gen_instance {
  my( $this, $assign, $path ) = @_;
  my $comp = $path->instance()->type();
  my $name = path_local_name($path);
  my $inst = $path->{last}[0];

  $this->write("$this->{pad}<rdl:" . $comp->type() . "_inst");
  $this->write(" name='" . $name . "'");
  $this->write(" type='" . $comp->name() . "'");

  $this->write(" num='" . $inst->num() . "'")
    if($inst->num() > 1);
  $this->write(" lsb='" . $inst->lsb() . "'")
    if($inst->lsb() != -1);
  if($comp->type() ne "field") {
    my $idx = $path->{last}[1];
    $idx = 0 if($idx < 0);
    $this->write(" addr='" . ($inst->addr() + $idx * $inst->incr()) . "'");
    $this->write(" incr='" . $inst->incr() . "'"); # not really used(?)
  }

  $this->{pad} .= " " x $indent;
  my $func = "gen_" . $comp->type() . "_component";
  {
    no strict 'refs';
    $this->$func($comp, $assign, $path);
  }
  $this->write(">\n"); #close addrmap tag
  $this->write($this->docprop_str($comp));

  $this->RegCodeGen::gen_instance($assign, $path);

  $this->{pad} = substr($this->{pad}, 0, -$indent);
  $this->write("$this->{pad}</rdl:" . $comp->type() . "_inst>\n");
}

sub gen_addrmap_component {
  my ($this, $comp, $assign, $path) = @_;
  $this->write(" width='" . $comp->{stats}{width}{max} . "'");
}

sub gen_component {
  my( $this, $comp, $assign, $path ) = @_;
  $this->RegCodeGen::gen_component($comp, $assign, $path);
}

sub gen_local_instances {
  my $this = shift;
  $this->RegCodeGen::gen_local_instances(@_);
}

sub gen_field_inst {
  my ($this, $assign, $path) = @_;
  $this->RegCodeGen::gen_field_inst($assign, $path);
}

sub gen_reg_inst {
  my ($this, $assign, $path) = @_;
  $this->RegCodeGen::gen_reg_inst($assign, $path);
}

sub gen_regfile_inst {
  my ($this, $assign, $path) = @_;
  $this->RegCodeGen::gen_regfile_inst($assign, $path);
}

sub gen_addrmap_inst {
  my ($this, $assign, $path) = @_;
  $this->RegCodeGen::gen_addrmap_inst($assign, $path);
}

sub gen_reg_component {
  my ($this, $comp, $assign, $path) = @_;
  $this->SUPER::gen_reg_component($comp);
  $this->SUPER::gen_reg_inst($path->{last}[0]);
}

##----------------------------------------------------------------------------
## docprop_str
##----------------------------------------------------------------------------

my %xmlesc = ('&' => '&amp;', '<' => '&lt;', '>' => '&gt;');

# optional name and description properties
sub docprop_str {
  my ($this, $comp) = @_;
  my $result = "";

  # replace end of line string literal with nothing ..
  # forces the xml output of $desc / $name to end up on one line
  #   output-->  <rdl:name>$field1</rdl:desc>
  # search at the end the line for the string literal "end of line"
  #   ex: $name = field1

  unless($::CL_xmldirect_nodoc) {
    my $name = $comp->desc_name() || "";
    chomp($name);
    $name =~ s[([&<>])][$xmlesc{$1}]g;
    $result = "$this->{pad}<rdl:name>$name</rdl:name>\n"
      if(length($name));

    # FIXME really should be "desclevel=reg" or somesuch
    unless($::CL_xmldirect_nodesc) {
      my $desc = $comp->desc() || "";
      chomp($desc);
      $desc =~ s[([&<>])][$xmlesc{$1}]g;
      if(length($desc)) {
	$desc .= "\n$this->{pad}" if( $desc =~ m/[\n]/ );
	$result .= "$this->{pad}<rdl:desc>$desc</rdl:desc>\n";
      }
    }
  }
  return($result)
}

# DENALI BUG WORKAROUND(S)
# blueprint release 3.4 xml.pm is broken...
##----------------------------------------------------------------------------
## gen_field_component
##----------------------------------------------------------------------------
sub gen_field_component {
  my ($this, $comp, $assign, $path) = @_;

  $this->write(" rclr='1'")      if( $comp->rclr() );
  $this->write(" woset='1'")     if( $comp->woset() );
  $this->write(" woclr='1'")     if( $comp->woclr() );

  $this->write(" we='1'")        if( $comp->we() );
  $this->write(" wel='1'")       if( $comp->wel() );

  $this->write(" sticky='1'")    if( $comp->sticky() );
  $this->write(" stickybit='1'") if( $comp->stickybit() );

  $this->write(" anded='1'")     if( $comp->anded() );
  $this->write(" ored='1'")      if( $comp->ored() );
  $this->write(" xored='1'")     if( $comp->xored() );

  $this->write(" resetsignal='".$comp->resetsignal()->name()."'")
    if( defined $comp->resetsignal() );

  $this->write(" encode='".$comp->encode()."'")
    if( defined $comp->encode() );

  # workaround for blueprint 3.4[.1-3] bug w/missing donttest attribute
  foreach my $ass (@$assign) {
    if($ass->{assign}->lhs()->type() eq 'attribute') {
      my $attr = $ass->{assign}->rhs()->get_ref()->attribute_instance();
      if($attr->name() eq 'donttest_field') {
	$this->write(" donttest='1'");
      }
    }
  }

  if( $comp->counter() ) {
    $this->write(" counter='1'");

    $this->write(" threshold='".$comp->threshold()."'")
      if( defined $comp->threshold() );

    $this->write(" saturate='".$comp->saturate()."'")
      if( defined $comp->saturate() );

    $this->write(" counterincr='".$comp->counterincr()."'")
      if( defined $comp->counterincr() );

    $this->write(" incrwidth='".$comp->incrwidth()."'")
      if( defined $comp->incrwidth() );
  }

  # workaround for blueprint 3.4[.x?] bug decoding incorrect attributes
  my $sw_acc = "";
  $sw_acc   .= "r"  if( $comp->sw_r() );
  $sw_acc   .= "w"  if( $comp->sw_w() );
  $sw_acc    = "na" if( $sw_acc eq "" );

  my $hw_acc = "";
  $hw_acc   .= "r"  if( $comp->hw_r() );
  $hw_acc   .= "w"  if( $comp->hw_w() );
  $hw_acc    = "na" if( $hw_acc eq "" );

  $this->write(" sw='$sw_acc'");
  $this->write(" hw='$hw_acc'");

  $this->SUPER::gen_field_inst($path->{last}[0]);
}

1;
