// ----------------------------------------------------------------------------
//
// File name    : csco_base_env_addrmap.e
//              :
// Project      : csco_base_env
//              :
// Created On   : 11/11/2005
//              :
//              :
// Description  : instantiate csco_addrmap in base env, and
//              : hook up all cross-references
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_base_env;

extend csco_tb_u {
  // instantiate addrmap env in testbench
  addrmap : csco_addrmap_env_u is instance;
    keep addrmap.tb_ptr == me;
    keep addrmap.tb_name == read_only(tb_name);

  // constrain addrmap driver pointer in top VSD
  keep driver.addrmap_driver == read_only(addrmap.driver);

  // generate/set the offsets of each region within its parent region(s)
  csco_reset() @sys.any is also {
    addrmap.build_addrmaps();
  };
};

extend FALSE'is_top csco_tb_u {
  post_generate() is also {
    // add env_scope of child base envs to the top-level base env.
    var parent_tb := get_enclosing_unit(is_top csco_tb_u);
    if(parent_tb != NULL) {
      parent_tb.addrmap.scope.add_child_scope(addrmap.scope);
    };
  };
};

// add reference to central addrmap driver in VSD
extend csco_tb_driver_u {
  addrmap_driver : csco_addrmap_driver_u;
};

// default interface for VSD sequences to do addrmap sequences
extend csco_tb_seq {
  !addrmap_seq : csco_addrmap_seq;
    keep addrmap_seq.driver == driver.addrmap_driver;
};


//------------------------------------------------------------
// addrmap extensions

extend csco_addrmap_env_u {
  USE_PARENT_CFG begin
    parent_config_type = csco_tb_config_s,
    parent_unit_type = csco_tb_u
  end;

  // reference to parent csco_tb_u unit
  tb_ptr : csco_tb_u;
    keep driver.tb_ptr == read_only(tb_ptr);

  // name of parent TB
  tb_name : csco_tb_name_t;
    keep driver.tb_name == read_only(tb_name);
    keep for each (s) in spaces {
      s.tb_name == tb_name;
    };

    // This is a restriction to setup the scope tree prior to 
    // post_generate() so that we can automatically collect
    // addrmap entry coverage. 
    keep scope == build_scope_tree(tb_ptr.config);
    keep gen (tb_ptr, tb_name) before (scope);

  build_scope_tree(tbc : csco_tb_config_s) : csco_addrmap_name_scope_s is {
    result = new csco_addrmap_name_scope_s with {
      .name = append(tbc.tb_name, "_ROOT");
    };
  };
}; // extend csco_addrmap_env_u

extend csco_addrmap_space_u {
  USE_PARENT_CFG begin
    parent_config_type = csco_tb_config_s,
    parent_unit_type = csco_tb_u
  end;

  // name of parent TB
  tb_name : csco_tb_name_t;
    csco_print_attr tb_name LONG precede=all_regions;
};

extend csco_addrmap_driver_u {
  // reference to parent csco_tb_u unit
  tb_ptr : csco_tb_u;
    keep gen (tb_ptr) before (reset_seq, init_test_seq, pre_test_seq, main_test_seq, post_test_seq);

  // name of parent TB
  tb_name : csco_tb_name_t;
};

extend csco_addrmap_seq {
  // name of parent TB
  tb_name : csco_tb_name_t;
    keep tb_name == read_only(driver.tb_name);

  // local config reference
  config : csco_config_base_struct_s;
    keep soft config == NULL;
    //keep soft config == read_only(driver.tb_ptr.config);
    //keep read_only(parent_sequence is a csco_tb_seq (seq)) =>
    //  soft config == read_only(seq.config);
    //keep read_only(parent_sequence is a csco_addrmap_seq (seq)) =>
    //  soft config == read_only(seq.config);

  post_generate() is also {
    if config == NULL {
      if parent_sequence is a csco_addrmap_seq (aseq) {
        config = aseq.config;
      }
      else if parent_sequence is a csco_tb_seq (tseq) {
        config = tseq.config;
      }
      else {
        config = driver.tb_ptr.config;
      };
    };
  };
};

extend CSCO_INIT_TEST'kind csco_addrmap_seq {
  // init sequence subtype
  init_kind : csco_tb_init_test_kind_t;
    keep soft init_kind == DEFAULT;
    keep read_only(parent_sequence is a CSCO_INIT_TEST'kind csco_tb_seq (seq)) =>
      soft init_kind == read_only(seq.init_kind);
    keep read_only(parent_sequence is a CSCO_INIT_TEST'kind csco_addrmap_seq (seq)) =>
      soft init_kind == read_only(seq.init_kind);

  // per-sequence init control
  enable : bool;
    keep soft enable == read_only(driver.tb_ptr.config.enable_auto_init);
};

// add each region to closest addrmap's list of regions
extend csco_addrmap_region_s {
  run() is also {
    var tb_ptr := try_enclosing_unit(csco_tb_u);

    assert(tb_ptr != NULL) else
      error("ERR_CSCO_BASE_ENV_ADDRMAP\n",
            "cannot find parent TB unit (csco_tb_u) for newly created addrmap region : ", csco_short_name());

    messagef(CSCO_BASE_ENV, MEDIUM, "Added %s to %s TB's address map\n", csco_short_name(), tb_ptr.tb_name);
    tb_ptr.addrmap.add_region(me);
  };
};

'>
