//-----------------------------------------------------------------------------
//
// Filename : csco_base_env_specman_configs.e
//   Language : 'e'
//   Author   : Mark Strickland
//
//   General Description: Sets Specman configuration defaults to more typically desired values for BaseEnv simulations (derived from csco_erm_ext/e/csco_rtp_common_inits.e)
//
//
//-----------------------------------------------------------------------------

<'

//========================================================================
// GLOBAL SETUP PARAMETERS
//========================================================================
// Setup print format, resource limits, coverage, and end of test message

extend sys {

  setup() is also {
// Set up coverage parameters
    set_config(cover, grading_formula, linear);
    set_config(cover, at_least_multiplier, 1);

// Created New Define to Deal with differences in how PMC and Cisco Test Runners work
    set_config(cover, dir, "./results");
    set_config(cover, test_name, str_split(get_symbol("PWD"), "/").top());

// Set watchdog for Max Structs to a very large #
    set_config(gen, max_structs, 5000000); 

//Set some generator optimization flags
    #ifdef SPECMAN_VERSION_4_2_1_OR_LATER {
      set_config(gen, static_analysis_opt, TRUE);
      set_config(gen, unit_reference_rule, TRUE);
    };
    #ifdef SPECMAN_VERSION_4_3_1_OR_LATER {
      set_config(gen, determinants_before_subtypes,default_off);
    };

// Set defaults for garbage collection and allowed memory limit
    set_config(memory, gc_threshold, 512M);               
    set_config(memory, gc_increment, 256M);
    set_config(memory, max_size, 3100M);
    set_config(memory, absolute_max_size, 3102M);

// Set threshold for on-the-fly garbage collection threshold
    set_config(memory, otf_threshold, 1000M); 
// Make on-the-fly garbage collection print a message
    set_config(memory, print_otf_msg, TRUE);

// Control whether printed struct references (e.g. foo-@34) are kept during garbage collection (less memory use when FALSE)
    set_config(memory, retain_printed_structs, FALSE);

    set_config(print, radix, hex, items, 8k, list_lines, 1k);

// Set Max Line Length before specman wraps output
    set_config(print, line_size, 300 );

// Make List print left to right not right to left
    set_config(print, list_starts_on_right, FALSE);

// Print Timescale in Nanseconds
    #ifdef SPECMAN_VERSION_4_0_1_OR_LATER {
      set_config(print,scale,ns);
    };

// Disable tick_max watchdog as csco_testflow has watchdogs for each phase
    set_config(run, tick_max, MAX_UINT);   

    #ifdef SPECMAN_VERSION_6_0_OR_LATER {
      set_config(simulation, enable_ports_unification, TRUE);   
    };

    // default all dut_error check effects to ERROR_CONTINUE
    // so that we can roll sim forward some more cycles before exiting
    set_check("...", ERROR_CONTINUE);

// Print current Specman configuration to the log file
    specman("show config memory");
    specman("show config gen");
    specman("show config cover");
  };

  // override name for sys, without affecting *every* message
  // (never do this anywhere else! this is a special case)
  short_name_path() : string is only { result = "SYS" };
};

'>
