// ----------------------------------------------------------------------------
//
// File name    : csco_base_env_addrmap.e
//              :
// Project      : csco_base_env
//              :
// Created On   : 05/11/2006
//              :
//              :
// Description  : create sequence for typical block reset needs
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_base_env;

// csco_tb level config flag representing whether the DUT contains the
// unitlevel reset registers (typically in a CFG block).
// if not, resets must be applied via the pins
// that would otherwise be driven by these registers
extend csco_tb_config_s {
  reset_regs_present : bool;
    keep soft reset_regs_present;
};

//------------------------------------------------------------
// define csco_addrmap sequence to apply reset via registers

extend csco_addrmap_seq_kind_t : [ APPLY_REG_RESETS ];

extend APPLY_REG_RESETS csco_addrmap_seq {
  // containers for the entry and field combinations that will be written
  !entries : list (key: entry) of ENTRY'addr_scheme csco_addrmap_trans_s;

  add(reset: csco_testflow_dut_reset_u) is {
    var reg := entries.key(reset.entry);
    if(reg == NULL) {
      reg = new;
      reg.entry = reset.entry;
      entries.add(reg);
    };
    var val := reset.field_active_value;
    if(reset.state == INACTIVE) { val = ~val };
    reg.fields.add(reg.set_field(reset.field, val));
  };

  body() @driver.clock is also {
    sync [0];
    for each (reg) in entries {
      addrmap_do WRITE entry_trans keeping {
        .entry == reg.entry;
        .fields == reg.fields;
      };
    };
  };
};

//------------------------------------------------------------
// install sequence to run whenever needed

extend csco_addrmap_driver_u {
  !reg_resets_seq : APPLY_REG_RESETS csco_addrmap_seq;
    keep reg_resets_seq.driver == me;

  apply_reg_reset : in method_port of csco_testflow_dut_reset_set_mp is instance;
    keep bind(apply_reg_reset, empty);

  apply_reg_reset(reset: csco_testflow_dut_reset_u) @sys.any is {
    if((reset.entry != NULL) &&
       (reset.driver.as_a(string) ~ "/(.+_)?REG(_.+)?/")) {
      var phase := csco_get_cur_test_phase();
      if(reg_resets_seq == NULL) {
        gen (reg_resets_seq);
        reg_resets_seq.start_sequence();
      };
      reg_resets_seq.add(reset);
      sync cycle @reg_resets_seq.ended;
      reg_resets_seq = NULL;
    };
  };
};

'>
