// ----------------------------------------------------------------------------
//
// File name    : csco_base_env_event.e
//              :
// Project      : csco_base_env
//              :
// Created On   : 02/01/2006
//              :
//              :
// Description  : instantiate csco_event_phy and csco_event in base env, and
//              : hook up all cross-references
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_base_env;

// propagate TB name to event models
extend csco_event_config_s {
  tb_name : csco_tb_name_t;
};


extend csco_event_phy_env_config_s {
  tb_name : csco_tb_name_t;
};

extend csco_event_phy_agent_config_s {
  tb_name : csco_tb_name_t;
};

extend csco_event_env_u {
  // backpointer to parent csco_tb_u unit
  tb_ptr : csco_tb_u;
  // name of parent TB
  tb_name : csco_tb_name_t;
    keep tb_name == read_only(config.tb_name);
};

extend csco_addrmap_driver_u {
  csco_event_phy_notify : in method_port of csco_event_phy_push_t is instance;
    keep bind(csco_event_phy_notify, empty);

  csco_event_phy_notify(trans: csco_event_phy_trans_s) @sys.any is {
    var event_trans := new csco_event_s with {
      //assign scope name
      .name = trans.scope;
      // default conversion assumes same name between packages
      .types = {trans.name.as_a(string).as_a(csco_event_name_t)};
    };
    csco_event_handler(event_trans);
  };
};



extend csco_tb_u {
  // instantiate event env unit
  eventmap : csco_event_env_u is instance;
    keep eventmap.tb_ptr == me;
    // FIXME this has ordering issues
    keep gen (eventmap) before (addrmap);
    keep addrmap.driver.csco_event_scope == read_only(eventmap.event_tree);

  // instantiate event monitor eVCs
  eventdetects : list of csco_event_phy_env_u is instance;
    keep gen (eventmap) before (eventdetects);

  keep for each in eventdetects {
    .evc_instance == read_only(config.eventdetects[index].evc_instance);
  };

  keep soft eventdetects.size() == config.eventdetects.size();

};

INSTANCE_CFG begin
  config_instance_name   = eventmap,
  child_config_type      = csco_event_config_s,
  parent_config_type     = csco_tb_config_s,
  parent_unit_type       = csco_tb_u,
  child_unit_type        = csco_event_env_u
end;

INSTANCE_CFG_LIST begin
  config_instance_name   = eventdetects,
  child_config_type      = csco_event_phy_env_config_s,
  parent_config_type     = csco_tb_config_s,
  parent_unit_type       = csco_tb_u,
  child_unit_type        = csco_event_phy_env_u
end;

extend csco_tb_config_s {
  keep eventmap.tb_name == read_only(tb_name);
  
  keep soft eventdetects.size() == read_only(eventmap.names.size());
  keep for each in eventdetects {
    .tb_name == read_only(tb_name);
    soft .evc_instance == read_only(eventmap.names[index]).as_a(string).as_a(csco_event_phy_evc_instance_t);
  };

};

// Propagate concept of event_scope to vsd.
extend csco_tb_seq {
  // current event context
  event_scope : csco_event_s;
    keep soft event_scope == read_only(driver.tb_ptr.eventmap.event_tree);
    keep read_only(parent_sequence is a csco_tb_seq (seq)) =>
      soft event_scope == seq.event_scope;
};

// Propagate concept of event_scope to addrmap seqs.
extend csco_addrmap_seq {
  keep read_only(parent_sequence is a csco_tb_seq (seq)) =>
    soft event_scope == seq.event_scope;
};

'>
