// ----------------------------------------------------------------------------
// File name    : csco_base_env_testflow.e
//              :
// Description  : error and end-of-test dance and messages
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2007 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_base_env;

extend csco_tb_config_s {
  // delay before calling stop_run, used by predefined csco_stop_run_hook(),
  // just so that we don't stop wave capture right at the end of the test.
  // the default behavior is to wait 100ns
  time_to_run_after_stop : time;
    keep soft time_to_run_after_stop == 100 ns;
};

extend sys {
  !csco_test_completed : bool;
  !csco_test_ending : bool;

  // setup to end test after user configured delay when error occurs
  run() is also {
    if(get_config(run, exit_on) in [error, all]) {
      set_config(run, error_command,
                 append("sys.csco_stop_run_wrapper(); ",
                        get_config(run, error_command)));
      message(CSCO_BASE_ENV, LOW, "setting 'error_commnd' run config to ", get_config(run, error_command));
    };
  };

  // test is ending, don't start TCMs
  extract() is first {
    csco_test_ending = TRUE;
  };

  // insert csco_stop_run_hook() before builtin call to stop_run()
  evc_util_check_objection_and_stop_run() @sys.any is first {
    message(NONE, "All testflow domains under sys have completed,",
            " calling csco_stop_run_hook() before calling stop_run()");
    csco_stop_run_hook();
  };

  // kick off TCM to end test (if not already ending)
  csco_stop_run_wrapper() is {
    if(!csco_test_ending) {
      csco_test_ending = TRUE;
      start csco_stop_run_delayed();
    };
  };

  // call stop_run() after time-consuming csco_stop_run_hook()
  csco_stop_run_delayed() @sys.any is {
    csco_stop_run_hook();
    stop_run();
  };

  // Allow user to insert time consuming logic prior to calling stop_run().
  // default behavior is to wait for delay configured by time_to_run_after_stop
  csco_stop_run_hook() @sys.any is {
    message(NONE, "csco_stop_run_hook() waiting ",
            top_tb.config.time_to_run_after_stop,
            " before calling stop_run()");
    wait delay(top_tb.config.time_to_run_after_stop) @sim;
  };
};

extend global {
  // record final objection check
  stop_run() is first {
    message(NONE, "");
    sys.csco_testflow.check_current_objections();
    // NB this check must be done here! (not from finalize_test)
    sys.csco_test_completed = (sys.get_objection_total(TEST_DONE) == 0);
  };

  // print end-of-test messages
  finalize_test() is also {
    csco_test_end_message();
    csco_print_cps();
  };

  // print the sim performance in 1ns "cycles" per second.
  // where a "cycle" is hardcoded to 1ns for consistency
  csco_print_cps() is {
    var clks := sys.time / 1 ns;
    var kcps := clks * 1000 / (session.user_time + session.system_time);

    message(NONE, "");
    message(NONE, "======================================================");
    messagef(NONE, "%d secs USER CPU time expired during sim\n",
             session.user_time);
    messagef(NONE, "%d secs  SYS CPU time expired during sim\n",
             session.system_time);
    messagef(NONE, "%d 1ns clock cycles expired during the simulation", clks);
    messagef(NONE, "Sim performance: %d.%03d cps\n",
             (kcps / 1_000), (kcps % 1_000));
    message(NONE, "======================================================");
  };

  // Method to print Pass/Fail for very (RTP testrunner script).
  // NB very is very sensitive to the content of these messages!
  csco_test_end_message() is {
    message(NONE, "");

    var msg : string;
    if(!sys.csco_test_completed) {
      msg = "???????? Testcase did not Complete! ????????";
    }
    else if((session.num_of_dut_errors != 0) || (session.num_of_errors != 0)) {
      msg = "!!!!!!!! Test has Failed! !!!!!!!!";
    }
    else {
      msg = "******** Test has Passed! ********";
    };
    for i from 1 to 8 {
      message(NONE, msg);
    };

    message(NONE, "");
    messagef(NONE, "Number of DUT Errors   : %d\n", session.num_of_dut_errors);
    messagef(NONE, "Number of DUT Warnings : %d\n", session.num_of_dut_warnings);
    messagef(NONE, "Number of User Errors  : %d\n", session.num_of_errors);
  };
};

'>
