//-----------------------------------------------------------------------------
//
//   Filename : csco_reflection_extensions_top.e
//   Language : 'e'
//   Author   : Jeff Brown & Aaron Kirk
//
//   Description: local extensions to the rf_manager
//                to fix broken API and add missing features
//
//-----------------------------------------------------------------------------
<'
// provide access to specman internal fields
package reflection;

//------------------------------------------------------------
// module import hierarchy maintenance.
// note: only the first importing module is available,
// so true dependency graph cannot be obtained from rf

extend rf_module {
  csco_get_importing_module() : rf_module is {
    var plen := str_len(m_.parent);
    if(plen > 0) {
      assert(plen > 2);
      // remove ".e" trash to get module name
      var pname := str_sub(m_.parent, 0, plen - 2);
      result = rf_manager.get_module_by_name(pname);
    };
  };

  csco_get_all_imported_modules() : list of rf_module is {
    var umodules : list of rf_module = rf_manager.get_user_modules();
    var ind := get_index() - umodules[0].get_index();
    if ind > 0 {
      result = umodules[..ind-1];
    };
  };

  csco_get_imported_modules() : list of rf_module is {
    result = csco_get_all_imported_modules().all(.m_.parent == append(m_.name, ".e"));
  };
};

extend rf_manager {
  csco_get_toplevel_modules() : list of rf_module is {
    var umodules : list of rf_module = rf_manager.get_user_modules();
    result = umodules.all(.m_.parent == "");
  };
};

extend any_unit {
  csco_get_child_units() : list of any_unit is {
    var rf_s := rf_manager.get_exact_subtype_of_instance(me);
    var rf_fields := rf_s.get_fields();
    var rf_vholder : rf_value_holder;
    var lsize : uint;
    var rf_elem_vholder : rf_value_holder;
    var uptr : any_unit;

    for each (rf_f) in rf_fields {
      if rf_f.is_unit_instance() {
        rf_vholder = rf_f.get_value(me);
        if rf_vholder.get_type() is a rf_list (rf_l) {
          lsize = rf_manager.get_list_size(rf_vholder);
          if lsize > 0 {
            for i from 0 to lsize - 1 {
              rf_elem_vholder = rf_manager.get_list_element(rf_vholder, i);
              uptr = rf_elem_vholder.get_value().unsafe();
              result.add(uptr);
            };
          };
        }
        else {
          uptr = rf_f.get_value_unsafe(me).unsafe();
          if uptr != NULL {
            result.add(uptr);
          };
        };
      };
    };
  };

  csco_get_all_child_units() : list of any_unit is {
    result = rf_manager.get_all_unit_instances(me);
  };
};


//------------------------------------------------------------
// missing value accessors
// FIXME these can probably go away based on beta docs

extend rf_value_holder {
  csco_set_type(val: rf_type) is {
    reflection_imp___t = val;
  };

  csco_get_int_value() : int is {
    result = me.as_a(rf_int_holder).reflection_imp___v;
  };
  csco_set_int_value(val: int) is {
    me.as_a(rf_int_holder).reflection_imp___v = val;
  };

  csco_get_string_value() : string is {
    result = me.as_a(rf_string_holder).reflection_imp___v;
  };
  csco_set_string_value(val: string) is {
    me.as_a(rf_string_holder).reflection_imp___v = val;
  };

  csco_get_struct_value() : base_struct is {
    result = me.as_a(rf_struct_holder).reflection_imp___v;
  };
  csco_set_struct_value(val: base_struct) is {
    me.as_a(rf_struct_holder).reflection_imp___v = val;
  };

  csco_get_long_value() : list of bit is {
    result = me.as_a(rf_struct_holder).reflection_imp___v.as_a(sn_vec).bytes;
  };
  csco_set_long_value(val: list of bit) is {
    me.as_a(rf_struct_holder).reflection_imp___v.as_a(sn_vec).bytes = val;
  };

  csco_get_list_value() : list of int is {
    result = me.as_a(rf_list_holder).reflection_imp___v;
  };
  csco_set_list_value(val: list of int) is {
    me.as_a(rf_list_holder).reflection_imp___v = val;
  };
};

//------------------------------------------------------------
// additional type information

extend rf_type {
  // TRUE = type has explicit declaration using type/struct/unit/extend/when
  // FALSE = type was never declared
  csco_is_explicitly_defined() : bool is empty;
};

extend rf_struct {
  // obtain layers local to the current struct/subtype
  csco_get_my_layers() : list of rf_struct_layer is undefined;
};

extend rf_like_struct {
  csco_get_my_layers() : list of rf_struct_layer is {
    var name := get_name();
    result = get_layers().all(.get_defined_entity().get_name() == name);
  };

  csco_is_explicitly_defined() : bool is {
    result = !sd_.layers.is_empty();
    assert(result);
  };
};

extend rf_when_subtype {
  csco_get_my_layers() : list of rf_struct_layer is {
    result = std_.layers.rf_object_.as_a(list of rf_struct_layer).all(.get_defined_entity() == me);
  };

  csco_is_explicitly_defined() : bool is {
    result = !std_.layers.is_empty();
  };
};

extend rf_numeric {
  csco_is_explicitly_defined() : bool is {
    result = !td_.layers.is_empty();
  };
};

extend rf_enum {
  csco_is_explicitly_defined() : bool is {
    result = !get_layers().is_empty();
  };
};

'>
