#!/usr/bin/perl -w
#-----------------------------------------------------------------------------
# filename: covgen.pl
# authors: spadix, lihuang
#
# description:
#   "poor man's e-code coverage" trace generation script
#
# this script and covlog.pl together implement simple, user-driven,
# non-scalable e-code coverage, which can be marginally useful for
# collecting coverage on algorithm intensive code with many code paths.
# to use these scripts:
#   - mark code paths of interest w/descriptive comments ending with the
#     tag string "(TRACE)"
#   - run covgen.pl, specifying the affected source file(s)
#   - source the generated trace.ecom from various tests or a regression
#     this will (hopefully) pollute the logfiles with lots of trace spew
#   - run covlog.pl on these logfiles to dump the collected coverage
#
#----------------------------------------------------------------------------
#>>>>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#----------------------------------------------------------------------------
# Copyright 2002-2005 (c) Cisco Systems Inc.
#----------------------------------------------------------------------------
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License
# at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is "Cisco generic block-based memory model".
# The Initial Developer of the Original Code is "Lisa Huang and Jeff Brown".
# Portions created by the Initial Developer are Copyright (C) 2002-2005
# Cisco Systems Inc. All Rights Reserved.
# Contributor(s): Lisa Huang and Jeff Brown
#----------------------------------------------------------------------------

use strict;
use IO::File;

open(ECOM, ">trace.ecom") || die;
open(RC, ">trace.rc") || die;

# - parses comments ending w/"(TRACE)" from specified e source files
# - dumps a trace.ecom file to be sourced during a test/regression run
# - dumps line#/comment database for subsequent logfile parsing by covlog.pl

RC->print("%trace = (\n");

foreach my $src (@ARGV) {
  my $grep = "egrep -n '\\\(TRACE\\\)\$' `sn_which.sh $src`";
  $src =~ /([^.\/]+)\.e$/;
  my $mod = $1;
  foreach (`$grep`) {
    /(\d+):.*?\/\/\s*(.*)/;
    my ($ln, $com) = ($1, $2);
    $com =~ s/\s*\(TRACE\)//;
    ECOM->print("trace line $ln \@$mod;\n");
    RC->print("  '$ln\@$mod' => '$com',\n");
  }
}

RC->print(");\n");

close(ECOM);
close(RC);
