#!/usr/bin/perl -w
#-----------------------------------------------------------------------------
# filename: covlog.pl
# authors: spadix, lihuang
#
# description:
#   "poor man's e-code coverage" trace generation script
#
# this script and covgen.pl together implement simple, user-driven,
# non-scalable e-code coverage, which can be marginally useful for
# collecting coverage on algorithm intensive code with many code paths.
# to use these scripts:
#   - mark code paths of interest w/descriptive comments ending with the
#     tag string "(TRACE)"
#   - run covgen.pl, specifying the affected source file(s)
#   - source the generated trace.ecom from various tests or a regression
#     this will (hopefully) pollute the logfiles with lots of trace spew
#   - run covlog.pl on these logfiles to dump the collected coverage
#
#----------------------------------------------------------------------------
#>>>>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#----------------------------------------------------------------------------
# Copyright 2002-2005 (c) Cisco Systems Inc.
#----------------------------------------------------------------------------
# The contents of this file are subject to the Mozilla Public License
# Version 1.1 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License
# at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is "Cisco generic block-based memory model".
# The Initial Developer of the Original Code is "Lisa Huang and Jeff Brown".
# Portions created by the Initial Developer are Copyright (C) 2002-2005
# Cisco Systems Inc. All Rights Reserved.
# Contributor(s): Lisa Huang and Jeff Brown
#----------------------------------------------------------------------------

use strict;
use vars qw(%trace %cov);

my $log = $ARGV[0] || die("specify logfile");

do "trace.rc" || die;

%cov = ();

# parse logfile lines matching
# -> 0 line  ->  MEM csco_blk_memory-@0.find_blk() at line 290 in @csco_blk_memory_top

open(LOG, "<$log");
while(<LOG>) {
  /\-\>\s*\d+\s*line\s*\-\>/ || next;
  /\-\>\s*\d+\s*line\s*\-\>.*?at line (\d+) in \@(\S+)/ || die;
  my ($ln, $mod) = ($1, $2);
  $cov{"$ln\@$mod"}++;
}
close(LOG);

foreach (sort(keys(%trace))) {
  /(\d+)(\@.*)/ || die;
  my ($ln, $mod) = ($1, $2);
  $cov{$_} ||= 0;
  printf("  %6d    line %4d $mod\t// $trace{$_}\n", $cov{$_}, $ln);
}

foreach (keys(%cov)) {
  print("unrecognized trace: line ", $_, "\n")
    if(!exists($trace{$_}));
}
