//-----------------------------------------------------------------------------
// filename: csco_blk_memory_top.e
//
// description:
//   very simple standalone example of each csco_blk_memory API method
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// Copyright 2002-2005 (c) Cisco Systems Inc.
//----------------------------------------------------------------------------
// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "Cisco generic block-based memory model".
// Portions created by the Initial Developer are Copyright (C) 2002-2005
// Cisco Systems Inc. All Rights Reserved.
//----------------------------------------------------------------------------
<'
// include the csco_blk_memory package
import csco_blk_memory/e/csco_blk_memory_top;

// define a new name for the example memory (optional)
extend csco_blk_memory_name_t : [ SAMPLE_MEMORY ];

extend sys {
  //----------------------------------------
  // example memory instance
  sample_memory : SAMPLE_MEMORY'name csco_blk_memory;

    // use a random value for uninitialized accesses
    keep sample_memory.default_value.reset_soft();

  //----------------------------------------
  // random values for a sample region/block access
  region_address : csco_blk_memory_addr_t;
  region_data : list of byte;
    keep soft region_data.size() in [0x02..0x40];

  // random values for a sample byte access
  byte_address : csco_blk_memory_addr_t;
    // avoid address collisions (extremely unlikely)
    keep ((byte_address < region_address) or
          (byte_address >= (region_address + region_data.size())));
    keep gen (region_address, region_data) before (byte_address);
  byte_data : byte;

  // random address for uninitialized read
  uninitialized_address : csco_blk_memory_addr_t;

  free_address : csco_blk_memory_addr_t;
    keep free_address >= region_address;
    keep free_address < (region_address + region_data.size());

  //----------------------------------------
  // execute some test accesses
  run() is also {
    // dump initial contents
    message(MEDIUM, "initial sample_memory contents:") {
      sample_memory.print();
    };

    // do some writes
    message(MEDIUM, "byte write:");
    sample_memory.write_byte(byte_address, byte_data);
    message(MEDIUM, "region write:");
    sample_memory.write(region_address, region_data);

    // dump current contents
    message(MEDIUM, "sample_memory contents after writes:") {
      sample_memory.print();
    };

    // show allocated regions
    message(MEDIUM, "allocated regions:");
    print sample_memory.get_alloc_region(0, 0);

    // check allocation
    message(MEDIUM, "byte check:");
    print sample_memory.check_alloc(byte_address);
    message(MEDIUM, "region check:");
    print sample_memory.check_alloc_region(region_address, region_data.size());
    message(MEDIUM, "uninitialized check:");
    print sample_memory.check_alloc_region(uninitialized_address, 0x60);

    // read back values
    message(MEDIUM, "byte read:");
    print sample_memory.read_byte(byte_address);
    message(MEDIUM, "region read:");
    print sample_memory.read(region_address, region_data.size());

    // uninitialized read (some overlap may occur...)
    message(MEDIUM, "uninitialized read:");
    print sample_memory.read(uninitialized_address, 0x60);

    // free one of the region bytes
    message(MEDIUM, "byte free:");
    sample_memory.free(free_address);
    message(MEDIUM, "sample_memory contents after free:") {
      sample_memory.print();
    };

    // memory wipe clean
    sample_memory.clear();
    message(MEDIUM, "sample_memory contents after clear:") {
      sample_memory.print();
    };
  };

  //----------------------------------------
  // show all accesses for this test
  keep logger.verbosity == FULL;
  keep logger.tags == { NORMAL; CSCO_BLK_MEMORY };
  short_name() : string is only { result = "EXAMPLE" };

  // more useful print options
  setup() is also {
    set_config(print, radix, HEX, list_index_radix, HEX);
    set_config(print, list_starts_on_right, FALSE);
  };
};

'>
