//-----------------------------------------------------------------------------
// filename: flat_mem.e
//
// description:
//   simple wrapper to make an array look like a memory
//   allows an address offset to the start of implemented memory
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// Copyright 2002-2005 (c) Cisco Systems Inc.
//----------------------------------------------------------------------------
// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "Cisco generic block-based memory model".
// Portions created by the Initial Developer are Copyright (C) 2002-2005
// Cisco Systems Inc. All Rights Reserved.
//----------------------------------------------------------------------------
<'

type flat_mem_addr_t : uint(bits: 64);
type flat_mem_name_t : [ MEM ];

// flat_mem is a very simple write/read interface to a list of bytes
// size is fixed at gen time
struct flat_mem {
  name : flat_mem_name_t;
    keep soft name == MEM;

  // byte size of memory
  size : flat_mem_addr_t;
    keep soft size == 0;

  // offset of first memory address for easy relocation
  offset : flat_mem_addr_t;
    keep soft offset == 0;

  default_value : byte;
    keep soft default_value == 0;

  // memory implementation
  locations : list of byte;
    keep soft locations.size() == value(size);
    keep for each in locations {
      it == value(default_value);
    };

  // basic write/read access
  write(addr: flat_mem_addr_t, data: list of byte) is {
    var start_offset := addr - offset;
    locations[start_offset..(start_offset + data.size() - 1)] = data;
  };

  read(addr: flat_mem_addr_t, size: flat_mem_addr_t) : list of byte is {
    var start_offset := addr - offset;
    result = locations[start_offset..(start_offset + size - 1)];
  };

  // single byte access
  write_byte(addr: flat_mem_addr_t, data: byte) is {
    locations[addr - offset] = data;
  };

  read_byte(addr: flat_mem_addr_t) : byte is {
    result = locations[addr - offset];
  };

  // check whether byte exists in memory
  byte_exists(addr: flat_mem_addr_t) : bool is {
    result = (addr >= offset) && (addr < offset + locations.size());
  };

  // clear entire contents of memory
  clear() is {
    locations.resize(locations.size());
  };
};

'>
