(also added a name and changed to a struct)

/*------------------------------------------------------------------------- 
File name   : shr_ram_main.e
Title       : Generic RAM model utility (shareware)
Project     : shr_ram utility
Created     : 30-Nov-2001
Description : Provide a generic RAM model. This is implemented as a simple
            : sparse byte-oriented RAM.
Notes       : 
--------------------------------------------------------------------------- 
Copyright 2002 (c) Verisity Design 
-------------------------------------------------------------------------*/ 

<'

type shr_ram_addr_t : uint(bits: 64);
type shr_ram_name_t : [ MEM ];

-- shr_ram_location_s represents a single memory location
struct shr_ram_location_s {
    address : shr_ram_addr_t;
    value   : byte;
}; -- struct shr_ram_location_s


-- shr_ram_env_u is a RAM implemented as a sparse byte-wide memory
struct shr_ram {
    name : shr_ram_name_t;
      keep soft name == MEM;

    -- keyed list of locations in the RAM
    !locations : list(key : address) of shr_ram_location_s;

    
    -- write a single byte to the RAM
    write_byte(addr : shr_ram_addr_t, data : byte) is {
        if locations.key_exists(addr) {
            locations.key(addr).value = data;
        } else {
            var temp : shr_ram_location_s = new;
            temp.address = addr;
            temp.value = data;
            locations.add(temp);
        };
    }; -- write_byte()
    

    -- check whether a byte already exists in the RAM
    byte_exists(addr : shr_ram_addr_t) : bool is {
        result = locations.key_exists(addr);
    }; -- byte_exists()


    -- read a single byte from the RAM
    read_byte(addr : shr_ram_addr_t) : byte is {
        if locations.key_exists(addr) then {
            result = locations.key(addr).value 
        } else {
            result = 0; -- higher levels of code should use byte_exists() method to check for this posibility
        };
    }; -- read_byte()
    
    -- write multiple bytes to the RAM
    write(addr : shr_ram_addr_t, data : list of byte) is {
        for each (d) in data do {
            write_byte(addr + index, d);
        };
    }; -- write()
    
    -- zero multiple bytes in the RAM
    zero(addr : shr_ram_addr_t, size : shr_ram_addr_t) is {
        for i from 0 to (size - 1) {
            write_byte(addr + i, 0);
        };
    }; -- zero()

    -- read multiple bytes from the RAM
    read(addr : shr_ram_addr_t, num_bytes : shr_ram_addr_t) : list of byte is {
        for i from 0 to (num_bytes - 1) {
            result.add(read_byte(addr + i));
        };
    }; -- read()
    
    -- clear entire contents of the RAM
    clear() is {
        locations.clear();
    }; -- clear()
        
};

'>
