//-----------------------------------------------------------------------------
// filename: test_cases.e
//
// description:
//   very directed test targeting specific corner cases
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// Copyright 2002-2005 (c) Cisco Systems Inc.
//----------------------------------------------------------------------------
// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "Cisco generic block-based memory model".
// Portions created by the Initial Developer are Copyright (C) 2002-2005
// Cisco Systems Inc. All Rights Reserved.
//----------------------------------------------------------------------------
<'
package csco_blk_memory;
define CSCO_BLK_MEMORY_DEBUG_ASSERTIONS;
import csco_blk_memory/tests/testbench;

extend sys {
  keep !enable_hash_mem;
  keep !enable_flat_mem;

  send_loop() @clk is only {
    var expect : list of byte;

    out("------------------------------------------------------------");
    out("discontiguous byte writes");
    expect.resize(0x21, TRUE, blk_mem.default_value, FALSE);
    for i from 0x20 to 0x40 step 2 {
      blk_mem.write_byte(i, i);
      expect[i - 0x20] = i;
    };
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("merge byte regions w/no overlap");
    blk_mem.write(0x22, (from 0x22 to 0x26).as_a(list of byte));
    expect[0x02..0x06] = (from 0x22 to 0x26).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("merge byte regions w/start adjacency");
    blk_mem.write(0x29, (from 0x29 to 0x2c).as_a(list of byte));
    expect[0x09..0x0c] = (from 0x29 to 0x2c).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("merge byte regions w/end adjacency");
    blk_mem.write(0x30, (from 0x30 to 0x35).as_a(list of byte));
    expect[0x10..0x15] = (from 0x30 to 0x35).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("merge byte regions w/double adjacency");
    blk_mem.write(0x39, (from 0x39 to 0x3d).as_a(list of byte));
    expect[0x19..0x1d] = (from 0x39 to 0x3d).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);
    
    out("------------------------------------------------------------");
    out("merge arbitrary regions w/double overhang");
    blk_mem.write(0x24, (from 0x44 to 0x51).as_a(list of byte));
    expect[0x04..0x11] = (from 0x44 to 0x51).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);
    
    out("------------------------------------------------------------");
    out("overwrite block start");
    blk_mem.write(0x22, (from 0x62 to 0x67).as_a(list of byte));
    expect[0x02..0x07] = (from 0x62 to 0x67).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("overwrite block end");
    blk_mem.write(0x3c, (from 0x1c to 0x1e).as_a(list of byte));
    expect[0x1c..0x1e] = (from 0x1c to 0x1e).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("overwrite block interior");
    blk_mem.write(0x2e, (from 0x8e to 0x94).as_a(list of byte));
    expect[0x0e..0x14] = (from 0x8e to 0x94).as_a(list of byte);
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x20, 0x21) == expect);

    out("------------------------------------------------------------");
    out("more start overlapping");
    blk_mem.write(0x1c, (from 0xcc to 0xd4).as_a(list of byte));
    expect = { (from 0xcc to 0xd4).as_a(list of byte); expect[0x05..] };
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x1c, 0x25) == expect);

    out("------------------------------------------------------------");
    out("more start overlapping");
    blk_mem.write(0x18, (from 0xd8 to 0xdf).as_a(list of byte));
    expect = { (from 0xd8 to 0xdf).as_a(list of byte); expect[0x04..] };
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.read(0x18, 0x29) == expect);

    // FIXME discontiguous random sized writes
    stop_run();
  };
};

'>
