//-----------------------------------------------------------------------------
// filename: test_cases.e
//
// description:
//   very directed test targeting specific corner cases
//
//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
// Copyright 2002-2005 (c) Cisco Systems Inc.
//----------------------------------------------------------------------------
// The contents of this file are subject to the Mozilla Public License
// Version 1.1 (the "License"); you may not use this file except in
// compliance with the License. You may obtain a copy of the License
// at http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is "Cisco generic block-based memory model".
// Portions created by the Initial Developer are Copyright (C) 2002-2005
// Cisco Systems Inc. All Rights Reserved.
//----------------------------------------------------------------------------
<'
package csco_blk_memory;
define CSCO_BLK_MEMORY_DEBUG_ASSERTIONS;
import csco_blk_memory/tests/testbench;

extend sys {
  keep !enable_hash_mem;
  keep !enable_flat_mem;

  run() is only {
    var addrs : list of int;

    out("------------------------------------------------------------");
    out("decreasing insertions");
    for i from 0 to 0x40 {
      insert_byte(0xc0 - (i << 1));
    };
    blk_mem.debug_dump_tree();
    blk_mem.print();
    assert(blk_mem.total_blks() == 0x41);
    blk_mem.clear();

    out("------------------------------------------------------------");
    out("increasing insertions");
    for i from 0 to 0x40 {
      insert_byte(0x40 + (i << 1));
    };
    blk_mem.debug_dump_tree();
    assert(blk_mem.total_blks() == 0x41);
    blk_mem.clear();

    out("------------------------------------------------------------");
    out("increasing and decreasing insertions");
    var addr0 := 0x60;
    var addr1 := 0x50;

    for i from 0 to 0x40 {
      if((i % 3) != 2) {
        insert_byte(addr0);
        addr0 += 2;
      } else {
        insert_byte(addr1);
        addr1 -= 2;
      };
    };
    blk_mem.debug_dump_tree();
    assert(blk_mem.total_blks() == 0x41);
    blk_mem.clear();

    out("------------------------------------------------------------");
    out("random insertions");
    addrs = (from 0x40 to 0xc0 step 2);
    while(!addrs.is_empty()) {
      var idx : int;
      gen (idx) keeping { it >= 0; it < addrs.size() };
      insert_byte(addrs[idx]);
      addrs.fast_delete(idx);
      blk_mem.print();
    };
    blk_mem.debug_dump_tree();
    assert(blk_mem.total_blks() == 0x41);
    blk_mem.clear();

    out("------------------------------------------------------------");
    out("extended random insertions");
    addrs = (from 0x10 to 0xf8 step 2);
    while(!addrs.is_empty()) {
      var idx : int;
      gen (idx) keeping { it >= 0; it < addrs.size() };
      insert_byte(addrs[idx]);
      addrs.fast_delete(idx);
    };
    assert(blk_mem.total_blks() == 0x75);

    out("------------------------------------------------------------");
    out("deletions");
    addrs = (from 0x10 to 0xf8 step 2);
    while(!addrs.is_empty()) {
      var idx : int;
      gen (idx) keeping { it >= 0; it < addrs.size() };
      outf("deleting byte @%02x\n", addrs[idx]);
      var blk := blk_mem.find_blk(addrs[idx]);
      assert(blk != NULL);
      blk_mem.delete(blk);
      addrs.fast_delete(idx);
    };

    // should be empty now...
    assert(blk_mem.root == NULL);
  };

  insert_byte(addr: csco_blk_memory_addr_t) is {
    outf("inserting byte @%02x\n", addr);
    assert(blk_mem.find_blk(addr) == NULL);
    blk_mem.insert(new csco_blk_memory_blk with {
      .start_addr = addr;
      .end_addr = addr;
      .data = { addr };
    });
  };
};

'>
