// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_clkgen_unit_dev_testbench_coverage.e
//              :
// Title        : Example coverage file for csco_clkgen modules used in unit
//                level tetbench example.
//
// Project      : na
//              :
// Created On   : 02/04/2003
//              :
//              :
// Purpose      : Define the specific coverage items for the two instances of
//              : the clock generator used in this module.
//              :
// Description  : Example of coverage on csco_clkgen config structs
//              :
//              :
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'

// Add Coverage for both our clock gen blocks


extend cpu_clk_gen'name csco_clkgen_config_s {
  //
  // Since everything is random on this clock we define a fair#
  // of coverage metrics
  //
  cover csco_clkgen_config_generated(name==cpu_clk_gen) using also text = "cpu_clk_gen coverage group" is also {
    item period     using ranges = {range([4900..5100],"",5)}, at_least = 1;
    item duty_cycle using ranges = {range([41..59],"",1)}, at_least = 1;
    item skew       using ranges = {range([0..5100],"",100)}, at_least = 1;
    item inv_phase;
    item jitter     using ranges = {range([0..500],"",50)}, at_least = 1;

    cross period, duty_cycle;
    cross period, jitter;

    item inst_name;
    item config_mode using ignore = (config_mode not in [DEFAULT,NO_JITTER]); // Ignore any modes not for this clock gen

    cross config_mode, inst_name;                          // Now cross mode with instance
  };
};

extend core_clk_gen'name csco_clkgen_config_s {

  // The core clk is basically fixed with only jutter being random
  // so for this instance we will just cover jitter
  //
  cover csco_clkgen_config_generated(name==core_clk_gen) using also text = "core_clk_gen coverage group" is also {
    item jitter     using ranges = {range([0..1000],"",50)}, at_least = 1;

    item inst_name;
    item config_mode using ignore = (config_mode not in [DEFAULT]);

    cross config_mode, inst_name;                          // Now Cross mode with instance
  };

};                                                         // End Extend

'>
