//==============================================================================
//
//   Filename : csco_clkgen_tb_2.v
//   Author   : Sean Smith
//   username : sesmith
//
//   General Description: Sample Verilog testbench for use with Very and Specman
//
//   History:
//     01/25/012 SWS    Initial template
//==============================================================================

`timescale 1ns/1ns

module csco_clkgen_tb_2();


`define OVL_ERROR_COUNTER csco_clkgen_tn_2.ovl_error_count

// Counter to Track Denali Errors
//
integer denali_error_count;

// Counter to track OVL Errors
//
integer ovl_error_count;


// Load The Specman Code and Include any necessary Wave Trace Files
//
// Waves control
//------------------------------------------------------------------------

initial
  begin
    $timeformat(-9,0," ns",12);
    denali_error_count = 0;
    ovl_error_count = 0;
    $sn("--");

    if ($test$plusargs("vcd")) begin
      `include "probes.vcd"
    end
    if ($test$plusargs("simvision")) begin
      `include "probes.simvision"
    end
    if ($test$plusargs("debussy")) begin
      `include "probes.debussy"
    end
    if ($test$plusargs("memdebug")) begin
      $system("puredata &");
    end
end

//
// Control SDF annotation
//

`ifdef GATES
  initial
     begin
       if ($test$plusargs("SDF_MIN"))
        begin
          $display("%t ns (%m) Back Annotating SDF File with Min Delays",$time);
          $sdf_annotate("/vob/ion/fpga/top/pion_top.sdf.gz", testbench.iPROTON.iPION );
          $display("%t ns (%m) Back Annotation Completed",$time);
        end

      if ($test$plusargs("SDF_NOM"))
        begin
          $display("%t ns (%m) Back Annotating SDF File with Nom Delays",$time);
          $sdf_annotate("/vob/ion/fpga/top/pion_top.sdf.gz", testbench.iPROTON.iPION );
          $display("%t ns (%m) Back Annotation Completed",$time);
        end

      if ($test$plusargs("SDF_MAX"))
        begin
          $display("%t ns (%m) Back Annotating SDF File with Max Delays",$time);
          $sdf_annotate("/vob/ion/fpga/top/pion_top.sdf.gz", testbench.iPROTON.iPION );
          $display("%t ns (%m) Back Annotation Completed",$time);
        end
      end
`endif

  //**********************************************************************************
  // Clock control
  //**********************************************************************************


  wire pci_clk_gen;

  csco_clkgen ipci_clk_gen (.clk(pci_clk_gen), ._en(1'b0));



`ifdef GATES
  initial
    $display("`define GATES has been asserted! Running Gate Level Simulation.");
`else
`endif


  //**********************************************************************************
  //* Instance the DUT
  //**********************************************************************************


  csco_clkgen_dut_2 dut_2 (
                     .clk(pci_clk_gen)
                    );


// Include any user defined things
// DON'T MOVE THIS. It needs to be at the bottom in order not to cause unncessary recompilation
// from NCSIM
//
`include "usertest.v"
endmodule
