package RDLEvents;
use warnings;
use strict;
use RDLPreGenerator;
use RDLParser;
use UNIVERSAL qw(isa);
our @ISA = qw(RDLPreGenerator RDLParser);

sub initialize {
  my ($me) = @_;
  $me->SUPER::initialize();
  $me->{handler}{eventlayer} = \&RDLPreGenerator::comment_out;
  $me->{handler}{event} = \&RDLPreGenerator::comment_out;
  $me->{handler}{addrmap} = \&addrmap;
  #$me->{handler}{END} = \&endmap;
}

sub endmap {
  print "\ncalled end of map\n";
};

my @blocks_procd = {};
my %block_event_map = ();
my $dbg = "TRUE"; 

sub addrmap {
  my ($me, $comp, $content) = @_;

  #$comp->user_dump();
  my $type = $comp->{typdef};
  my $map = $comp->{typedef} || return (undef);
  my $block = $map->{id} || return (undef);
 
  #check if this is the type definition
  if ( grep { $_ eq $block } @blocks_procd) {
    return(undef);
  };
  #add block to defined list of blocks
  push @blocks_procd, $block;

  print "RDLEvents: Checking for rdl event syntax extensions in block $block...\n";
  
  my $eventlist = RDLEvents->new ({
      parent => '',
      inst   => '',
      kind   => "top",
      events => [] 
    });
    
  my $event_top = RDLEvents->new ({
      parent => '',
      inst   => '',
      kind   => "ev_mux",
      events => [] 
    });

  my $top = "FALSE";

  
  my @orphan_events;
  foreach my $inst (@{$map->{instances}}) {
    if ($inst->{kind} =~ m[^(eventlayer|eventgroup)]) {
        # expand instance tree (unroll) and
        # propagate properties to leaf instances
        $inst->expand_instances();
        # whatever processing
        #$inst->user_dump();
        push (@{$eventlist->{events}}, eventhier($inst));
    } elsif ($inst->{kind} =~ m[^(event)]) {
        push(@orphan_events, convert_to_event($inst));
    } elsif ($inst->{kind} =~ m[^addrmap]) {
      my $block_type = "$inst->{typedef}->{id}";
      $top = "TRUE";
      print "RDLEvents: Toplevel instance of $inst->{id} in $block\n";
      my $range_limit = 0;
      if (defined($inst->{range}[0])) {
        $range_limit = $inst->{range}[0];
      } else {
        $range_limit = 1;
      };
       
      for (my $range = 0; $range < $range_limit; $range++) {
        my $event =  RDLEvents->new ({
          parent => '',
          inst   => $inst->{id},
          inst_idx => $range,
          inst_max => $range_limit,
          kind   => 'summary',
          events => []
        });
        #$event->{inst} = $inst->{id} . "[$range]" if ($range_limit > 1);

        #append the event tree from each block to the top level event
        foreach my $ev_blk ( @{$block_event_map{$block_type}->{events}}) {
          push (@{$event->{events}}, $ev_blk);
        };
        
        push (@{$event_top->{events}}, $event);
        #$event_top->dump_event_tree();
      };
    };
  };  
  #if there are any orphan events create a layer and add it ->
  if (@orphan_events > 0) {
    my $ev_block = create_summary($block); 
    foreach my $ev (@orphan_events) {
      push (@{$ev_block->{events}}, $ev);
    };
    push (@{$eventlist->{events}}, $ev_block);
  };
  #$eventlist->dump_event_tree();
      
  if ($top eq "TRUE") {
      print "RDLEvents: Executing actions for toplevel addrmap $block\n";
      $event_top->dump_event_tree() if ($dbg eq "TRUE");
      dump_top_meta_data($block, "ev_mux" ,$event_top);  
      my $regdefs = top_hier("ev_mux_registers", $event_top);
      my $meta_file_name = $top . "events.xml";
      print "Dumping top level ev_mux rdl to $ENV{PROJ_VOB}/asic/ev_mux/rtl/ev_mux_registers.rdl\n";
      if (!(-e "$ENV{PROJ_VOB}/asic/ev_mux")) {
        die "RDLEvent ERROR: $ENV{PROJ_VOB}/asic/ev_mux directory not created"
      };
      open(EVMUXFILE, ">$ENV{PROJ_VOB}/asic/ev_mux/rtl/ev_mux_registers.rdl");
      print EVMUXFILE "`include \"$ENV{PROJ_VOB}/asic/common/rtl/events.rdlh\"";
      print EVMUXFILE $regdefs;
      close(EVMUXFILE);
      $regdefs .= "\n    ev_mux_registers ev_mux_registers;\n";
      substr($content, $map->{start} - $comp->{start} + 1, 0) = $regdefs;
      return ($content);
  } else {
    #$eventlist->dump_event_tree();
    $block_event_map{$block} = $eventlist;    
    #expand $event in to macro calls and return as a string
    my $regdefs = expand_hier($block, $eventlist); 
    #print "RDLEvents content $content\n";
    if (defined $regdefs) {
      substr($content, $map->{start} - $comp->{start} + 1, 0) = $regdefs;
      #print "RDLEvents content $regdefs\n";
      #If this is the first time event syntax is defined, read the
      #default events file in 
      if ($#blocks_procd eq 1) {
        my $ev_file = "$ENV{PROJ_VOB}/asic/common/rtl/events.rdlh";
        open(BASE_EVFILE, "$ev_file");
        my $ev_content = "/*Base events content from $ev_file*/\n";
        while (<BASE_EVFILE>) {
          $_ =~ s/
//g;
          $ev_content .= $_;
        };
        $content = $ev_content . $content;
      };
      return ($content);
    } else {
      return(undef);
    };
  };
};

sub top_hier {
  my ($block, $eventlist) = @_;

  my $summary_size = @{$eventlist->{events}} - 1; #FIXME why -1?
  my $summary_holder = "
    blockname = $block,
    register=$block,
    size=$summary_size,
    ";
    
  foreach my $event ( @{$eventlist->{events}}) {
    $summary_holder .= "
    event=$event->{inst}$event->{inst_idx}," if ($event->{inst_max} > 1);
    $summary_holder .= "
    event=$event->{inst}," if ($event->{inst_max} eq 1);
  };
    
  my $output_string = RDLEvents->gen_summary_reg($summary_holder);
  
  $output_string = "
  //Define blueprint event handler master
  addrmap $block {
    $output_string
    };
  ";
  return($output_string);
};
  
sub expand_hier {
  my ($block, $eventlist) = @_;

  my $leaf_holder = "";
  my $summary_holder = "";
  my $leaf_summary_holder = "";
  my $output_string;
  #$eventlist->user_dump();

  my $summary_size = @{$eventlist->{events}};
    
  if ($summary_size > 32) {
    print "Cannot support > 32 event registers in a layer";
    exit(1);
  };

  $summary_holder .= "
    blockname=$block,
    register=$block,
    size=$summary_size,
    ";

  $leaf_summary_holder .= "blockname=$block"; 
  
  my $multiple_regs = 0;
  
  foreach my $eventlayer ( @{$eventlist->{events}}) {

    #print "Event layer is $eventlayer->{id} \n";
    my $bitname = $eventlayer->{id};

    $multiple_regs = 1 if ( $eventlayer->{range} > 1);   
    my $range_limit = $eventlayer->{range} || 0;
    for (my $range = 0; $range < $range_limit; $range++) {
      my $size = @{$eventlayer->{events}};
      $bitname = $eventlayer->{id};
      $leaf_holder = "
        blockname=$block,";
      $leaf_holder .= "
        register=$bitname";

      if ($range_limit > 1) {   
        $leaf_holder .= "_$range,";
      } else {
        $leaf_holder .= ",";
      };
        
      $leaf_holder .= "
        size=$size,";
    
      foreach my $ev (@{$eventlayer->{events}}) {

        #if (!defined(  || ) the error FIXME
        $leaf_holder .= "
        event=$ev->{id}, 
        desc=Place holder for description, 
        severity=$ev->{severity}, 
        action=$ev->{action},";
        if (defined($ev->{offset}) ) {
          $leaf_holder .= "
        offset=$ev->{offset},";
        };
      };
    
      $summary_holder .= "
      event=$eventlayer->{id}_$range," if ($range_limit > 1);

      $summary_holder .= "
      event=$eventlayer->{id}," if ($range_limit == 1);
      
      $leaf_summary_holder .= ",$bitname\_$range" if ($range_limit > 1);
      $leaf_summary_holder .= ",$bitname" if ($range_limit == 1);

      $output_string .= RDLEvents->gen_leaf_reg($leaf_holder);
    };
  };
  #create a hierarchical layer only if there are multiple leaf registers
  if ($multiple_regs eq 1) {
    $output_string .= RDLEvents->gen_summary_reg($summary_holder);
    $output_string .= RDLEvents->gen_leaf_summary_connect($leaf_summary_holder);
  };
    
  return($output_string);

};

sub eventhier {
  my ($node, $ind) = @_;
  $ind ||= '    ';
  my $event;

  # error checking as desired: correct nesting, homogeneity, etc...

  # this version just dumps the tree to stdout
  #print("Print here: $ind$node->{kind}: $node->{id}\n");

  if($node->{kind} =~ /eventlayer/) {
    #create an event layer object
    $event = RDLEvents->new ({
      parent => '',
      inst   => '',
      kind   => "summary",
      handler => '',
      id     => $node->{id},
    });

    if (defined ($node->{range}[0])) {
      $event->{range} = $node->{range}[0];
    } else {
      $event->{range} = 1;
    };

    foreach my $inst (@{$node->{instances}}) {
      #unroll the event groups
      evgrp_unroll($inst, $event);
    };
          
    #$event contains the tree of events in this address map
    #Need to convert this tree in to macro calls that will instance
    #the interrupt hierarchy

  }
  return($event);
}

sub create_summary {
  my ($blockname) = @_;
  my $event;

  $event = RDLEvents->new ({
    parent => '',
    inst   => '',
    range  => '1',
    kind   => "summary",
    handler => '',
    id     => $blockname,
    events => []
  });

  return($event);
}

sub convert_to_event {
  my ($node, $ind) = @_;
  $ind ||= '    ';
  my $event;
  #create an event layer object
  $event = RDLEvents->new ({
    parent => '',
    inst   => '',
    kind   => "leaf",
    handler => '',
    id     => $node->{id},
  });

  if (defined ($node->{range}[0])) {
    $event->{range} = $node->{range}[0];
  } else {
    $event->{range} = 1;
  };
  foreach my $prop (@{$node->{properties}}) {
    if ($prop->{kind} =~ /severity|action|desc/) {
      $event->{$prop->{kind}} = $prop->{value};
    };
  };
  return($event);
}



sub dump_event_tree {
  my ($me, $ind, $prune) = @_;
  $ind ||= '';
  $prune ||= 0;
  if(!defined($me)) {
    print('undef');
  } elsif(!ref($me)) {
    print("'$me'");
  } elsif(ref($me) eq 'SCALAR') {
    print("\\'$$me'");
  } elsif(ref($me) eq 'ARRAY') {
    if($prune) {
      print('[ (' . scalar(@$me) . ' items) ... ]')
    } else {
      print('[ (' . scalar(@$me) . ' items)');
      if(@$me) {
	print("\n");
	my $idx = 0;
	foreach my $item (@$me) {
	  print($ind . '  [' . $idx++ . '] => ');
	  dump_event_tree($item, $ind . '  ');
	}
	print("$ind]");
      } else {
	print("]");
      }
    }
  } elsif(isa($me, __PACKAGE__)) {
    if($prune) {
      print(ref($me) . " { ... }");
    } else {
      print(ref($me) . " {\n");
      foreach my $field (sort({ref($me->{$a}) cmp ref($me->{$b})} keys(%{$me}))) {
	      print("$ind  $field => ");
	      dump_event_tree($me->{$field}, $ind . '  ',
		    $field =~ /^(typedef|content|parent|inst)$/);
      }
      print("$ind}");
    }
  } else {
    print($me);
  }
  print("\n");
}


sub evgrp_unroll {
  my ($ev_node, $ev_layer) = @_;
  my $event;
  my $prop;
  my $offset;

  if ($ev_node->{kind} eq "eventgroup") {

    #FIXME offsets
    #offset defined only for groups
    #if (defined($ev_node->{range}) ) {
    #  $offset = $ev_node->{range}[2];
    #} else {
    $offset = undef;
    #}

    foreach my $inst (@{$ev_node->{instances}}) {
    
      $event = RDLEvents->new ({
        kind   => "leaf",
        id     => "$ev_node->{id}_$inst->{id}"
        });

      #print "RDLEvents Debug evgrp_unroll offset = $offset\n";
        
      $event->{offset} = $offset;

      foreach $prop (@{$inst->{properties}}) {
        if ($prop->{kind} =~ /severity|action/) {
          $event->{$prop->{kind}} = $prop->{value};
        };
      };

      push (@{$ev_layer->{events}}, $event);
    };
  } else {
    $event = RDLEvents->new ({
        kind   => "leaf",
        id     => "$ev_node->{id}"
        });
    foreach $prop (@{$ev_node->{properties}}) {
      if ($prop->{kind} =~ /severity|action/) {
        $event->{$prop->{kind}} = $prop->{value};
      };
    };
   
    my $block = $ev_node->{block};

    foreach my $block_prop ( @{ $block->{properties} } ) {
      if ($block_prop->{kind} =~ /severity|action|desc/) {
        $event->{$block_prop->{kind}} = $block_prop->{value};
      };
    };
    
    push (@{$ev_layer->{events}}, $event);
  }
};

# replace custom property w/global attribute assignment of same name
# FIXME uses syntax loophole - add attribute instance, etc
# FIXME value processing, etc...
sub prop_to_attr {
  my ($stmt, $content) = @_;
  my $prop = $stmt->{property};
  my $kind = $prop->{kind};

  $prop->{value} ||= '"true"';
  $prop->{value} = "\"$prop->{value}\""
    unless($prop->{value} =~ /"/);
  my $attr = (@{$prop->{path}} && (join('.', @{$prop->{path}}) . '.') || '') .
    "${kind}->attributevalue = $prop->{value} ;\n";
  print STDERR ("prop2attr: " . $attr);
  return("/* custom property \"$prop->{kind}\" converted to attribute: */\n" .
	 comment_out(undef, $content) . "\n" . $attr);
}

sub comment_out {
  my ($comp, $content) = @_;
  # avoid nesting embedded comments
  $content =~ s[(/\*|\*/)][*//*]g;
  return('/* ' . $content . ' */');
}


#======================================================================
#package RDLExtender::Event;
#use warnings;
#use strict;
#our @ISA = qw(RDLExtender::Node);
sub dump_top_meta_data {
  my ($top, $blockname, $eventlist) = @_;

  my $meta_file_name = $top . "events.xml";
  print "Dumping event xml data to $meta_file_name\n";
  open(METAFILE, ">$meta_file_name");
  print METAFILE "<?xml version=\"1.0\" ?> 
  ";

  print METAFILE "\n<event ";
  print METAFILE "name = \"ev_mux_registers\" blockname = \"ev_mux_registers\" xmlns=\"csco_event\" ";
  print METAFILE "top =\"TRUE\">\n";
  
  my $result = "";
  $eventlist->dump_event_tree();
  my $hier_suffix = "";
  foreach my $event ( @{$eventlist->{events}}) {
    $hier_suffix = $event->{inst_idx} if ($event->{inst_max} > 1);
      $result = dump_block_meta_data($event, $hier_suffix, $event->{events}[0]);
    print METAFILE "$result";
  };
  
  print METAFILE "</event>\n";
};

sub dump_block_meta_data {
  my ($parentevent, $hier_suffix, $eventlist) = @_;

  my $result = "";
  my $space = "  ";
  my $blockname = $parentevent->{inst};
  if ($parentevent->{inst_max} > 1) {
   print METAFILE "$space<event name = \"$parentevent->{inst}$parentevent->{inst_idx}\" "; 
   print METAFILE "$space typename = \"$eventlist->{id}\" blockname = \"$blockname\[$hier_suffix]\" summary = \"TRUE\">\n";
  } else {
   print METAFILE "$space<event name = \"$parentevent->{inst}\" typename = \"$eventlist->{id}\" "; 
   print METAFILE "$space blockname = \"$blockname\" summary = \"TRUE\">\n";
  };  

  $blockname .= "[$hier_suffix]" if ($hier_suffix ne "");
  foreach my $ev ( @{$eventlist->{events}}) {
    #$parentevent->dump_event_tree();   
    #$ev->dump_event_tree();   
    $result .= recursive_method($blockname, $ev, $space);
  };

  if ($result =~ /summary/) {
    $result .= "$space</event>\n";
  };
  return($result);
};

sub recursive_method {
  my ($blockname, $events, $space) = @_;
  my $result = "";
  
  if ($events->{kind} =~ "summary") {
    #print "$events->{range}\n";
    for (my $i=0; $i < $events->{range} || 0; $i++) {
    $result .= "$space<event name = \"$events->{id}$i\" blockname = \"$blockname\"\n";
    $result .= "$space summary = \"TRUE\">\n";
    
    my $end = "\n$space</event>\n";
    
    foreach my $event ( @{$events->{events}}) {
      $result .= recursive_method($blockname, $event, $space);
    };
    $result .= $end;
    }

  } elsif ($events->{kind} =~ "leaf") {
    my $desc = $events->{desc} || "";
    $desc =~ s/\"//g;

    $result = "$space  <event name = \"$events->{id}\" blockname = \"$blockname\"\n";
    $result .= "$space  summary = \"FALSE\" severity = \"$events->{severity}\"\n";
    $result .= "$space  action = $events->{action} \n$space  desc = \"$desc\"> \n";
    $result .= "$space  </event>";
  };
  return($result);
};

sub gen_leaf_reg {
   my $class = shift;
   my $parameter_string = shift;
   my (@ids,@names,@descriptions);


   my @reg_array = split (/,/, $parameter_string);

   my $block_name = shift(@reg_array);
   $block_name =~ s/blockname=//;
   $block_name =~ s/^\s+|^\n//g;
   my $reg_name = shift(@reg_array);
   $reg_name =~ s/register=//;
   $reg_name =~ s/^\s+|^\n//g;
   my $num_bits = shift(@reg_array);
   $num_bits =~ s/size=//;
   $num_bits =~ s/^\s+|^\n//g;
   my @event_bits;
   
   my (%desc, %severity, %action, %offset);

   my $index = 0;


   #grab all the bit names and properties
   foreach my $element ( @reg_array ) {
     if ($element =~ /event=/) {
       $event_bits[$index] = $';
       $desc{$event_bits[$index]} = "undef";
       $severity{$event_bits[$index]} = "undef";
       $action{$event_bits[$index]} = "undef";
       $offset{$event_bits[$index]} = "undef";
       $index += 1;
     } 
     elsif ($element =~ /desc=/) {
       $desc{$event_bits[$index-1]} = $';
     } 
     elsif ($element =~ /severity=/) {
       $severity{$event_bits[$index-1]} = $';
     } 
     elsif ($element =~ /action=/) {
       $action{$event_bits[$index-1]} = $';
     } 
     elsif ($element =~ /offset=/) {
         $offset{$event_bits[$index-1]} = $';
     } else {
       print "\n WARNING Ignored unknown event property $element defined in " . $block_name . " bit name " . 
         $event_bits[$index-1];
     };
   };

   #resolving implicit offsets to not collide with explicit offsets
   my @sorted_offsets = sort values(%offset);
   #print "RDLEvent Debug sorted offset @sorted_offsets\n";
   
   $index = 0; 
   foreach my $element (@event_bits) {
     if ($offset{$element} =~ /undef/) {
       #print "RDLEvent Debug Offset $element $offset{$element} \n";
       while ( $index eq $sorted_offsets[0]) {
         pop @sorted_offsets;
         $index += 1;
       };
       #index is an implicit offset
       $offset{$element} = $index;
       $index++;
     };
   };
    

   my @val = sort values(%offset);
   #print "RDLEvent Debug offset values = @val \n";

   #Instance Write 1 to clear enable register
   my $output_string = "

   regfile {
     reg enable_rw1c_reg {
     name=\"Enable RW1C\";
     desc=\"This is an enable register write-one-clear.\";
     rdl_interrupt_enable_rw1c_field
     ";
   foreach my $element (@event_bits) {
     my $tmpoff = $offset{$element};
     $output_string .= "
     $element [$tmpoff :$tmpoff] = 1'b0,";
   };
   chop($output_string);
   $output_string .= ";
     };
   ";
   
   #Instance Write 1 to set enable register
   $output_string .= "
     reg enable_rw1s_reg {
     name=\"Enable RW1S\";
     desc=\"This is an enable register write-one-to-set.\";
     rdl_interrupt_enable_rw1s_field
     ";
   foreach my $element (@event_bits) {
     my $tmpoff = $offset{$element};
     $output_string .= "
     $element [$tmpoff :$tmpoff] = 1'b0,";
   };
   chop($output_string);
   $output_string .= ";
     };
   ";

   #Instance status rw1c register
   $output_string .= "
     reg status_rw1c_reg {
     name=\"INT_STAT_RW1C\";
     desc=\"This register contains the interrupt status.
           This register has access read and write one clear.\";
     ";
   foreach my $element (@event_bits) {
     my $tmpoff = $offset{$element};
     $output_string .= "
       rdl_interrupt_leaf_status_rw1c_field
         $element = 1'b0;
         $element\->name= \"$element\";
         $element\->desc=\"
                $desc{$element}\";";
   };
   chop($output_string);
   $output_string .= ";
     };
   ";

   #Instance status rw1s register
   $output_string .= "
     reg status_rw1s_reg {
     name=\"INT_STAT_RW1S\";
     desc=\"This register contains the interrupt status.
           This register has access read and write one set.\";";
           
   foreach my $element (@event_bits) {
     my $tmpoff = $offset{$element};
     $output_string .= "
       rdl_interrupt_leaf_status_rw1s_field
         $element = 1'b0;
         $element\->name=\"$element\";
         $element\->desc=\"
                $desc{$element}\";
         $element\->attribute=donttest_field;";
   };
   chop($output_string);
   $output_string .= ";
     };
   ";
   
   $output_string .= "
      reg first_reg {
         name=\"INT_FIRST\";
         desc=\"This register contains the first interrupt.
               This register has access read and write one clear.\";";

   $output_string .= "
   rdl_interrupt_first_field first[@{[$num_bits - 1]} : 0] = $num_bits\'b0;
      };

      status_rw1c_reg int_stat_rw1c;
      int_stat_rw1c->name=\"$reg_name Interrupt Status Register RW1C\";
      int_stat_rw1c->desc=\"
            This register contains the $reg_name interrupt status.
            This register has access read and write one clear.\";
      alias int_stat_rw1c status_rw1s_reg int_stat_rw1s;
      int_stat_rw1s->attribute=donttest_reg;
      int_stat_rw1s->name=\"$reg_name Interrupt Status Register RW1S\";
      int_stat_rw1s->desc=\"
            This register contains the $reg_name interrupt status.
            This register has access read and write one set.\";

      enable_rw1c_reg int_en_rw1c;
      int_en_rw1c->name=\"$reg_name Interrupt Enable Register RW1C\";
      int_en_rw1c->desc=\"
            This register contains the interrupt enables for
            the $reg_name Interrupt Status register RW1C.
            This register has access read and write one clear.\";

      alias int_en_rw1c enable_rw1s_reg int_en_rw1s;
      int_en_rw1s->name=\" $reg_name Interrupt Enable Register RW1S\";
      int_en_rw1s->desc=\"
            This register contains the interrupt enables for
            the $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      enable_rw1c_reg halt_en_rw1c;
      halt_en_rw1c->name=\" $reg_name Halt Enable Register RW1C\";
      halt_en_rw1c->desc=\"
            This register contains the interrupt halt enables for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";

      alias halt_en_rw1c enable_rw1s_reg halt_en_rw1s;
      halt_en_rw1s->name=\"$reg_name Halt Enable Register RW1S\";
      halt_en_rw1s->desc=\"
            This register contains the interrupt halt enables for
            the . $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      enable_rw1c_reg first_en_rw1c;
      first_en_rw1c->name=\" $reg_name. First Enable Register RW1C\";
      first_en_rw1c->desc=\"
            This register contains the interrupt first enables for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";

      alias first_en_rw1c enable_rw1s_reg first_en_rw1s;
      first_en_rw1s->name=\" $reg_name. First Enable Register RW1S\";
      first_en_rw1s->desc=\"
            This register contains the interrupt first enables for
            the $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      first_reg int_first;
      int_first->name=\"$reg_name. Interrupt First Register\";
      int_first->desc=\"
            This register contains the first interrupt for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";
      "; 
      
      foreach my $element (@event_bits) {
        $output_string .= "
        int_stat_rw1c.$element->enable = 
          int_en_rw1c.$element;
          ";

        $output_string .= "
        int_stat_rw1c.$element->haltenable = 
          halt_en_rw1c.$element;
          ";
      };

      my @rev_event_bits = reverse(@event_bits);
      
      $output_string .= "
      int_first.first->hwenable = 
        {";

      foreach my $element (@rev_event_bits) {
         $output_string .= "
         first_en_rw1c.$element,";
      };

      chop($output_string);
      $output_string .= "
        };
      ";
        
      $output_string .= "
      int_first.first = {";
      
      foreach my $element (@rev_event_bits) {
         $output_string .= "
         int_stat_rw1c.$element->event,";
      };

      chop($output_string);
      $output_string .= "
        };
      ";
      
      $output_string .= &rdl_interrupt_enable ('int', \@event_bits);
      $output_string .= &rdl_interrupt_enable ('halt', \@event_bits);
      $output_string .= &rdl_interrupt_enable ('first', \@event_bits);

      my $int_regfile_inst_name = "$reg_name";
      my $int_att_inst_name = "$int_regfile_inst_name\_att";
      my $int_stat_inst_name = "$int_regfile_inst_name\.int_stat_rw1c";
      
      $output_string .= "
      } $reg_name;
      ";

      #$output_string .= "
      #int_reg_att_type $int_att_inst_name;
      #$int_att_inst_name->attributereference = $int_stat_inst_name;
      #"; 

  return($output_string);      

};

sub rdl_interrupt_enable {
   my ($type, $events) = @_;
   my (@event_bits) = @$events;
   my ($name,$desc);

   if ($type eq 'int') {
      $name = '';
      $desc = 'This bit enables the propagation of';
   }
   elsif ($type eq 'halt') {
      $name = 'Halt ';
      $desc = 'This bit enables halt due to';
   }
   elsif ($type eq 'first') {
      $name = 'First ';
      $desc = 'This bit enables first latching of';
   };

   my $output_string = "";
   
   foreach my $element (@event_bits) {
     $output_string .= "$type\_en_rw1c.$element->name = 
         \"$element Enable RW1C\";
         
     $type\_en_rw1c.$element->desc = 
         \"$desc the $element Interrupt.\";
     ";
         
   };

   foreach my $element (@event_bits) {
     $output_string .= "$type\_en_rw1s.$element->name = 
         \"$element Enable RW1C\";
         
     $type\_en_rw1c.$element->desc = 
         \"$desc the $element Interrupt.\";
     ";
   
   };

   return($output_string);

};


sub gen_summary_reg {

   my $class = shift;
   my $parameter_string = shift;
   my (@ids,@names,@descriptions);

   my @reg_array = split (/,/, $parameter_string);

   my $block_name = shift(@reg_array);
   $block_name =~ s/blockname=//;
   $block_name =~ s/^\s+|^\n//g;
   my $reg_name = shift(@reg_array);
   $reg_name =~ s/register=//;
   $reg_name =~ s/^\s+|^\n//g;
   my $num_bits = shift(@reg_array);
   $num_bits =~ s/size=//;
   $num_bits =~ s/^\s+|^\n//g;
   my @event_bits;

   my $index = 0;

   #grab all the bit names and properties
   foreach my $element ( @reg_array ) {
     if ($element =~ /event=/) {
       $event_bits[$index] = $';
       $index += 1;
     } else {
       print "\n WARNING Ignored unknown property $element defined in " . $block_name . "
       bit name $event_bits[$index-1]";
     };
   };

   $index = 0;
   
   #Instance Write 1 to clear enable register
   my $output_string = "

   regfile {
      
     name=\"$reg_name Hierarchical Interrupt Register File\";
     desc=\"This is the set of $reg_name
         hierarchical interrupt registers.\";

     reg enable_rw1c_reg {
     name=\"Enable RW1C\";
     desc=\"This is an enable register write-one-clear.\";
     rdl_interrupt_enable_rw1c_field
     ";
     
   foreach my $element (@event_bits) {
     $output_string .= "
     $element [$index :$index] = 1'b0,";
     $index += 1;
   };
   chop($output_string);
   $output_string .= ";
     };
   ";

   #Instance Write 1 to set enable register
   $output_string .= "
     reg enable_rw1s_reg {
     name=\"Enable RW1S\";
     desc=\"This is an enable register write-one-to-set.\";
     rdl_interrupt_enable_rw1s_field
     ";
   
   $index = 0;
   
   foreach my $element (@event_bits) {
     $output_string .= "
     $element [$index :$index] = 1'b0,";
     $index += 1;
   };
   chop($output_string);
   $output_string .= ";
     };
   ";
   
   #Instance status rw1c register
   $output_string .= "
     reg status_rnc_reg {
     name=\"INT_STAT_RNC\";
     desc=\"This register contains the hierarchical interrupt status.
           This register has access read and no clear.\";
     ";
   foreach my $element (@event_bits) {
     $output_string .= "
       rdl_interrupt_hierarchical_status_field
         $element = 1'b0;
         $element\->name= \"$element\";
         $element\->desc=\"
                Hierarchical interrupt event\";";
   };
   chop($output_string);
   $output_string .= ";
     };
   ";

   $output_string .= "
      reg first_reg {
         name=\"INT_FIRST\";
         desc=\"This register contains the first interrupt.
               This register has access read and write one clear.\";";

   my $bit_width = $num_bits + 1;
   
   $output_string .= "
      rdl_interrupt_first_field first[$num_bits: 0] = $bit_width\'b0;
      };

      status_rnc_reg int_stat_rnc;
      int_stat_rnc->name=\"$reg_name Hierarchical Interrupt Status Register RW1C\";
      int_stat_rnc->desc=\"
            This register contains the $reg_name interrupt status.
            This register has access read.\";

      enable_rw1c_reg int_en_rw1c;
      int_en_rw1c->name=\"$reg_name Interrupt Enable Register RW1C\";
      int_en_rw1c->desc=\"
            This register contains the interrupt enables for
            the $reg_name Interrupt Status register RW1C.
            This register has access read and write one clear.\";

      alias int_en_rw1c enable_rw1s_reg int_en_rw1s;
      int_en_rw1s->name=\" $reg_name Interrupt Enable Register RW1S\";
      int_en_rw1s->desc=\"
            This register contains the interrupt enables for
            the $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      enable_rw1c_reg halt_en_rw1c;
      halt_en_rw1c->name=\" $reg_name Halt Enable Register RW1C\";
      halt_en_rw1c->desc=\"
            This register contains the interrupt halt enables for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";

      alias halt_en_rw1c enable_rw1s_reg halt_en_rw1s;
      halt_en_rw1s->name=\"$reg_name Halt Enable Register RW1S\";
      halt_en_rw1s->desc=\"
            This register contains the interrupt halt enables for
            the . $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      enable_rw1c_reg first_en_rw1c;
      first_en_rw1c->name=\" $reg_name. First Enable Register RW1C\";
      first_en_rw1c->desc=\"
            This register contains the interrupt first enables for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";

      alias first_en_rw1c enable_rw1s_reg first_en_rw1s;
      first_en_rw1s->name=\" $reg_name. First Enable Register RW1S\";
      first_en_rw1s->desc=\"
            This register contains the interrupt first enables for
            the $reg_name Interrupt Status Register RW1S.
            This register has access read and write one set.\";

      first_reg int_first;
      int_first->name=\"$reg_name. Interrupt First Register\";
      int_first->desc=\"
            This register contains the first interrupt for
            the $reg_name Interrupt Status Register RW1C.
            This register has access read and write one clear.\";
      "; 
      
      foreach my $element (@event_bits) {
        $output_string .= "int_stat_rnc.$element->enable = 
          int_en_rw1c.$element;
          ";

        $output_string .= "int_stat_rnc.$element->haltenable = 
          halt_en_rw1c.$element;
          ";
      };

      my @rev_event_bits = reverse(@event_bits);
      
      $output_string .= "int_first.first->hwenable = 
        {";

      foreach my $element (@rev_event_bits) {
         $output_string .= "
         first_en_rw1c.$element,";
      };

      chop($output_string);
      $output_string .= "
        };
      ";
        
      $output_string .= "int_first.first = {";
      
      foreach my $element (@rev_event_bits) {
         $output_string .= "
         int_stat_rnc.$element->event,";
      };

      chop($output_string);
      $output_string .= "
        };
      ";
      
      $output_string .= &rdl_interrupt_enable ('int', \@event_bits);
      $output_string .= &rdl_interrupt_enable ('halt', \@event_bits);
      $output_string .= &rdl_interrupt_enable ('first', \@event_bits);

      my $int_regfile_inst_name = "$reg_name";
      my $int_att_inst_name = "$int_regfile_inst_name\_att";
      my $int_stat_inst_name = "$int_regfile_inst_name\.int_stat_rnc";
      
      $output_string .= "
      } $reg_name;
      ";

      #$output_string .= "
      #int_reg_att_type $int_att_inst_name;
      #$int_att_inst_name->attributereference = $int_stat_inst_name;
      #"; 

  return($output_string);      

};

sub gen_leaf_summary_connect {
  my $class = shift;
  my $parameter_string = shift;

  my @reg_array = split (/,/, $parameter_string);

  #print @reg_array;
  my $block_name = shift(@reg_array);
  $block_name =~ s/blockname=//;
  $block_name =~ s/^\s+|^\n//g;
  
  my $parent_int_reg = "$block_name\.int_stat_rnc";
  
  my $output_string = "";
  my $parent_field = "";

  foreach my $element ( @reg_array ) {
    $element =~ s/^\s+|^\n//g;
    $output_string .= "
    $parent_int_reg.$element\->next = 
      $element.int_stat_rw1c->intr;
    $parent_int_reg.$element\->attribute = 
      $element\_att;
    ";
  };
  return($output_string);      
};

1;
