//------------------------------------------------------------------------
// Title        : csco_event package Env.
// Project      : csco_event
//              :
// Created On   : 01/10/2006
//              :
//              :
// Description  : 
//              :
//------------------------------------------------------------------------
//------------------------------------------------------------------------
// Copyright 2006 (c) Cisco Systems Inc.
//------------------------------------------------------------------------
<'
package csco_event;

// csco_addrmap extensions for csco_event eVC
extend csco_addrmap_driver_u {
  // FIXME this is somewhat broken...
  csco_event_scope : csco_event_s;
  // FIXME what is the "right" way todo this?
  keep gen (csco_event_scope)
    before (reset_seq, init_test_seq, pre_test_seq, main_test_seq, post_test_seq);

  // sequence that will handle events
  !csco_event_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft csco_event_seq.driver == me;
    keep soft csco_event_seq.event_seq_kind == DETECT;

  // called when events are detected
  // kicks off addrmap_seq to handle detection
  csco_event_handler(trans: csco_event_s) @clock is {
    //set a current scope for this triggered sequence
    var current_event_scope := csco_event_scope.get_child_scope(trans.name);
    gen csco_event_seq keeping {
      .event_type == trans.types[0];
      .event_scope == current_event_scope;
    };
    var seq := csco_event_seq;
    seq.start();
    sync cycle @seq.ended;
  };
};

// Extend the base csco_addrmap sequence with event_scope sub-type
// event scope determines current operating event scope in the event
// tree
extend csco_addrmap_seq {
  // current event context
  event_scope : csco_event_s;
    keep soft event_scope == read_only(driver.csco_event_scope);
    keep read_only(parent_sequence is a csco_addrmap_seq (seq)) =>
      soft event_scope == seq.event_scope;
};


//------------------------------------------------------------
// all event control/testing use a single base addrmap sequence

extend csco_addrmap_seq_kind_t : [ CSCO_EVENT ];

extend CSCO_EVENT csco_addrmap_seq {
  keep soft default_scope == read_only(driver.default_scope.get_scope_first_match(event_scope.addrmap_scope()));

  // sequence subkind
  event_seq_kind : csco_event_addrmap_seq_kind_t;

  // type of event for operation
  event_type : csco_event_name_t;

  // Flag when set will cause auto-test sequences to pick random event paths to 
  // test. If event_random==FALSE, the event tree will be tested left to right
  // in a structured fashion
  event_random : bool;

  // automagic default parameter propagation
  keep read_only(parent_sequence is a CSCO_EVENT csco_addrmap_seq (seq)) =>
    soft event_type == seq.event_type and
    soft event_seq_kind == seq.event_seq_kind and
    soft event_random == seq.event_random;

  // sequence for recursion
  !event_seq : CSCO_EVENT csco_addrmap_seq;

  // sequences to enable events during testing
  !enable_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft enable_seq.event_seq_kind == ENABLE_CFG;
  // sequences to disable events during testing
  !disable_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft disable_seq.event_seq_kind == DISABLE_CFG;
};


//------------------------------------------------------------
// configure the enable bits for all events in a tree
// FIXME skip updates if shadow already matches
extend csco_event_addrmap_seq_kind_t : [ ENABLE_CFG ];
    
extend ENABLE_CFG CSCO_EVENT csco_addrmap_seq {
  body() @driver.clock is {
    messagef(LOW, "Starting %s.EvPath = %s\n", 
             kind, event_scope.addrmap_scope());

    var field_value_pairs : list of csco_addrmap_field_value_pair_s;
    
    for each (e) in event_scope.children {
      if (!e.children.is_empty()) {
        // recurse
        do event_seq keeping { .event_scope == e };
      };
      var n:string = str_replace(e.name, "/\[|\]/", "");
      var field_value:= new csco_addrmap_field_value_pair_s with {
        .name = n;
        .value = e.is_enabled[event_type.as_a(uint)-e.first_idx].as_a(bit);
        .value_kind = CONSTANT;
      };
      field_value_pairs.add(field_value);
    };

    if (event_scope.parent != NULL) { 
      addrmap_do WRITE entry_trans keeping {
        .entry == ((event_type.as_a(uint) == 1) ? 
          "int_en_rw1s" : "halt_en_rw1s");
        .fields == field_value_pairs;
      };
    };
  };
};


//------------------------------------------------------------
// configure the enable bits for all events in a tree
// FIXME skip updates if shadow already matches
extend csco_event_addrmap_seq_kind_t : [ DISABLE_CFG ];
    
extend DISABLE_CFG CSCO_EVENT csco_addrmap_seq {
  body() @driver.clock is {
    messagef(LOW, "Starting %s.EvPath = %s\n", 
             kind, event_scope.addrmap_scope());

    var field_value_pairs: list of csco_addrmap_field_value_pair_s;

    for each (e) in event_scope.children {
      if (!e.children.is_empty()) {
        // recurse
        do event_seq keeping { .event_scope == e };
      };
      var n:string = str_replace(e.name, "/\[|\]/", "");
      var field_value:= new csco_addrmap_field_value_pair_s with {
        .name = n;
        .value = ~e.is_enabled[event_type.as_a(uint)-e.first_idx].as_a(bit);
        .value_kind = CONSTANT;
      };
      field_value_pairs.add(field_value);
    };

    addrmap_do WRITE entry_trans keeping {
      .entry == ((event_type.as_a(uint) == 1) ? 
            "int_en_rw1c" : "halt_en_rw1c");
      .fields == field_value_pairs;
    };
  };
};

//------------------------------------------------------------
// This sequence is launched when an interrupt event is detected
// It starts reading registers at the highest level specified, looks
// for the lowest level leaf events that are set and launches clear 
// sequences one leaf event at a time.
extend csco_event_addrmap_seq_kind_t : [ DETECT ];
    
extend DETECT CSCO_EVENT csco_addrmap_seq {
  // sequence to clear a detected leaf event
  !clear_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft clear_seq.event_seq_kind == CLEAR;

  body() @driver.clock is {
    var is_summary:= 
      event_scope.children.has(.is_summary[event_type.as_a(uint)-event_scope.first_idx]);

    addrmap_do READ entry_trans keeping {
      .entry == ((event_type.as_a(uint) == 1) ?
        ((is_summary) ? "int_stat_rnc" : "int_stat_rw1c"):
        ((is_summary) ? "halt_stat_rnc" : "int_stat_rw1c"));
    };

    var reg_value: uint = pack(packing.low, entry_trans.data);
    var found := FALSE;

    for each (e) in event_scope.children {
      if ((reg_value[index:index] == 1) and 
          (e.is_enabled[event_type.as_a(uint)-e.first_idx])) {
          
        messagef(CSCO_EVENT, LOW, "Detected event %s", e.name);
        
        if (!e.children.is_empty()) {
          // recurse
          do event_seq keeping { .event_scope == e };
        }
        else {
          e.update_count(event_type);
          e.wait_on_detect = FALSE;
          do clear_seq keeping { .event_scope == e };
        };
        //Found an original error
        found = TRUE;
      };
    };
    //Check if a path was found
    if (!found) {
      if (reg_value != 0) {
        for each (e) in event_scope.children {
          if ((reg_value[index:index] == 1) and (!(e.is_enabled[event_type.as_a(uint)-e.first_idx]))) {
            dut_error("ERR_CSCO_EVENT_DISABLED_EVENT_TRIGGERED\n",
            "Event ", e.name, " found in event tree of type ",
            event_type, " @ ", e.parent.name, " level of tree.",
            "\nBut this event is DISABLED according to the eventmap configuration.",
            "\nNon-csco_event originated write to enable register may have occured.",
            "\nAll event related config has to originate from the csco_event config sequences."); 
          };
        };
      } else {
        dut_error("ERR_CSCO_EVENT_NO_ORIGIN\n",
          "No leaf level originating event found in event tree of type ",
          event_type,
          "\nThe last event register read had no bits set. Hence the DETECT",
          "\nsequence could not find event activation path beyond ", 
          event_scope.name,
          "\n\nPossible issue with *posted* write to clear operation not",
          "\ncompleting before re-arming of interrupt." );
      };
    };
  };
};

//------------------------------------------------------------
// Clears a single leaf event.
// Launches hook sequences before and after the clear operation
extend csco_event_addrmap_seq_kind_t : [ CLEAR ];

extend CLEAR CSCO_EVENT csco_addrmap_seq {
  // hook sequence invoked before a clear operation
  !pre_clear_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft pre_clear_seq.event_seq_kind == EMPTY;
  // hook sequence invoked after a clear operation
  !post_clear_seq : CSCO_EVENT csco_addrmap_seq;
    keep soft post_clear_seq.event_seq_kind == EMPTY;

  keep default_scope == read_only(parent_sequence.as_a(CSCO_EVENT csco_addrmap_seq).default_scope);

  body() @driver.clock is {
    
    do pre_clear_seq;

    messagef(CSCO_EVENT, LOW, "Clear event %s.%s", 
             event_scope.parent.name, event_scope.name);

    addrmap_do WRITE entry_trans keeping {
      .entry  ==  "int_stat_rw1c";
      .post_kind == NON_POSTED;
      .fields == {.set_field(event_scope.name, 1)};
      .unspecified_field_kind == SHADOW;
    };

    //Read Back the register to make sure write propogated
    addrmap_do READ entry_trans keeping {
      .entry  ==  "int_stat_rw1c";
    };
    
    event_scope.wait_on_clear = FALSE;

    do post_clear_seq;
  };
};


//------------------------------------------------------------
// For each summary bit, 
//   disable it, check for propagation from leaves
//   enable it, check for propagation from leaves
// also for each leaf, check if it propagates when disabled
// If random_event == TRUE, do the leaf event test based on a 
// coin toss
extend csco_event_addrmap_seq_kind_t : [ AUTO_TEST ];

extend AUTO_TEST CSCO_EVENT csco_addrmap_seq {
  body() @driver.clock is {
    messagef(LOW, "Starting %s.EvPath = %s\n", 
             kind, event_scope.addrmap_scope());

    for each (e) in event_scope.children {
      var test_event: bool;
      gen test_event keeping {
        read_only(!event_random) => (it == TRUE)
      };

      if (!e.children.is_empty()) {
        //disable the summary event, configure it and test leaf events
        e.enable(e.name, DISABLE, event_type);

        var n:string = str_replace(e.name, "/\[|\]/", "");
        addrmap_do WRITE entry_trans keeping {
          .entry == ((event_type.as_a(uint) == 1) ? 
            "int_en_rw1c" : "halt_en_rw1c");
          .fields == {.set_field(n, 1)};
          .unspecified_field_kind == SHADOW;
        };

        // recurse
        do event_seq keeping { .event_scope == e };

        //enable the summary event, configure it and test leaf events
        e.enable(e.name, ENABLE, event_type);
        
        n = str_replace(e.name, "/\[|\]/", "");
        addrmap_do WRITE entry_trans keeping {
          .entry == ((event_type.as_a(uint) == 1) ? 
            "int_en_rw1s" : "halt_en_rw1s");
          .fields == {.set_field(n, 1)};
          .unspecified_field_kind == SHADOW;
        };

        // recurse
        do event_seq keeping { .event_scope == e };
      }
      else if (test_event and !e.is_summary[event_type.as_a(uint)-e.first_idx]) {
        messagef(CSCO_EVENT, LOW, "Testing event %s.%s",
                 event_scope.name, e.name);
          
        //Enable the event
        e.enable(e.name, ENABLE, event_type);
        do enable_seq;

        e.auto_test = TRUE;
        e.update_exp_count(e.name, 1, 1,event_type);

        //Set the test bit
        addrmap_do WRITE entry_trans keeping {
          .entry  ==  "int_stat_rw1s";
          .fields == {.set_field(e.name, 1)};
          .unspecified_field_kind == SHADOW;
        };

        //Wait until event is cleared
        e.wait_on_clear();

        //If the summary level is already disabled, no point in checking 
        //if leaf level enable/disable works
        if (e.will_it_propagate(event_type)) {
          //disable the event and test to check if it is propagated
          e.enable(e.name, DISABLE, event_type);

          do disable_seq;

          //Set the test bit and check if it propogates
          addrmap_do WRITE entry_trans keeping {
            .entry  ==  "int_stat_rw1s";
            .fields == {.set_field(e.name, 1)};
            .unspecified_field_kind == SHADOW;
          };

          wait[100];

          //No event should have been detected
          //Once the event is re-enabled event will fire
          e.enable(e.name, ENABLE, event_type);
 
          e.auto_test = TRUE;
          e.update_exp_count(e.name, 1, 1, event_type);

          do enable_seq;

          //Event should have been detected. Wait until
          //it is cleared
          e.wait_on_clear();
        }
        else {
          //if the event wont propagate, clear sequence will never
          //launch. So reset the event before returning
          addrmap_do WRITE entry_trans keeping {
            .entry  ==  "int_stat_rw1c";
            .fields == {.set_field(e.name, 1)};
            .unspecified_field_kind == SHADOW;
          };

          //Read to ensure write has completed
          addrmap_do READ entry_trans keeping {
            .entry  ==  "int_stat_rw1c";
          };
          
        };
      };
    };
  };
};

//------------------------------------------------------------
// Disable all summary events
// Enable all leaf event
// Set all leaf event bits
// Check for no event
// This sequence has to be run from the block or top level

extend csco_event_addrmap_seq_kind_t : [ AUTO_SUMMARY ];
extend AUTO_SUMMARY CSCO_EVENT csco_addrmap_seq {
  body() @driver.clock is {
  
    event_scope.enable_all(DISABLE, event_type);
    do disable_seq;

    messagef(LOW, "Starting %s.EvPath = %s\n", 
             kind, event_scope.addrmap_scope());

    for each (e) in event_scope.children {
      if (!e.children.is_empty()) {
        // recurse
        do event_seq keeping { .event_scope == e };
      }
      else {
        //Enable the leaf event and test it
        e.enable(e.name, ENABLE, event_type);

        do enable_seq;
        
        // Set the test bit. It shouldnt propogate
        addrmap_do WRITE entry_trans keeping {
          .entry  ==  "int_stat_rw1s";
          .fields == {.set_field(e.name, 1)};
          .unspecified_field_kind == SHADOW;
        };

        wait[100];
        
        // Clear the test bit. It shouldnt have propogated
        addrmap_do WRITE entry_trans keeping {
          .entry  ==  "int_stat_rw1c";
          .fields == {.set_field(e.name, 1)};
          .unspecified_field_kind == SHADOW;
        };
        
      };
    };
  };
};

//------------------------------------------------------------
//Find a leaf register level
//
//Step1
//enable one random first interrupts
//set all but relevant test events. Then set relevant test event
//check if the first test event alone was captured
//clear all events
//disable all first interrupts
//
//Step2
//
//enable all but one first interrupt
//set all but enabled test event
//check if capture bit for disabled event is zero
//disable all events
//clear all events
//
extend csco_event_addrmap_seq_kind_t : [ AUTO_FIRST_LEAF ];

extend AUTO_FIRST_LEAF CSCO_EVENT csco_addrmap_seq {

  body()@driver.clock is also {

    messagef(LOW, "Starting %s. EvPath = %s\n", 
             event_seq_kind, event_scope.addrmap_scope());

    var field_value_pairs : list of csco_addrmap_field_value_pair_s;
    
    for each (e) in event_scope.children {
      if (!e.children.is_empty()) {
        do event_seq keeping { .event_scope == e; };
      };
      if (!e.is_summary[event_type.as_a(uint)]) {
        var test_event: csco_event_s = e;

        messagef(CSCO_EVENT, LOW, "Testing first capture for EvPath = %s\n", 
             test_event.addrmap_scope());

        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1s";
          .fields == {.set_field(test_event.name, 1)};
          .unspecified_field_kind == SHADOW;
        };
        //set all other test bits
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1s";
          .fields == test_event.set_only_relevant(0);
        };

        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };

        check that (pack(packing.low, entry_trans.data) == 0) else 
          dut_error("ERR_CSCO_EVENT_FIRST_ENABLE\n",
            "First capture set when expecting zero");

        //set test bit of interest
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1s";
          .fields == {.set_field(test_event.name, 1)};
          .unspecified_field_kind == SHADOW;

        };
        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };
         
        var reg_value: uint = pack (packing.low, entry_trans.data);
        check that (reg_value == test_event.set_only_relevant_phy(1)) 
          else dut_errorf("CSCO_EVENT_FIRST_EMPTY: first reg is incorrect: Exp %x Act %x",
            test_event.set_only_relevant_phy(1), reg_value );
          
        //Clear the test register
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1c";
          .fields == test_event.set_all(1);
        };
        //Clear the capture enable
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1c";
          .fields == test_event.set_all(1);
        };
        //Clear the capture register
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_first";
          .fields == {.set_field("first", test_event.set_only_relevant_phy(1))};
          .unspecified_field_kind == SHADOW;
        };
        //Enable capture for all but the test event
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1s";
          .fields == test_event.set_only_relevant(0);
        };
        //set test bit of interest
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1s";
          .fields == {.set_field(test_event.name, 1)};
          .unspecified_field_kind == SHADOW;
        };
        //set all other test bits
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1s";
          .fields == test_event.set_only_relevant(0);
        };
        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };
        
        reg_value = pack (packing.low, entry_trans.data);
        check that (reg_value == test_event.set_only_relevant_phy(1)) 
          else dut_errorf("CSCO_EVENT_FIRST_EMPTY: first reg is incorrect: Exp %x Act %x",
            test_event.set_only_relevant_phy(1), reg_value );
            
        //Clear the test register
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_stat_rw1c";
          .fields == test_event.set_all(1);
        };
        //clear all the bits
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_first";
          .fields == {.set_field("first", test_event.set_all_phy(1))};
        };
        //clear all enables
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1c";
          .fields == test_event.set_all(1);
        };
        //read to sync 
        addrmap_do READ entry_trans keeping {
          .entry == "int_en_rw1c";
        };

      };
    };
  };
};

//Wrapper sequence to check first interrupt at summary level
//Start by enabling all events
extend csco_event_addrmap_seq_kind_t : [ AUTO_FIRST_SUMMARY ];

extend AUTO_FIRST_SUMMARY CSCO_EVENT csco_addrmap_seq {
  
  body()@driver.clock is also {
    //enable all events in the path
    event_scope.enable_all(ENABLE, event_type);
    do ENABLE_CFG CSCO_EVENT addrmap_seq;
    do AUTO_FIRST_SUMMARY_TEST CSCO_EVENT addrmap_seq;
  };
};

//------------------------------------------------------------
//Find top summary level register
//
//Step 1
//Enable all the capture bits
//Select one of the summary bits
//descend one level
//if summary level again, repeat above 3 steps
//if leaf level set one of the events
//go up a level and set a test bit from another path
//check first register value for each summary

extend csco_event_addrmap_seq_kind_t : [ AUTO_FIRST_SUMMARY_TEST ];

extend AUTO_FIRST_SUMMARY_TEST CSCO_EVENT csco_addrmap_seq {

  body()@driver.clock is also {

    messagef(LOW, "Starting %s. EvPath = %s\n", 
             event_seq_kind, event_scope.addrmap_scope());

    for each (e) in event_scope.children {
      if (!e.children.is_empty()) {
        do event_seq keeping { .event_scope == e; };
      };
      if (e.is_summary[event_type.as_a(uint)]) {
        var test_event: csco_event_s = e;

        messagef(CSCO_EVENT, LOW, "Testing first capture for EvPath = %s\n", 
             test_event.addrmap_scope());

        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1s";
          .fields == {.set_field(test_event.name, 1)};
          .unspecified_field_kind == SHADOW;
        };

        var alternate_event: csco_event_s;
        var alternate_index: uint;
        gen alternate_index keeping {
          it != index; 
          it < event_scope.children.size();
        };
        alternate_event = event_scope.children[alternate_index];

        //set some other other test bit
        do AUTO_FORCE_SUMMARY CSCO_EVENT addrmap_seq keeping {
          .event_scope == alternate_event;
        };

        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };

        check that (pack(packing.low, entry_trans.data) == 0) else 
          dut_error("ERR_CSCO_EVENT_FIRST_ENABLE\n",
            "First capture set when expecting zero");
        
        do AUTO_FORCE_SUMMARY CSCO_EVENT addrmap_seq keeping {
          .event_scope == test_event;
        };

        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };

        var reg_value :uint = pack(packing.low, entry_trans.data);

        check that (reg_value == test_event.set_only_relevant_phy(1)) 
          else dut_errorf("CSCO_EVENT_FIRST_EMPTY: first reg is incorrect: Exp %x Act %x",
            test_event.set_only_relevant_phy(1), reg_value);
       
        //Clear the capture enable
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1c";
          .fields == test_event.set_all(1);
        };
        //Clear the capture register
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_first";
          .fields == {.set_field("first", test_event.set_only_relevant_phy(1))};
          .unspecified_field_kind == SHADOW;
        };

        //Make sure underlying events are cleared by auto test before
        //progressing to next test bit
        repeat {
          messagef(CSCO_EVENT, HIGH, "Polling summary register %s",
            event_scope.name);
          addrmap_do READ entry_trans keeping {
            .entry == "int_stat_rnc";
          };
          wait[10];
        } until ( pack(packing.low, entry_trans.data) == 0);

        //Enable capture for all but the test event
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1s";
          .fields == test_event.set_only_relevant(0);
        };

        do AUTO_FORCE_SUMMARY CSCO_EVENT addrmap_seq keeping {
          .event_scope == test_event;
        };
        
        //set alternate test bit
        do AUTO_FORCE_SUMMARY CSCO_EVENT addrmap_seq keeping {
          .event_scope == alternate_event;
        };
        
        //read first int reg
        addrmap_do READ entry_trans keeping {
          .entry == "int_first";
        };
        reg_value = pack (packing.low, entry_trans.data);

        check that (reg_value == test_event.set_only_relevant_phy(0)) 
          else dut_errorf("CSCO_EVENT_FIRST_EMPTY: first reg is incorrect: Exp %x Act %x",
            test_event.set_only_relevant_phy(0), reg_value);
          
        //clear all the enables first
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_en_rw1c";
          .fields == test_event.set_all(1);
        };
        //clear all the bits
        addrmap_do WRITE entry_trans keeping {
          .entry == "int_first";
          .fields == {.set_field("first", test_event.set_all_phy(1))};
        };
 
        //Make sure underlying events are cleared by auto test before
        //progressing to next test bit
        repeat {
          messagef(CSCO_EVENT, HIGH, "Polling summary register %s",
            event_scope.name);
          addrmap_do READ entry_trans keeping {
            .entry == "int_stat_rnc";
          };
          wait[10];
        } until ( pack(packing.low, entry_trans.data) == 0);

      };
    };
  };

};

//Sequence to force an underlying leaf event to force a summary event
//to be triggered. This is required as there is no test reg decode for
//the summary events
extend csco_event_addrmap_seq_kind_t : [ AUTO_FORCE_SUMMARY ];

extend AUTO_FORCE_SUMMARY CSCO_EVENT csco_addrmap_seq {
  
  body()@driver.clock is also {
   
    messagef( LOW, "Starting %s. EvPath = %s\n", 
             event_seq_kind, event_scope.addrmap_scope());
    //find a leaf event
    while (event_scope.is_summary[event_type.as_a(uint)]) {
      event_scope = event_scope.children[0];
    };
    default_scope = 
      driver.default_scope.get_scope_first_match(
        event_scope.parent.addrmap_scope());
    //set the testbit
    addrmap_do WRITE entry_trans keeping {
      .entry == "int_stat_rw1s";
      .fields == {.set_field(event_scope.name, 1)};
      .unspecified_field_kind == SHADOW;
    };
    event_scope.update_exp_count(event_scope.name, 1, 1, event_type);
  };
};
   
'>
