//------------------------------------------------------------------------
// Title        : csco_event package Env.
// Project      : csco_event
//              :
// Created On   : 01/10/2006
//              :
//              :
// Description  : 
//              :
//------------------------------------------------------------------------
//------------------------------------------------------------------------
// Copyright 2006 (c) Cisco Systems Inc.
//------------------------------------------------------------------------
<' 
package csco_event;

import csco_xml/e/sax/csco_xml_sax_top;

extend csco_event_s {
  populate_from_xml_file(filename: string, top_scope: string) is {
    assert(!types.is_empty()) else 
      error("ERR_CSCO_EVENT_NO_NAMES\n",
            "Event types not specified for event tree, make sure you've set ", 
            get_unit().e_path(), ".config.names");

    filename = files.add_file_type(filename, ".xml", TRUE);
    messagef(LOW, "Reading event definitions from %s\n", filename);
    var reader := new csco_xml_sax_XMLReader;
    var handler := new csco_event_populate_xml with {
      .top_scope = top_scope;
      .root_event = me;
    };
    reader.setContentHandler(handler);
    reader.parseURI(filename);
  };
};

// subtype SAX reader to make your parser
struct csco_event_populate_xml like csco_xml_sax_Handler {
  // for caller filtering of monolithic event file
  !top_scope : string;
  !root_event : csco_event_s;

  // currently parsing event
  !current_event : csco_event_s;

  // XML namespace used in document
  !ns : string;
  // string values treated as boolean FALSE (everything else is TRUE)
  !false : list of string;

  init() is also {
    // initialize constants
    ns = "http://www.cisco.com/2007/csco_event"; // FIXME use a URI (as per spec)
    false = { "0"; ""; "FALSE"; "false" };
  };

  startElement(namespace: string, lname: string, qname: string, atts: csco_xml_sax_Attributes) is {
    // FIXME note this isn't quite right (but will work w/current assumptions)
    if(namespace != ns) {
      return;
    };

    if (lname == "copyright") {
      return;
    };
    if (lname == "creator") {
      return;
    };
    if (lname == "pubdate") {
      return;
    };
    
    if(current_event != NULL) {
      if (lname == "event") {
        assert(current_event.is_summary.or_all(it)) else
          error("ERR_CSCO_EVENT_POPULATE_XML_ILLEGAL_NESTING\n",
                locator, ": Unexpected leaf event ", atts.getValueQN("name"),
                " nested within another leaf event ", current_event.name);
        var new_event := new csco_event_s with {
          .parent = current_event;
        };
        parse_event(new_event, atts);
        current_event.children.add(new_event);
        current_event = new_event;
      } else {
        if (lname == "severity") {
          current_event.severity = atts.getValueQN("severity");
        } else if (lname == "action") {
          current_event.action = atts.getValueQN("action");
        } else if (lname == "description") {
          current_event.longdesc = atts.getValueQN("description");
        };
      };  
    }
    else {
      // setup root of event tree
      current_event = root_event;
      //FIXME use xml to find types
      parse_event(current_event, atts);
    };
  };

  endElement(namespace: string, lname: string, qname: string) is {
    if(namespace == ns) {
      if(current_event.parent != NULL) {
        if (lname == "event") {
          current_event = current_event.parent;
        };
      };
    };
  };

  parse_event(e: csco_event_s, atts: csco_xml_sax_Attributes) is {
    var att := atts.getValueQN("summary");
    var is_summary := att not in false;
    att = atts.getValueQN("top");
    is_summary ||= att not in false;

    e.name = atts.getValueQN("name");
    e.blockname = atts.getValueQN("blkinst");
    e.typename = atts.getValueQN("reginst");

    // FIXME XML style note: would prefer to see text blocks as element content
    e.severity = atts.getValueQN("severity");
    e.action =  atts.getValueQN("action");
    e.longdesc =  atts.getValueQN("description");

    // FIXME make this xml dependent
    var num_types:= root_event.types.size();
    e.types = root_event.types;
    e.first_idx = root_event.first_idx;
    e.is_summary.resize(num_types, TRUE, is_summary, FALSE);
    e.is_enabled.resize(num_types);
    e.actual_count.resize(num_types);
    e.min_count.resize(num_types);
    e.max_count.resize(num_types);

    // FIXME unused
    var groupoffset := atts.getValueQN("groupoffset");
  };
};

'>
