# script to extract OVL initialization messages from a logfile
# and generate specman coverage buckets

# start with e code header
BEGIN {
  print "<'"
  print "extend csco_ovl_specman_instance {"
  print "  cover fired is also {"
  print "    item id using also ranges = {"
}

# process each OVL instance
# this script uses the severity specified in the instance
# parameter to derive the coverage expectations

/^OVL_NOTE.*initialized @ / {
  sev = $7
  gsub(/,/, "", sev)
  sev = sev + 0

  # FIXME make this configurable
  if(sev <= 2)
    next              # ignore errors/warnings
  else if(sev >= 5)
    atleast = sev - 5 # use severity as bucket at_least count
  else
    atleast = 0       # default at_least count

  # pull out assertion instance path
  base = $5
  sub(/\.ovl_init_msg$/, "", base)

  # generate bucket
  spew_bucket(instance_id(base ".ovl_error"), base, atleast)
}

END {
  print "    };\n  };\n};\n'>"
}

#------------------------------------------------------------
# output a specman coverage bucket definition

function spew_bucket(id, name, atleast) {
  if(atleast)
    printf "range([0x%08x], \"%s\", UNDEF, %d);\n", id, name, atleast
  else
    printf "range([0x%08x], \"%s\");\n", id, name
}

#------------------------------------------------------------
# calculate the 32-bit bucket ID (hash)
# of a OVL module instance path

function instance_id(path,    hash, i) {
  hash = 0
  for(i = 1; i <= length(path); i++) 
    hash = ((hash * 31) + ord[substr(path, i, 1)]) % 0x100000000
  return hash
}

# initialize hash for converting characters to equivalent ASCII code
BEGIN {
  for(i = 0; i < 127; i++) ord[sprintf("%c", i)] = i
}
