// ----------------------------------------------------------------------------
//
// File name    : csco_ovl_specman_top.e
//              :
//              :
// Description  : OVL-Specman integration package
//              : (use with associated package ovl_task.h)
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2004-2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_ovl_specman;

define CSCO_OVL_SPECMAN_VERSION_2_0;
define CSCO_OVL_SPECMAN_VERSION_2_0_OR_LATER;
define CSCO_OVL_SPECMAN_VERSION_1_0_OR_LATER;

import evc_util/e/evc_util_top;

//========================================================================
// API definition
//========================================================================

// API debug messages
extend message_tag : [ CSCO_OVL_SPECMAN ];

// user-configurable action to take when an event occurs
// based on verilog severities defined in ovl_header.h
type csco_ovl_specman_severity_t : [
  // causes specman dut_error() w/ERROR check_effect
  // (stops the running simulation)
  FATAL_ERROR     = 0,
  // causes specman dut_error() w/ERROR_CONTINUE check_effect
  // (simulation continues running and fails at the end)
  NON_FATAL_ERROR = 1,
  // causes specman dut_error() w/WARNING check_effect
  // (simulation may still pass)
  WARNING         = 2,
  // generates messagef() logging of all triggered events
  NOTE_ALL        = 3,
  // messagef() log of first event, subsequent events are IGNOREd
  NOTE_FIRST      = 4,
  // messagef() log of first N events, subsequence events are IGNOREd
  // FIXME runtime updates make this NOTE_NEXT_N!
  NOTE_FIRST_N    = 5,
  // as a parameter to (set|start)_match*(),
  // current event severity setting will be unaffected
  // (only coverage setting will be updated)
  UNCHANGED       = 0xffff_fffe,
  IGNORE          = 0xffff_ffff
] (bits: 32);

// coverage setting for an ovl instance
type csco_ovl_specman_coverage_t : [
  // no functional coverage will be collected
  UNCOVERED,
  // event occurrences will collected via specman functional coverage
  COVERED,
  // as a parameter to (set|start)_match*(),
  // current coverage setting will be unaffected
  // (only event severity will be updated)
  UNCHANGED
];

// user method_port type to receive ovl notifications
method_type csco_ovl_specman_hook_t(ovl: csco_ovl_specman_instance);

//------------------------------------------------------------
// global instance of the single interface/control unit
    
extend sys {
  csco_ovl_specman : csco_ovl_specman_u is instance;
    // default connection is via stubs file
    keep soft csco_ovl_specman.hdl_path() == "~/specman";
};

//------------------------------------------------------------
// toplevel container unit for OVL related functionality

unit csco_ovl_specman_u like any_env {
  short_name() : string is { result = "OVL" };

  // enable event conversion
  // generatable, settable, clearable, etc
  enable : bool;
    keep soft enable;

  enable_match_check : bool;
    keep soft enable_match_check;

  //----------------------------------------
  // set interface
  //   configure severity and coverage behavior of assertion instances
  //   these are "one-shot" updates to the current state w/no history
  //   or undo capability

  // configure severity of all assertion instances which match a RE
  set_match(pattern: string,
            severity: csco_ovl_specman_severity_t,
            coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect only 1 assertion instance to match
  // triggers an error if multiple instances match
  set_match1(pattern: string,
             severity: csco_ovl_specman_severity_t,
             coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect exactly N assertion instances to match
  // triggers an error if number of matching instances is not N
  set_matchN(n: uint,
             pattern: string,
             severity: csco_ovl_specman_severity_t,
             coverage: csco_ovl_specman_coverage_t) is undefined;

  // revert parameters to verilog instanced defaults
  reset_match(pattern: string) is undefined;

  //----------------------------------------
  // start/stop interface
  //   create a temporary window of alternate severity and
  //   coverage behavior which can be turned on/off/reset

  // configure severity of all assertion instances which match a RE
  start_match(pattern: string,
              severity: csco_ovl_specman_severity_t,
              coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect only 1 assertion instance to match
  // triggers an error if multiple instances match
  start_match1(pattern: string,
               severity: csco_ovl_specman_severity_t,
               coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect exactly N assertion instances to match
  // triggers an error if number of matching instances is not N
  start_matchN(n: uint,
               pattern: string,
               severity: csco_ovl_specman_severity_t,
               coverage: csco_ovl_specman_coverage_t) is undefined;

  // revert parameters to values before set_match() call
  stop_match(pattern: string) is undefined;

  //----------------------------------------
  // user messaging and hook port

  // display general status information
  //show_status() is undefined; // pre-defined for any_env

  // display detailed status about OVL instances
  // which have been affected from their default state
  show_detail() is undefined;

  // user extensible hook called for every OVL event
  hook : out method_port of csco_ovl_specman_hook_t is instance;
    keep bind(hook, empty);

  // convenient logger for OVL NOTEs
  logger : message_logger is instance;
    keep soft logger.verbosity == MEDIUM;
    keep soft logger.tags == { NORMAL };

  // alternate interface to obtain an event instance object
  get_instance(path: string) : csco_ovl_specman_instance is undefined;

  // retrieve the list of all assertion instances.
  // this will only work once the simulator has
  // initialized assertions with specman (after the first tick)
  get_all_instances() : list of csco_ovl_specman_instance is undefined;

  // write coverage definitions to the specified file
  // for the currently initialized assertions.
  // only assertions set to COVERED will be written
  write_coverage(filename: string) is undefined;

  // check that the loaded coverage bucket definitions
  // match the current configuration
  assert_cov_cfg_consistent() is undefined;
  is_cov_cfg_consistent() : bool is undefined;

  //----------------------------------------
  // random statistics

  // total number of events processed by this module
  get_total_events() : uint is undefined;

  // maximum number of events processed in a single tick
  get_max_tick_events() : uint is undefined;
};


//------------------------------------------------------------
// unit level hierarchical interface
// incorporates calling unit's hdl_path() with the search pattern

extend any_unit {
  //----------------------------------------
  // set interface
  //   configure severity and coverage behavior of assertion instances
  //   these are "one-shot" updates to the current state w/no history
  //   or undo capability

  // configure severity of all assertion instances which match a RE
  csco_ovl_specman_set_match(pattern: string,
                             severity: csco_ovl_specman_severity_t,
                             coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect only 1 assertion instance to match
  // triggers an error if multiple instances match
  csco_ovl_specman_set_match1(pattern: string,
                              severity: csco_ovl_specman_severity_t,
                              coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect exactly N assertion instances to match
  // triggers an error if number of matching instances is not N
  csco_ovl_specman_set_matchN(n: uint,
                              pattern: string,
                              severity: csco_ovl_specman_severity_t,
                              coverage: csco_ovl_specman_coverage_t) is undefined;

  // revert parameters to verilog instanced defaults
  csco_ovl_specman_reset_match(pattern: string) is undefined;

  //----------------------------------------
  // start/stop interface
  //   create a temporary window of alternate severity and
  //   coverage behavior which can be turned on/off/reset

  // configure severity of all assertion instances which match a RE
  csco_ovl_specman_start_match(pattern: string,
                               severity: csco_ovl_specman_severity_t,
                               coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect only 1 assertion instance to match
  // triggers an error if multiple instances match
  csco_ovl_specman_start_match1(pattern: string,
                                severity: csco_ovl_specman_severity_t,
                                coverage: csco_ovl_specman_coverage_t) is undefined;

  // expect exactly N assertion instances to match
  // triggers an error if number of matching instances is not N
  csco_ovl_specman_start_matchN(n: uint,
                                pattern: string,
                                severity: csco_ovl_specman_severity_t,
                                coverage: csco_ovl_specman_coverage_t) is undefined;

  // revert parameters to values before set_match() call
  csco_ovl_specman_stop_match(pattern: string) is undefined;

  //----------------------------------------
  // alternate interface to obtain an event instance object
  csco_ovl_specman_get_instance(path: string) : csco_ovl_specman_instance is undefined;
};


//------------------------------------------------------------
// event instance object
// referenced by hook port when event instance is fired

struct csco_ovl_specman_instance {
  // full HDL path of event instance
  get_hdl_path() : string is undefined;

  // hash id for defining coverage buckets
  get_coverage_id() : uint is undefined;

  // string message from verilog instantiation
  get_description() : string is undefined;
  // trigger specific message used by some OVLs
  get_cause() : string is undefined;

  // number of times this ovl instance has fired
  get_count() : uint is undefined;

  // generate errors/warnings/notes
  get_severity() : csco_ovl_specman_severity_t is undefined;
  get_instanced_severity() : csco_ovl_specman_severity_t is undefined;
  set_severity(severity: csco_ovl_specman_severity_t) is undefined;
  set_severity_note_next_n(n: int) is undefined;

  // collect functional coverage for this instance?
  get_coverage() : csco_ovl_specman_coverage_t is undefined;
  set_coverage(coverage: csco_ovl_specman_coverage_t) is undefined;

  // coverage event
  event fired;
};


//========================================================================
// implementation details
//========================================================================

package type csco_ovl_specman_match_cmd_t : [ SET, RESET, START, STOP ];

package struct csco_ovl_specman_match_def {
  package cmd : csco_ovl_specman_match_cmd_t;
  n : uint;
  pattern : string;
  severity : csco_ovl_specman_severity_t;
  coverage : csco_ovl_specman_coverage_t;
};


//------------------------------------------------------------
extend csco_ovl_specman_u {
  // connect to the simulator event counter
  sim_event_count : in simple_port of uint is instance;
    keep bind(sim_event_count, external);
    keep soft sim_event_count.hdl_path() == "csco_ovl_specman_count";

  // coverage config consistency check
  !loaded_cov_cfg_id : uint;

  // total events processed by specman
  package !total_events : uint;
  package !max_tick_events : uint;
  package !event_hist[8] : list of uint;

  package !initialized : bool;

  // list of OVL instances encountered so far
  package !instances : list (key: path) of csco_ovl_specman_instance;

  // list of match patterns to apply to new instances
  package !patterns : list of csco_ovl_specman_match_def;
  // current cmd
  package !match : csco_ovl_specman_match_def;

  init() is also {
    match = new;
  };

  // coverage id collision checking
  package !coverage_ids : list (key: it) of uint;

  // read-only accessors
  get_total_events() : uint is { result = total_events };
  get_max_tick_events() : uint is { result = max_tick_events };
  get_instance(path: string) : csco_ovl_specman_instance is {
    result = instances.key(path);
  };

  assert_cov_cfg_consistent() is {
    assert(is_cov_cfg_consistent()) else
      error("ERR_CSCO_OVL_SPECMAN_COV_CFG_INCONSISTENT\n",
            "the OVL coverage configuration loaded with the environment\n",
            "does not match the current configuration.\n",
            "this will not change test behavior, but tests run under this env",
            "may have unexpected coverage results (missing and/or extra holes)",
            "hint: you can fix this by using:\n",
            "    write ovl coverage <file>\n",
            "and installing the updated file in your environment");
  };
  is_cov_cfg_consistent() : bool is {
    result = (loaded_cov_cfg_id == get_current_cov_cfg_id());
  };
  get_loaded_cov_cfg_id() : uint is { result = loaded_cov_cfg_id };
  get_current_cov_cfg_id() : uint is {
    var crc : uint;
    var sig : list of byte;
    sig = %{instances.all(.coverage == COVERED).coverage_id.sort(it)};
    result = sig.crc_32(0, sig.size())
  };

  get_all_instances() : list of csco_ovl_specman_instance is {
    assert(initialized) else
      error("ERR_CSCO_OVL_SPECMAN_UNINITIALIZED\n",
            "get_all_instances() may not be called until after the first tick\n",
            "hint: try adding a delay before assertion processing:\n",
            "    wait cycle @sys.tick_start;\n",
            "    var ovls := csco_ovl_specman.get_all_instances();\n",
            "    <process assertion instances>");
    result = instances.as_a(list of csco_ovl_specman_instance);
  };

  // random utility
  dump_matches(pattern: string) is {
    for each in instances {
      if(.path ~ pattern) {
        outf("    [%x] ", index);
        .show_detail();
      };
    };
  };

  //----------------------------------------
  set_match(pattern: string,
            severity: csco_ovl_specman_severity_t,
            coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = SET;
    match.n = 0;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  set_match1(pattern: string,
             severity: csco_ovl_specman_severity_t,
             coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = SET;
    match.n = 1;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  set_matchN(n: uint,
             pattern: string,
             severity: csco_ovl_specman_severity_t,
             coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = SET;
    match.n = n;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  reset_match(pattern: string) is {
    match.cmd = RESET;
    match.n = 0;
    match.pattern = pattern;
    _apply_match(match);
  };

  start_match(pattern: string,
              severity: csco_ovl_specman_severity_t,
              coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = START;
    match.n = 0;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  start_match1(pattern: string,
               severity: csco_ovl_specman_severity_t,
               coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = START;
    match.n = 1;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  start_matchN(n: uint,
               pattern: string,
               severity: csco_ovl_specman_severity_t,
               coverage: csco_ovl_specman_coverage_t) is {
    match.cmd = START;
    match.n = n;
    match.pattern = pattern;
    match.severity = severity;
    match.coverage = coverage;
    _apply_match(match);
  };

  stop_match(pattern: string) is {
    match.cmd = STOP;
    match.n = 0;
    match.pattern = pattern;
    _apply_match(match);
  };

  package _relative_pattern(pattern: string, base: string) : string is {
    base = str_replace(str_replace(base, "/^[~][/]/", ""), "/", ".");
    if(str_sub(pattern, 0, 1) == "/") {
      // AWK-style RE
      result = append("/", base, "[.]", str_sub(pattern, 1, MAX_INT));
    }
    else {
      result = append(base, ".", pattern);
    };
  };

  //----------------------------------------
  // handle all match interface variations
  package _apply_match(match: csco_ovl_specman_match_def) is {
    if((match.cmd == START) || ((match.cmd in [SET, RESET]) && !initialized)) {
      patterns.add(match);
      me.match = new;
    };

    var depth := patterns.size();
    if(match.cmd == STOP) {
      depth = patterns.last_index(.pattern == match.pattern);
      assert(depth >= 0) else
        error("ERR_CSCO_OVL_SPECMAN_UNEXPECTED_STOP\n",
              "stop_match(): corresponding start_match() pattern not found:\n",
              "    \"", match.pattern, "\"");
      patterns.delete(depth);
    };

    var count := 0;
    for each (ovl) in instances {
      if(ovl._apply_match(match, depth)) {
        count += 1;
        if(match.cmd == STOP) {
          _apply_stop(ovl, depth);
        };
      };
    };

    // FIXME different RESET & STOP formatting
    if(initialized) {
      messagef(CSCO_OVL_SPECMAN, MEDIUM,
               "%s %s/%s 0x%x instances matching \"%s\"\n",
               match.cmd, match.severity, match.coverage,
               count, match.pattern) {
        if(at_message_verbosity(HIGH)) {
          dump_matches(match.pattern);
        };
      };
    } else if(match.cmd != STOP) {
      messagef(CSCO_OVL_SPECMAN, MEDIUM,
              "%s %s/%s deferred matching until first simulator tick \"%s\"\n",
              match.cmd, match.severity, match.coverage, match.pattern);
    };

    if( (match.n > 0) && (match.n != count) &&
        enable_match_check && initialized ) {
      out("matched instances:");
      dump_matches(match.pattern);
      assert(FALSE) else
        error("ERR_CSCO_OVL_SPECMAN_UNEXPECTED_MATCHES\n",
              appendf("pattern \"%s\"\nmatched 0x%x instances (!= 0x%x)\n",
                      match.pattern, count, match.n));
    };
  };

  // STOP algorithm implementation
  package _apply_stop(ovl: csco_ovl_specman_instance, depth: int) is {
    var sev_set := FALSE;
    var cov_set := FALSE;
    for each (m) in reverse (patterns) {
      if(ovl.path ~ m.pattern) {
        if(index >= depth) {
          sev_set ||= (m.severity != UNCHANGED);
          cov_set ||= (m.coverage != UNCHANGED);
        }
        else {
          if(!sev_set) {
            if(index <= ovl.default_severity_depth) {
              sev_set = TRUE;
              ovl.set_severity(ovl.default_severity);
            }
            else if(m.severity != UNCHANGED) {
              sev_set = TRUE;
              ovl.set_severity(m.severity);
            };
          };
          if(!cov_set) {
            if(index <= ovl.default_coverage_depth) {
              cov_set = TRUE;
              ovl.set_coverage(ovl.default_coverage);
            }
            else if(m.coverage != UNCHANGED) {
              cov_set = TRUE;
              ovl.set_coverage(m.coverage);
            };
          };
        };
        if(sev_set && cov_set) {
          break;
        };
      };
    };
    if(!sev_set) {
      // reset to instanced severity
      ovl.set_severity(ovl.default_severity);
    };
    if(!cov_set) {
      // reset to instanced coverage
      ovl.set_coverage(ovl.default_coverage);
    };
  };

  // an excellent string hashing function.
  // adapted from glib's g_str_hash().
  // Investigation by Karl Nelson <kenelson@ece.ucdavis.edu>.
  // Do a web search for "g_str_hash X31_HASH" if you want to know more.
  // this is the same function as that described in Kernighan and Pike,
  // "The Practice of Programming", section 2.9
  // (unfortunately, specman doesn't give us access to it's hash function)
  ascii_hash(ascii: list of byte) : uint is {
    for each (char) in ascii {
      result = (result << 5) - result + char;  // result * 31 + char
    };
  };

  // lookup an OVL instance, create if it does not exist
  package _lookup(path: string) : csco_ovl_specman_instance is {
    result = instances.key(path);
    if(result == NULL) {
      result = new csco_ovl_specman_instance with {
        .path = path;
        .coverage_id = ascii_hash(path.as_a(list of byte));
      };
      instances.add(result);

      // coverage id collision checking
      // unlikely, but doesn't hurt to check
      if(!coverage_ids.key_exists(result.coverage_id)) {
        coverage_ids.add(result.coverage_id);
      } else {
        var collisions := instances.all(.coverage_id == result.coverage_id);
        warning("WARN_CSCO_OVL_SPECMAN\n",
                "OVL instance coverage id collision: id=",
                result.coverage_id, "\n    ",
                str_join(collisions.apply(.path), "\n    "), "\n");
      };
    };
  };

  show_banner() is also {
    out("Copyright 2004-2006 (c) Cisco Systems Inc.");
    show_config();
  };

  show_config() is {
    outf("    enable : %s\n", enable);
  };

  _update_hist(severity: csco_ovl_specman_severity_t) is {
    var sevint := severity.as_a(int);
    if(sevint > csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int)) {
      sevint = csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int);
    };
    sys.csco_ovl_specman.event_hist[sevint] += 1;
  };


  show_status() is also {
    outf("%s OVL/specman integration layer status:\n", e_path());
    show_config();
    outf("    initialized instances   : 0x%x\n", instances.size());
    outf("       FATAL_ERROR    : 0x%x\n",
         instances.count(.severity == FATAL_ERROR));
    outf("       NON_FATAL_ERROR: 0x%x\n",
         instances.count(.severity == NON_FATAL_ERROR));
    outf("       WARNING        : 0x%x\n",
         instances.count(.severity == WARNING));
    outf("       NOTE_ALL       : 0x%x\n",
         instances.count(.severity == NOTE_ALL));
    outf("       NOTE_FIRST     : 0x%x\n",
         instances.count(.severity == NOTE_FIRST));
    outf("       NOTE_FIRST_N   : 0x%x\n",
         instances.count((.severity.as_a(int) - csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int)) >= 0));
    outf("       IGNORE         : 0x%x\n",
         instances.count(.severity == IGNORE));
    outf("       COVERED        : 0x%x\n",
         instances.count(.coverage == COVERED));
    outf("    active match patterns   : 0x%x\n", patterns.size());
    outf("    maximum tick events     : 0x%x\n", max_tick_events);
    outf("    total events processed  : 0x%x\n", total_events);
    outf("       FATAL_ERROR    : 0x%x\n", event_hist[0]);
    outf("       NON_FATAL_ERROR: 0x%x\n", event_hist[1]);
    outf("       WARNING        : 0x%x\n", event_hist[2]);
    outf("       NOTE_ALL       : 0x%x\n", event_hist[3]);
    outf("       NOTE_FIRST     : 0x%x\n", event_hist[4]);
    outf("       NOTE_FIRST_N   : 0x%x\n", event_hist[5]);
    outf("       IGNORE         : 0x%x\n", event_hist[6]);
    outf("       COVERED        : 0x%x\n", event_hist[7]);
  };

  show_detail() is also {
    show_status();
    for each in instances {
      .show_detail();
    };
  };

  // insert some interface code and defines into verilog stubs
  verilog code {
    // these defines may or not be usable, depending on compile
    "`include \"csco_ovl_specman_defines.h\"";
    "integer csco_ovl_specman_count;";
    "initial csco_ovl_specman_count = 0;";
    "reg [`CSCO_OVL_SPECMAN_MAX_INST_W-1:0] csco_ovl_specman_head;";
    "initial csco_ovl_specman_head = 0;";
  };

  write_coverage(filename: string) is also {
    if(filename !~ "/\.e$/") {
      filename = append(filename, ".e");
    };

    var cov := files.open(filename, "w", "OVL coverage definitions");
    // preamble
    files.write(cov, append("<", "'"));
    files.write(cov, "extend csco_ovl_specman_u {");
    files.write(cov, "    init() is also {");
    writef(cov,      "        loaded_cov_cfg_id = 0x%08x;\n",
           get_current_cov_cfg_id());
    files.write(cov, "    };");
    files.write(cov, "};");
    files.write(cov, "extend csco_ovl_specman_instance {");
    files.write(cov, "    cover fired is also {");
    files.write(cov, "        item coverage_id using also ranges = {");

    var count := 0;
    for each (ovl) in instances {
      if(ovl.coverage == COVERED) {
        count += 1;
        if(ovl.instanced_severity not in [FATAL_ERROR..NOTE_FIRST]) {
          var atleast := (ovl.instanced_severity.as_a(uint) -
                          csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(uint));
          writef(cov, "range([0x%08x], \"%s\", UNDEF, %d);\n",
                 ovl.coverage_id, ovl.path, atleast);
        }
        else {
          writef(cov, "range([0x%08x], \"%s\");\n", ovl.coverage_id, ovl.path);
        };
      };
    };

    // closing
    files.write(cov, "        };");
    files.write(cov, "    };");
    files.write(cov, "};");
    files.write(cov, append("'", ">"));
    files.close(cov);

    out("wrote ", count, " coverage definitions from ", instances.size(),
        " assertions to ", filename);

    if(instances.is_empty()) {
      warning("no instances were available to write coverage for\n",
              "(hint: run the first tick between specman and the simulator)");
    };
  };
};

define <csco_ovl_specman_write_ovl_coverage'command>
    "write ovl coverage <file>" as {
  sys.csco_ovl_specman.write_coverage("<file>");
};


//------------------------------------------------------------
extend any_unit {
  // set interface
  csco_ovl_specman_set_match(pattern: string,
                             severity: csco_ovl_specman_severity_t,
                             coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.set_matchN(0, pattern, severity, coverage);
  };

  csco_ovl_specman_set_match1(pattern: string,
                              severity: csco_ovl_specman_severity_t,
                              coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.set_matchN(1, pattern, severity, coverage);
  };

  csco_ovl_specman_set_matchN(n: uint,
                              pattern: string,
                              severity: csco_ovl_specman_severity_t,
                              coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.set_matchN(n, pattern, severity, coverage);
  };

  csco_ovl_specman_reset_match(pattern: string) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.reset_match(pattern);
  };

  // start/stop interface
  csco_ovl_specman_start_match(pattern: string,
                               severity: csco_ovl_specman_severity_t,
                               coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.start_matchN(0, pattern, severity, coverage);
  };

  csco_ovl_specman_start_match1(pattern: string,
                                severity: csco_ovl_specman_severity_t,
                                coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.start_matchN(1, pattern, severity, coverage);
  };

  csco_ovl_specman_start_matchN(n: uint,
                                pattern: string,
                                severity: csco_ovl_specman_severity_t,
                                coverage: csco_ovl_specman_coverage_t) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.start_matchN(n, pattern, severity, coverage);
  };

  csco_ovl_specman_stop_match(pattern: string) is {
    pattern = sys.csco_ovl_specman._relative_pattern(pattern, full_hdl_path());
    sys.csco_ovl_specman.stop_match(pattern);
  };

  // alternate interface to obtain an event instance object
  csco_ovl_specman_get_instance(path: string) : csco_ovl_specman_instance is {
    path = sys.csco_ovl_specman._relative_pattern(path, full_hdl_path());
    return(sys.csco_ovl_specman.get_instance(path));
  };
};


//------------------------------------------------------------
extend enable csco_ovl_specman_u {
  // emit an event each time the error count changes
  event sim_count_change is change(sim_event_count$)@sim;

  // translate event counter increments into specman activities
  on sim_count_change {
    if((sim_event_count$ - total_events) > max_tick_events) {
      max_tick_events = sim_event_count$ - total_events;
    };

    // retrieve first event hdl instance path
    var bytes : list of byte = 'csco_ovl_specman_head';

    while(total_events < sim_event_count$) {
      // verilog to specman string conversion:
      // (FIXME find more efficient version?)
      // bytes.as_a(string).as_a(list of byte).reverse().as_a(string) // orig
      //   - chop off any trailing 0 bytes (by converting to string)
      //   - reverse the order (after going back to bytes)
      //   - and convert back to string
      // bytes.all(it != 0).reverse().as_a(string) // saves 1 conversion
      // ...also remove trailing task name
      var task_idx := bytes.first_index(it == 0x2e); // '.'
      var inst := bytes.all((it != 0) && (index > task_idx)).reverse().as_a(string);

      // retrieve data for each OVL event triggered during the tick
      var ovl := _lookup(inst);

      // update instance message
      // retrieve the message to print
      bytes = '~/(inst).sn_msg';

      if(initialized) {
        ovl.cause = bytes.all(it != 0).reverse().as_a(string);
        ovl.count += 1;
        // perform any user actions first
        hook$(ovl);
        // so they can affect internal actions
        ovl._trigger();
      }
      else {
        // initialize new OVL event instance
        var sev : csco_ovl_specman_severity_t = '~/(inst).sn_sev';
        ovl.instanced_severity = sev;
        ovl.set_severity(sev);
        ovl.default_severity = sev;

        var cov := ovl._inferred_coverage(sev);
        ovl.set_coverage(cov);
        ovl.default_coverage = cov;

        ovl.description = bytes.all(it != 0).reverse().as_a(string);
        ovl.cause = "*INITIALIZED*";
      };

      // next event
      total_events += 1;
      bytes = '~/(inst).sn_next';
    };

    if(!initialized) {
      initialized = TRUE;
      var init_patterns := patterns;
      patterns = { };
      // apply outstanding match patterns
      // (FIXME integrate w/apply_stop algorithm)
      for each (m) in init_patterns {
        _apply_match(m);
      };
    };
  };
};


//------------------------------------------------------------
extend csco_ovl_specman_instance {
  package path : string;
  package severity : csco_ovl_specman_severity_t;
  package coverage : csco_ovl_specman_coverage_t;
  package coverage_id : uint;
  // FIXME decompose these to enumerated types
  package description : string;
  package cause : string;
  package count : uint;

  // retain original severity
  package instanced_severity : csco_ovl_specman_severity_t;
  // fallback defaults 
  package default_severity : csco_ovl_specman_severity_t;
  package default_coverage : csco_ovl_specman_coverage_t;
  // fallback match depth
  package default_severity_depth: int;
  package default_coverage_depth: int;

  init() is also {
    coverage = COVERED;
  };

  cover fired is {
    item coverage_id;
  };

  // read-only user accessors
  get_hdl_path() : string is { result = path };
  get_coverage_id() : uint is { result = coverage_id };
  get_description() : string is { result = description };
  get_cause() : string is { result = cause };
  get_count() : uint is { result = count };
  get_severity() : csco_ovl_specman_severity_t is { result = severity };
  get_instanced_severity() : csco_ovl_specman_severity_t is {
    result = instanced_severity;
  };
  get_coverage() : csco_ovl_specman_coverage_t is { result = coverage };

  // write accessors
  set_severity(sev: csco_ovl_specman_severity_t) is {
    assert(sev != UNCHANGED) else
      error("ERR_CSCO_OVL_SPECMAN_ILLEGAL_SEVERITY\n",
            "attempt to set event severity field to illegal value:\n",
            "    severity=", sev, " for ", path);
    var total_count := '~/(path)/error_count';
    if(total_count < count) {
      sys.csco_ovl_specman._update_hist(severity);
      count = total_count;
    };
    severity = sev;
    if((coverage == COVERED) && (sev == IGNORE)) {
      // get verilog events for coverage reporting
      sev = NOTE_ALL;
    }
    else if(_ignored()) {
      // safely ignore verilog events
      sev = IGNORE;
    };
    '~/(path)/sn_sev' = sev;
  };

  set_severity_note_next_n(n: int) is {
    set_severity((count + csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int) + n).as_a(csco_ovl_specman_severity_t));
  };

  set_coverage(cov: csco_ovl_specman_coverage_t) is {
    assert(cov != UNCHANGED) else
      error("ERR_CSCO_OVL_SPECMAN_ILLEGAL_COVERAGE\n",
            "attempt to set event coverage field to illegal value:\n",
            "    coverage=", cov, " for ", path);
    coverage = cov;
  };

  show_detail() is {
    out("    ", severity, "/", coverage, " count=", count,
        " : ", path);
  };

  // package internal implementation
  package _inferred_coverage(sev: csco_ovl_specman_severity_t) : csco_ovl_specman_coverage_t is {
    if(sev in [FATAL_ERROR, NON_FATAL_ERROR, IGNORE]) {
      result = UNCOVERED;
    } else {
      result = COVERED;
    };
  };

  package _ignored() : bool is {
    result = ( (coverage == UNCOVERED) &&
               ( (severity == IGNORE) ||
                 ((severity not in [FATAL_ERROR..NOTE_ALL]) && !_noted()) ));
  };

  package _noted() : bool is {
    result = ( (severity == NOTE_ALL) ||
               ((severity == NOTE_FIRST) && (count <= 1)) ||
               (count + csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int) <= severity.as_a(int)) )
  };

  package _severity_str() : string is {
    if(severity.as_a(int) <= csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int)) {
      result = severity.as_a(string);
    } else {
      var n := (severity.as_a(int) - csco_ovl_specman_severity_t'NOTE_FIRST_N.as_a(int));
      if(count <= n) {
        result = append("NOTE_NEXT(", n - count, ")");
      } else {
        result = append("NOTE_FIRST(", n, ")");
      };
    }
  };

  package _trigger() is {
    // handle coverage first
    if(coverage == COVERED) {
      sys.csco_ovl_specman.event_hist[7] += 1;
      emit fired;
    };

    // take action based on severity
    if(_noted()) {
      sys.csco_ovl_specman._update_hist(severity);
      messagef(MEDIUM, "%s : %s : %s\n", _severity_str(), description, cause) {
        if(at_message_verbosity(HIGH)) {
          out(path);
        };
      };
    }
    else if(severity in [FATAL_ERROR..WARNING]) {
      sys.csco_ovl_specman._update_hist(severity);
      dut_error("ERR_CSCO_OVL_SPECMAN_EVENT\n",
                "OVL ", _severity_str(), " : ", description, " : ", cause, "\n",
                path);
    }
    else {
      sys.csco_ovl_specman.event_hist[6] += 1;
      if(coverage == UNCOVERED) {
        // disable event in verilog, avoid passing to specman
        '~/(path).sn_sev' = csco_ovl_specman_severity_t'IGNORE;
      };
    };
  };

  package _apply_match(match: csco_ovl_specman_match_def, depth: int) : bool is {
    if(match.cmd == STOP) {
      if(default_severity_depth > depth) {
        default_severity_depth -= 1;
        assert(default_severity_depth >= 0);
      };
      if(default_coverage_depth > depth) {
        default_coverage_depth -= 1;
        assert(default_coverage_depth >= 0);
      };
    };

    result = path ~ match.pattern;
    if(result) {
      if((severity == IGNORE) ||
         severity.as_a(int) >= csco_ovl_specman_severity_t'NOTE_FIRST.as_a(int)) {
        // refresh ignored count
        var count : uint = '~/(path)/error_count'
      };
      if(match.cmd == RESET) {
        // FIXME apply STARTs or not? (currently not)
        match.severity = instanced_severity;
        match.coverage = _inferred_coverage(match.severity);
      };
      case(match.cmd) {
        [SET, RESET]: {
          if(match.severity != UNCHANGED) {
            set_severity(match.severity);
            default_severity = severity;
            default_severity_depth = depth;
          };
          if(match.coverage != UNCHANGED) {
            set_coverage(match.coverage);
            default_coverage = coverage;
            default_coverage_depth = depth;
          };
        };

        START: {
          if(match.severity != UNCHANGED) {
            set_severity(match.severity);
          };
          if(match.coverage != UNCHANGED) {
            set_coverage(match.coverage);
          };
        };
      };
      '~/(path)/sn_sev' = severity;
    };
  };
};


//------------------------------------------------------------
// modify default error processing to produce the configured effect
// but _only_ if dut_error effect is one of the ERROR* effects
// (so global set_check() overrides still work correctly)

extend dut_error_struct {
  pre_error() is also {
    if(source_struct() is a csco_ovl_specman_instance (ovl) &&
       check_effect() in [ERROR, ERROR_AUTOMATIC, ERROR_CONTINUE]) {
      case(ovl.severity) {
        NON_FATAL_ERROR {
          set_check_effect(ERROR_CONTINUE);
        };
        WARNING {
          set_check_effect(WARNING);
        };
        [NOTE_ALL, NOTE_FIRST, NOTE_FIRST_N, IGNORE] {
          // should never get here...
          set_check_effect(IGNORE);
        };
      };
    };
  };
};

'>
