
`timescale 1ns/100ps
// defines to enable integration and hookup OVL modules
`ifdef CSCO_OVL_SPECMAN_INTEGRATION
`else
  `define CSCO_OVL_SPECMAN_INTEGRATION
`endif

// maximum number of events which should occur during a tick (in bits)
`ifdef CSCO_OVL_SPECMAN_MAX_TICK_EVENTS_W
`else
  `define CSCO_OVL_SPECMAN_MAX_TICK_EVENTS_W 6
`endif

// maximum size of an OVL message or instance path
`ifdef CSCO_OVL_SPECMAN_MAX_MSG_W
`else
  `define CSCO_OVL_SPECMAN_MAX_MSG_W (256 * 8)
`endif

// new toplevel module to collect OVL events
// for processing by specman
module csco_ovl_specman_top();
  parameter max_tick_events_w = `CSCO_OVL_SPECMAN_MAX_TICK_EVENTS_W;
  parameter max_tick_events = 1 << max_tick_events_w;
  parameter max_msg_w = `CSCO_OVL_SPECMAN_MAX_MSG_W;

  // count of total events collected so far
  reg [31:0] event_count;
  initial event_count = 0;

  // string transfer buffers
  reg [max_msg_w - 1:0] instance;
  reg [max_msg_w - 1:0] message;

  // buffer for OVL events which occurred during current tick
  reg [7:0] severities[0:max_tick_events - 1];
  reg [max_msg_w - 1:0] instances[0:max_tick_events - 1];
  reg [max_msg_w - 1:0] messages[0:max_tick_events - 1];

  initial begin
    $display("%m: OVL/specman integration - verilog initialized");
    $display("%m:   maximum events/tick: %x", max_tick_events);
    $display("%m:   maximum message len: %x", max_msg_w >> 3);
  end

  task trigger;
    input [7:0] sev;
    begin
      severities[event_count[max_tick_events_w - 1:0]] = sev;
      instances[event_count[max_tick_events_w - 1:0]] = instance;
      messages[event_count[max_tick_events_w - 1:0]] = message;
      event_count = event_count + 1;
    end
  endtask

endmodule
