// ----------------------------------------------------------------------------
//
// File name    : ovl_task.h
//              :
// Developers   : spadix, sesmith, andrewmc
//              :
// Description  : replacement OVL common tasks to integrate with specman
//              : via the csco_ovl_specman package
//              :
// ----------------------------------------------------------------------------
// >>>>>>>>>>>>>>>>>>>>>>>>>>> COPYRIGHT NOTICE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
// ----------------------------------------------------------------------------
// Copyright 2003-2006 (c) Cisco Systems Inc.
// Cisco Confidential - All rights reserved
// ----------------------------------------------------------------------------

`include "ovl_header.h"
`include "csco_ovl_specman_defines.h"

// SFORMAT_IS_A_FUNCTION:
//   define this for VCS (older versions?)
//   which has $sformat system *function*
//   instead of the now-standard task
//   (ref IEEE Std 1364-2001 17.2.3)
//   ...once VCS is up to date, this can be removed

`ifdef SFORMAT_IS_A_FUNCTION
  `define CSCO_OVL_SPECMAN_SFORMAT  length = $sformat
  `define CSCO_OVL_SPECMAN_LENGTH   integer length;
`else
  `define CSCO_OVL_SPECMAN_SFORMAT  $sformat
  `define CSCO_OVL_SPECMAN_LENGTH
`endif

  reg [`CSCO_OVL_SPECMAN_MAX_INST_W-1:0] sn_next;
  // FIXME could reduce this string size
  reg [`CSCO_OVL_SPECMAN_MAX_MSG_W-1:0] sn_msg;
  integer sn_sev;

  task ovl_error;
      input [8*63:0] event_msg;
    begin
      error_count = error_count + 1;
      if(sn_sev >= 0) begin
        // FIXME enums would be better than verilog strings...
        sn_msg = event_msg;
        ovl_specman_link;
      end
    end
  endtask

  task ovl_init_msg;
    $display("OVL_NOTE: %s initialized @ %m Severity: %0d, Message: %s",
             assert_name,severity_level, msg);
  endtask

  initial begin : sn_init
    `CSCO_OVL_SPECMAN_LENGTH
    sn_sev = severity_level;
    `CSCO_OVL_SPECMAN_SFORMAT(sn_msg, "%s : %s", assert_name, msg);
    #0 ovl_specman_link;
  end

  task ovl_specman_link;
      `CSCO_OVL_SPECMAN_LENGTH
    begin
      // link to prior errors and clear out previous strings
      sn_next = `CSCO_OVL_SPECMAN_HEAD;
      `CSCO_OVL_SPECMAN_HEAD = 0;
      `CSCO_OVL_SPECMAN_SFORMAT(`CSCO_OVL_SPECMAN_HEAD, "%m");
      `CSCO_OVL_SPECMAN_COUNT = `CSCO_OVL_SPECMAN_COUNT + 1;
    end
  endtask
