// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_rate_stats_trans_ext.e
//              :
// Project      : csco_rate_stats
//              :
// Created On   : 11/10/2005
//              :
//              :
// Description  : This file extends the csco_trans_s base type and adds fields
//                required to support the rate/stats package.
//                
//              :
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'
package csco_rate_stats;

import csco_trans/e/csco_trans_top;


// This struct contrains all the fields necesssary to perform rate/stats
// calculations on transactions.
struct csco_trans_rate_stats_s {
  // Indicates that this transaction should not be included in rate/stats
  // calculations.
  !dont_include : bool;

  // should be set by each transaction type to indicate data 
  // size of the TR instance.
  data_size : uint;
    keep soft data_size == 0;

  // should be set by each transaction type to indicate string 
  // describing unit in which data_size is expressed.  For 
  // example, if the data size is in units of bytes, then field 
  // should be set to: "bytes"
  data_unit : string;
    keep soft data_unit == "";

  csco_print_get_numeric_field_value(pf : csco_print_field_s) : string is also {
    result = append(result, " ", data_unit);
  };
}; // struct csco_trans_rate_stats_s


extend csco_trans_s {
  // All transactions inheriting from csco_trans_s will have rate_stats 
  // support.
  rate_stats : csco_trans_rate_stats_s;
    csco_print_attr rate_stats LONG expand=ON;

  init() is also {
    if rate_stats == NULL {
      rate_stats = new;
    };
  };
};  // extend csco_trans_s


extend csco_trans_tracking_s {
  exclude_stats() is {
    get_parent_trans().rate_stats.dont_include = TRUE;
  };

  set_leaf_and_exclude_stats() is {
    get_parent_trans().rate_stats.dont_include = TRUE;
    clear_exp_child_cnt();
  };
};


'>
