// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_rate_stats_types.e
//              :
// Project      : csco_rate_stats
//              :
// Created On   : 09/19/2004
//              :
//              :
// Description  : This file contains some common type definitions for the
//                csco_rate_stats package.
//                
//              :
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'
package csco_rate_stats;


extend message_tag : [CSCO_RATE_STATS];

// Indicates whether a rate/stats I/O interface is modelling
// the ingress or the egress interface of a rate/stats stream.
type csco_rate_stats_io_dir_t : [INPUT, OUTPUT];

// Name of a rate/stats stream. Used for per-instance when-subtyping.
type csco_rate_stats_stream_name_t: [NONE];

// Whether a rate/stats stream is physical or merged. A physical stream
// has input and output I/O interfaces where rate/stats are captured. A
// merged stream is simply a collection of physical streams whose rate/stats
// info are merged together to resemble a single stream.
type csco_rate_stats_stream_kind_t: [PHYSICAL, MERGED];

// Algorithm for calculating transaction/data rate over an interface. 
// DROP_FIRST : caclulate elapsed time using the end times of the 1st and last
//              transactions. The 1st transaction is dropped.
// DROP_LAST : caclulate elapsed time using the start times of the 1st and last
//             transactions. The last transaction is dropped.
// AVG_GAP : calculate elapsed time using the time between the start of the 1st 
//           packet and the end of the last packet plus the avg gap between
//           all packets
type csco_rate_stats_calculation_type: [DROP_FIRST, DROP_LAST, AVG_GAP];

type csco_rate_stats_merge_alg_t : [ALL_TRANS, ACTIVE_OVERLAP, MANUAL];


'>
