// ----------------------------------------------------------------------------
//
// File name    : csco_tb.e
//              :
// Project      : csco_tb
//              :
// Created On   : 11/11/2005
//              :
//              :
// Description  : This file defines the base wrapper unit for all
//              : testbenches. Standard fields include tb_name, instance
//              : name, and a config struct. 
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_tb;

unit csco_tb_u like any_env {
  // name for testbench environment.
  tb_name : csco_tb_name_t;
    keep soft tb_name != NONE;
    keep soft config.tb_name == tb_name;

  // per instance when-determinant.
  inst_name : csco_tb_inst_t;
    keep soft config.inst_name == inst_name;

  !test_kind : csco_tb_test_kind_t;
  csco_config_update_subtypes() is also {
    test_kind = config.test_kind;
  };

  // whether this is the top-most csco_tb instance. soft constrained to FALSE
  // and over-written by the top-most csco_tb_u unit instantiated under sys.
  is_top : bool;
    keep soft !is_top;

  // testflow methods in csco_tb_u use sys.any as their event.
  CREATE_TESTFLOW;

  // default short_name is tb_name
  short_name() : string is { result = tb_name.as_a(string) };

  keep soft agent() == "verilog";
};


DEFINE_CFG_STRUCT begin
  config_prefix = csco_tb,
  parent_unit_type = csco_tb_u
end;

extend csco_tb_config_s {
  tb_name : csco_tb_name_t;

  inst_name : csco_tb_inst_t;

  test_kind : csco_tb_test_kind_t;
    keep soft test_kind == UNDEFINED;

  is_top : bool;
    keep soft !is_top;

  enable_auto_init : bool;
    keep soft enable_auto_init;
};


//------------------------------------------------------------------------
// if this package is included, there should be one instance of csco_tb in sys.
// each testbench that extends csco_tb_name_t may *soft* constrain
// the tb_name of this top_tb to that locally defined subtype

extend sys {
  top_tb : is_top csco_tb_u is instance;

  short_name() : string is only { };
};


//------------------------------------------------------------------------
extend csco_tb_u {
  keep soft config.is_top == read_only(is_top);

  run() is also {
    // integration checks
    assert(tb_name != NONE) else
      error("ERR_CSCO_TB_UNCONSTRAINED_NAME\n",
            "tb_name for csco_tb at cannot be NONE at:\n",
            "    ", e_path(), "\n",
            "(did you remember to soft constrain top_tb.tb_name?)");

    // FIXME use standard sys.csco_config_util method to check consistency
    assert(tb_name == config.tb_name) else
      error("ERR_CSCO_TB_UNCONSTRAINED_CONFIG_TB_NAME\n",
            "tb_name mismatch between tb and config:\n",
            "    ", e_path(), ".tb_name        == ", tb_name, "\n",
            "    ", e_path(), ".config.tb_name == ", config.tb_name, "\n",
            "this means you did not specify the correct subtype to INSTANCE_CFG\n",
            "or provide another constraint\n");

    assert(inst_name == config.inst_name) else
      error("ERR_CSCO_TB_UNCONSTRAINED_CONFIG_INST_NAME\n",
            "inst_name mismatch between tb and config:\n",
            "    ", e_path(), ".inst_name        == ", inst_name, "\n",
            "    ", e_path(), ".config.inst_name == ", config.inst_name, "\n",
            "this means you did not specify the correct subtype to INSTANCE_CFG\n",
            "or provide another constraint\n");

    assert((is_top == config.is_top) &&
           (is_top => (e_path() == "sys.top_tb"))) else
      error("ERR_CSCO_TB_ILLEGAL_IS_TOP\n",
            "don't mess with is_top!\n",
            "    ", e_path(), ".is_top        == ", is_top, "\n",
            "    ", e_path(), ".config.is_top == ", config.is_top, "\n");
  };
};

'>
