// ----------------------------------------------------------------------------
//
// File name    : csco_tb_seq.e
//              :
// Project      : csco_tb
//              :
// Created On   : 11/11/2005
//              :
//              :
// Description  : Virtual sequence driver that will always be in
//              : sys.top_tb.  Other sequence drivers will be
//              : responsible for connecting to this driver when
//              : they are instanced.
//              :
//              : All testcases written for a testbench should be a
//              : sequence of this VSD. This file creates standard
//              : test phase sequences and automatically launches them
//              : in the appropriate test phase. So the user simply needs
//              : to extend one of these sequences to define new test 
//              : logic for a test phase.
//              :
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2005-2006 (c) Cisco Systems Inc.
// ----------------------------------------------------------------------------
<'
package csco_tb;

// Create virtual sequence driver type.
sequence csco_tb_seq using
  created_driver = csco_tb_driver_u,
  created_kind = csco_tb_seq_kind_t;

extend csco_tb_driver_u {
  short_name() : string is { result = "VSD" };

  event clock is only @sys.any;

  CREATE_TESTFLOW clock;

  // reference to parent TB env, pushed down from tb env
  tb_ptr : csco_tb_u;

  // name of testbench environment
  tb_name : csco_tb_name_t;
    keep tb_name == read_only(tb_ptr.tb_name);
};

extend csco_tb_seq {
  // name of testbench environment, pushed down from the tb env.
  tb_name : csco_tb_name_t;
    keep tb_name == read_only(driver.tb_name);

  // local config reference
  config : csco_config_base_struct_s;
    keep soft config == read_only(driver.tb_ptr.config);
    keep read_only(parent_sequence is a csco_tb_seq (seq)) =>
      soft config == read_only(seq.config);
};

// standard do-nothing sequence
extend csco_tb_seq_kind_t : [ EMPTY ];


//------------------------------------------------------------
// instantiate virtual sequence driver in testbench

extend csco_tb_u {
  driver : csco_tb_driver_u is instance;
    keep driver.tb_ptr == me;
};


//------------------------------------------------------------
// standard test flow sequences for virtual sequence driver

extend csco_tb_seq_kind_t : [
  CSCO_PRE_RESET, CSCO_RESET, CSCO_INIT_TEST, CSCO_PRE_TEST, CSCO_MAIN_TEST, CSCO_POST_TEST, CSCO_POST_TEST_CHECK
];

extend CSCO_INIT_TEST'kind csco_tb_seq {
  // init sequence subtype
  init_kind : csco_tb_init_test_kind_t;
    keep soft init_kind == DEFAULT;

  // pre-sequence init control
  enable : bool;
    keep soft enable == read_only(driver.tb_ptr.config.enable_auto_init);
};

extend csco_tb_driver_u {
  // root sequence for each testflow phase
  !pre_reset_seq : CSCO_PRE_RESET'kind csco_tb_seq;
    keep pre_reset_seq.driver == me;

  csco_pre_reset() @clock is also {
    gen pre_reset_seq;
    pre_reset_seq.start_sequence();
  };

  !reset_seq : CSCO_RESET'kind csco_tb_seq;
    keep reset_seq.driver == me;

  csco_reset() @clock is also {
    gen reset_seq;
    reset_seq.start_sequence();
  };

  !init_test_seq : CSCO_INIT_TEST'kind csco_tb_seq;
    keep init_test_seq.driver == me;

  csco_init_test() @clock is also {
    gen init_test_seq;
    init_test_seq.start_sequence();
  };

  !pre_test_seq : CSCO_PRE_TEST'kind csco_tb_seq;
    keep pre_test_seq.driver == me;

  csco_pre_test() @clock is also {
    gen pre_test_seq;
    pre_test_seq.start_sequence();
  };

  !main_test_seq : CSCO_MAIN_TEST'kind csco_tb_seq;
    keep main_test_seq.driver == me;

  csco_main_test() @clock is also {
    gen main_test_seq;
    main_test_seq.start_sequence();
  };

  !post_test_seq : CSCO_POST_TEST'kind csco_tb_seq;
    keep post_test_seq.driver == me;

  csco_post_test() @clock is also {
    gen post_test_seq;
    post_test_seq.start_sequence();
  };

  !post_test_check_seq : CSCO_POST_TEST_CHECK'kind csco_tb_seq;
    keep post_test_check_seq.driver == me;

  csco_post_test_check() @clock is also {
    gen post_test_check_seq;
    post_test_check_seq.start_sequence();
  };
};

'>
