// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
//
// File name    : csco_testflow_types.e
//              :
// Project      : csco_testflow
//              :
// Created On   : 11/08/2005
//              :
//              :
// Description  : This file contains some common type definitions and 
//                instantiates a package specific utility unit in sys.
//                
//              :
// Assumptions  : none
//              :
// Limitations  : none
//              :
// Known Errors : none
//              :
// Notes        :
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Copyright 2003 (c) Cisco Systems Inc.
//
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

<'
package csco_testflow;


extend message_tag : [CSCO_TESTFLOW];

// The Specman run() phase is split into several stages
type csco_testflow_stage_t : [
  UNDEFINED, 
  CSCO_PRE_RESET_PROP,
  CSCO_PRE_RESET,
  CSCO_RESET_PROP,
  CSCO_RESET,
  CSCO_INIT_TEST_PROP,
  CSCO_INIT_TEST,
  CSCO_PRE_TEST_PROP,
  CSCO_PRE_TEST,
  CSCO_MAIN_TEST,
  CSCO_POST_TEST,
  CSCO_POST_TEST_CHECK
];

// One objection kind for each of the test phases
extend objection_kind : [
  CSCO_PRE_RESET_PROP_DONE,
  CSCO_PRE_RESET_DONE,
  CSCO_RESET_PROP_DONE,
  CSCO_RESET_DONE,
  CSCO_INIT_TEST_PROP_DONE,
  CSCO_INIT_TEST_DONE,
  CSCO_PRE_TEST_PROP_DONE,
  CSCO_PRE_TEST_DONE,
  CSCO_MAIN_TEST_DONE,
  CSCO_POST_TEST_DONE,
  CSCO_POST_TEST_CHECK_DONE
];

// Scalar subtype based on objection_kind.
type csco_testflow_objection_kind_t : objection_kind [
  CSCO_PRE_RESET_PROP_DONE..CSCO_POST_TEST_CHECK_DONE
];


// Defines the different types of reset. This type is used as the input
// argument to reset_unit(). Only the POWER_ON, COLD and WARM values should
// be used by the user. 
type csco_testflow_reset_kind_t : [CSCO_POWER_ON, CSCO_PRE_RESET, CSCO_RESET, CSCO_INIT_TEST];

// Can be extended to define additional testflow domains. By default, only
// the GLOBAL domain is defined.
type csco_testflow_domain_t : [CSCO_GLOBAL];

method_type csco_testflow_hook_t() @sys.any; 


unit csco_testflow_utils_u { };

extend sys {
  // Globally visible code space for csco_testflow logic.
  csco_testflow_utils : csco_testflow_utils_u is instance;
};


'>
