This test implements the transaction linking example from the user guide.

<'
import csco_erm_ext/e/csco_rtp_common_inits;
import csco_base_env/e/csco_base_env_top;



extend csco_trans_kind_t : [HIGH_LEVEL];

struct high_level_trans_s like csco_trans_s {
  keep tracking.trans_kind == HIGH_LEVEL;

  init() is also {
    tracking.trans_kind = HIGH_LEVEL;
    tracking.watchdog_timer = 500 ns;
  };

  high_level_fields : uint;

  print_long() is also {
    print_field_value("high_level_fields", hex(high_level_fields));
  };
};

sequence high_level_seq using
  item = high_level_trans_s,
  created_kind = high_level_seq_kind_t,
  created_driver = high_level_driver_u;



extend csco_trans_kind_t : [INPUT];

struct input_trans_s like csco_trans_s {
  keep tracking.trans_kind == INPUT;

  init() is also {
    tracking.trans_kind = INPUT;
    tracking.watchdog_timer = 500 ns;
  };

  tr_kind : [READ, RD_RESP, PACKET, WRITE];

  print_long() is also {
    print_field_value("tr_kind", tr_kind.as_a(string));
  };
};

sequence input_seq using
  item = input_trans_s,
  created_kind = input_seq_kind_t,
  created_driver = input_driver_u;



extend csco_trans_kind_t : [OUTPUT];

struct output_trans_s like csco_trans_s {
  init() is also {
    tracking.trans_kind = OUTPUT;
    tracking.watchdog_timer = 500 ns;
  };

  output_fields : uint;

  print_long() is also {
    print_field_value("output_fields", hex(output_fields));
  };
};



extend csco_tb_name_t : [CSCO_TRANS];

extend sys {
  keep top_tb.config.tb_name == CSCO_TRANS;
  keep logger.format == RTP_LONG;
  keep logger.verbosity == FULL;
  keep logger.tags == {NORMAL; CSCO_TRANS};
}; // extend sys

extend high_level_driver_u {
  CREATE_TESTFLOW;

  keep bfm_interaction_mode == PUSH_MODE;

  input_driver : input_driver_u;

  event clock is only @sys.any;

  send_to_bfm(tr : high_level_trans_s) @clock is also {
    input_driver.send_transaction(tr);
  };
};

extend input_driver_u {
  CREATE_TESTFLOW;

  bfm_ptr : input_bfm_u;

  keep bfm_interaction_mode == PUSH_MODE;

  !high_level_translate : HIGH_LEVEL_TRANSLATE input_seq;

  event clock is only @sys.any;

  send_transaction(tr : high_level_trans_s) @clock is {
    wait [100];                               // emulate queueing delay in input driver.
    gen high_level_translate keeping {
      .driver == me;
      .high_level_trans == tr;
    };
    high_level_translate.start_sequence();
  };

  send_to_bfm(tr : input_trans_s) @clock is also {
    bfm_ptr.driver_trans(tr);
  };
};

extend input_seq_kind_t : [HIGH_LEVEL_TRANSLATE];

extend HIGH_LEVEL_TRANSLATE input_seq {
  high_level_trans : high_level_trans_s;

  !read_trans : READ input_trans_s;
  !pkt_trans : PACKET input_trans_s;
  !write_trans : WRITE input_trans_s;

  body() @driver.clock is {
    high_level_trans.tracking.incr_exp_child_cnt(3);

    do read_trans;
    do pkt_trans;
    do write_trans;
  };

  mid_do(s : input_trans_s) is also {
    high_level_trans.tracking.link_trans(s);
  };
};

unit input_bfm_u {
  CREATE_TESTFLOW;

  bfm_notify : in method_port of csco_trans_forward_mp is instance;
    keep bind (bfm_notify, empty);

  !cur_read_tr : READ input_trans_s;
  !cur_pkt_tr : PACKET input_trans_s;
  !cur_write_tr : WRITE input_trans_s;

  event resp_received;

  driver_trans(tr : input_trans_s) @sys.any is {
    wait [100];                       // emulate bus delays.

    case tr.tr_kind {
      READ : { 
        cur_read_tr = tr.as_a(READ input_trans_s); 
      };
      PACKET : { 
        cur_pkt_tr = tr.as_a(PACKET input_trans_s); 
      };
      WRITE : { 
        cur_write_tr = tr.as_a(WRITE input_trans_s); 
      };
    };
    tr.tracking.incr_exp_child_cnt(1);

    sys.top_tb.cur_input_trans = tr;
    emit sys.top_tb.new_input_trans;

    if tr.tr_kind == READ {
      wait @resp_received;
    };
  };

  bfm_notify(tr : csco_trans_s) is {
    var input_tr : input_trans_s = tr.as_a(input_trans_s);

    case input_tr.tr_kind {
      READ : { 
        cur_read_tr.tracking.link_trans(input_tr);
      };
      PACKET : { 
        cur_pkt_tr.tracking.link_trans(input_tr);
      };
      WRITE : { 
        cur_write_tr.tracking.link_trans(input_tr);
      };
      RD_RESP : {
        input_tr.tracking.set_leaf();
        emit resp_received;
      };
    };
  };
};

unit input_mon_u {
  CREATE_TESTFLOW;

  tr_forward : out method_port of csco_trans_forward_mp is instance;
    keep bind (tr_forward, empty);

  event new_tr is @sys.top_tb.new_input_trans;

  on new_tr {
    start collect_trans(sys.top_tb.cur_input_trans);
  };

  collect_trans(tr : input_trans_s) @sys.any is {
    var mon_tr : input_trans_s;

    mon_tr = new;
    wait [10];                  // emulate transaction length.
    mon_tr.tr_kind = tr.tr_kind;
    tr_forward$(mon_tr);
  };
};

unit scoreboard_u {
  CREATE_TESTFLOW;

  sb_upstream_notify : in method_port of csco_trans_forward_mp is instance;
    keep bind (sb_upstream_notify, empty);

  sb_downstream_notify : in method_port of csco_trans_forward_mp is instance;
    keep bind (sb_downstream_notify, empty);

  !cur_rd_tr : input_trans_s;
  !cur_pkt_tr : input_trans_s;

  sb_upstream_notify(tr : csco_trans_s) is {
    if tr is a input_trans_s (input_trans) {
      case (input_trans.tr_kind) {
        READ: {
          input_trans.tracking.incr_exp_child_cnt(1);
          cur_rd_tr = input_trans;
        };
        RD_RESP : { 
          cur_rd_tr.tracking.link_trans(input_trans);
        };
        PACKET : {
          input_trans.tracking.incr_exp_child_cnt(1);
          cur_pkt_tr = input_trans;
        };
        WRITE : {
          input_trans.tracking.set_leaf();
        };
      };
    }
    else {
      assert FALSE else
        error("received non-input transaction on scoreboard upstream transaction port");
    };
  };

  sb_downstream_notify(tr : csco_trans_s) is {
    if tr is a output_trans_s (output_trans) {
      cur_pkt_tr.tracking.link_trans(output_trans);
    }
    else {
      assert FALSE else
        error("received non-output transaction on scoreboard downstream transaction port");
    };
  };
};

unit output_mon_u {
  CREATE_TESTFLOW;

  tr_forward : out method_port of csco_trans_forward_mp is instance;
    keep bind (tr_forward, empty);

  event new_tr is @sys.top_tb.new_output_trans;

  on new_tr {
    start collect_trans(sys.top_tb.cur_output_trans);
  };

  collect_trans(tr : output_trans_s) @sys.any is {
    var mon_tr : output_trans_s;

    mon_tr = new;
    wait [10];                  // emulate transaction length.
    mon_tr.output_fields = tr.output_fields;
    tr_forward$(mon_tr);
  };
};

unit dut_u {
  CREATE_TESTFLOW;

  event new_tr is @sys.top_tb.new_input_trans;

  on new_tr {
    case (sys.top_tb.cur_input_trans.tr_kind) {
      READ : {
        start emit_response();
      };
      PACKET : {
        start emit_packet();
      };
    };
  };

  emit_response() @sys.any is {
    wait [100];                                       // emulate DUT propagation delay.
    gen sys.top_tb.cur_input_trans keeping {
      .tracking.disable_watchdog == TRUE;
      .tr_kind == RD_RESP;
    };
    emit sys.top_tb.new_input_trans;
  };

  emit_packet() @sys.any is {
    wait [100];                                       // emulate DUT propagation delay.
    gen sys.top_tb.cur_output_trans keeping {
      .tracking.disable_watchdog == TRUE;
    };
    emit sys.top_tb.new_output_trans;
  };
};

extend csco_tb_u {
  event new_input_trans;
  !cur_input_trans : input_trans_s;

  event new_output_trans;
  !cur_output_trans : output_trans_s;


  clkgen : csco_clkgen_u is instance;
    keep clkgen.config.active_passive == ACTIVE;
    keep clkgen.config.period == 2000;
    keep clkgen.config.edge_mode == POSEDGE;
    keep clkgen.sig_clock.hdl_path() == "~/testbench.clock";

  high_level_driver : high_level_driver_u is instance;
    keep high_level_driver.input_driver == read_only(input_driver);

  input_driver : input_driver_u is instance;
    keep input_driver.bfm_ptr == read_only(input_bfm);

  input_bfm : input_bfm_u is instance;

  input_mon : input_mon_u is instance;

  scoreboard : scoreboard_u is instance;

  output_mon : output_mon_u is instance;

  dut_u is instance;

  connect_ports() is also {
    do_bind(input_mon.tr_forward, input_bfm.bfm_notify);
    do_bind(input_mon.tr_forward, scoreboard.sb_upstream_notify);
    do_bind(output_mon.tr_forward, scoreboard.sb_downstream_notify);
    do_bind(output_mon.tr_forward, sys.csco_trans_env.leaf_tr_sink);
  };
}; 




extend high_level_seq_kind_t : [TEST];

extend TEST high_level_seq {
  !item : high_level_trans_s;

  body() @driver.clock is {
    do item;

    wait [5000];

    do item;

    wait [5000];
  };
};


extend MAIN_TEST'kind csco_tb_seq {
  !high_level_test : TEST'kind high_level_seq;

  body() @driver.clock is also {
    gen high_level_test keeping {
      .driver == sys.top_tb.high_level_driver;
    };
    high_level_test.start_sequence();
  };
};


'>
