<'
package csco_verbosity;


extend any_unit {
  package dut_error_dump_priv() is {
    outf("\n");
    out("** Below is the error dump for ", e_path(), " **");
    dut_error_dump();
  };

  // The user should extend this method to print out any information
  // that might be useful in debugging a dut error.
  dut_error_dump() is empty;

  dut_error_dump_all() is {
    if session.num_of_dut_errors == 1 { //dump only for first error encountered
      // Figure out if the dut_error_dump() method for the current
      // unit instance has been extended to be more than just an 
      // empty method. If so, call it to print out error dump.
      var rf_s := rf_manager.get_exact_subtype_of_instance(me);
      if rf_s.get_method("dut_error_dump").get_relevant_layers(rf_s).size() > 1 {
        dut_error_dump_priv();
      };

      var cunits := csco_get_child_units();
      for each (c) in cunits {
        c.dut_error_dump_all();
      };
    };
  };
};


extend csco_verbosity_utils_u {
  auto_dut_error_dump_level : auto_dut_error_dump_level_t;
    keep soft auto_dut_error_dump_level == ENV;
};


extend dut_error_struct {
  // Automatically invoke system-wide error dump when a dut_error is encountered.
  write() is first {
    if check_effect() not in [WARNING, IGNORE] {
      case (sys.csco_verbosity_utils.auto_dut_error_dump_level) {
        GLOBAL : {
          try {sys.dut_error_dump_all();};
        };

        ENV : {
          var parent_env : any_env = source_struct().try_enclosing_unit(any_env);
          if parent_env != NULL {
            try {parent_env.dut_error_dump_all();};
          }
          else {
            // Do local unit dump if an env cannot be found.
            var parent_u := source_struct().get_unit();
            try {parent_u.dut_error_dump_all();};
          };
        };

        LOCAL : {
          var parent_u := source_struct().get_unit();
          parent_u.dut_error_dump_all();
        };
      };
    };
  };
};


'>
