--------------------------------------------------------------------------------
--
--   Filename : loggers.e
--   Language : 'e'
--   Author   : Gary Singer
--
--   General Description: Control Message Verbosity
--
--   History:
--   8/29/05  GSS  Initial Coding
--
--------------------------------------------------------------------------------
<'

import csco_verbosity_top;

extend message_tag: [TEST1, TEST2, TEST3]; // Add some dummy message tags, just to show in available_loggers.txt

unit unit1
{
  logger : message_logger is instance;
  short_name() : string is
  {
    result = "unit1";
  };

  pr() is
  {
    messagef (NONE, "Message at verbosity NONE");
    messagef (MEDIUM, "Message at verbosity MEDIUM");
    messagef (FULL, "Message at verbosity FULL");
  };
};

unit unit2
{
  logger : message_logger is instance;
  short_name() : string is
  {
    result = "unit2";
  };
  pr() is
  {
    messagef (NONE, "Message at verbosity NONE");
    messagef (MEDIUM, "Message at verbosity MEDIUM");
    messagef (FULL, "Message at verbosity FULL");
  };
};

extend sys
{
  unit1 : unit1 is instance;
  unit2 : unit2 is instance;   

  pr() is
  {
    messagef (NONE, "Message at verbosity NONE");
    messagef (MEDIUM, "Message at verbosity MEDIUM");
    messagef (FULL, "Message at verbosity FULL");
  };

  post_generate() is first
  {
    // Add a tag to sys for csco_verbosity to see the csco_verbosity debug messages (set_default MEDIUM in verbose.txt will change this tag)
    specman ("set message -logger=sys.logger -add -verbosity=HIGH -tags={CSCO_VERBOSITY}");

    // Add dummy tags to loggers (no messages use them in this test - just for display in available_loggers.txt)
    specman("set message -add HIGH -tags={TEST1}");
    specman("set message  -add FULL -tags={TEST2}");
    specman("set message  -logger=sys.unit1.logger -add FULL -tags={TEST3}");
  };
  
  run() is also
  {
    pr();
    unit1.pr();
    unit2.pr();
    start check_messages();
  };

  check_messages() @sys.any is
  {
    while (to_specman_scale (sys.time, ns) < 150)
    {
      wait delay(to_specman_scale (5, ns));
      pr();
      unit1.pr();
      unit2.pr();
    };
  };

  check() is only
  {
    print sys.time;
  };
};



'>
